Package: assessor
Title: Assessment Tools for Regression Models with Discrete and
        Semicontinuous Outcomes
Version: 1.3.1
Authors@R: c(
    person("Lu", "Yang", email="luyang@umn.edu", role = "aut"),
    person("Jeonghwan", "Lee", email="lee03938@umn.edu", role=c("cre", "aut")))
Description: Provides assessment tools for regression models with discrete and semicontinuous outcomes proposed in Yang (2021) <doi:10.1080/10618600.2021.1910042>, Yang (2024) <doi:10.1080/10618600.2024.2303336>, Yang (2024) <doi:10.1093/biomtc/ujae007>, and Yang (2026) <doi:10.1002/cjs.70046>. It calculates the double probability integral transform (DPIT) residuals. It also constructs QQ plots of residuals the ordered curve for assessing mean structures, quasi-empirical distribution function for overall assessment, and a formal goodness-of-fit test.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://jhlee1408.github.io/assessor/
BugReports: https://github.com/jhlee1408/assessor/issues
Imports: tweedie, MASS, VGAM, np, pscl
Suggests: statmod, rmarkdown, knitr, AER, testthat (>= 3.0.0), mgcv
Config/testthat/edition: 3
Depends: R (>= 3.5)
LazyData: true
NeedsCompilation: no
Packaged: 2026-04-20 22:10:17 UTC; jeonghwanlee
Author: Lu Yang [aut],
  Jeonghwan Lee [cre, aut]
Maintainer: Jeonghwan Lee <lee03938@umn.edu>
Repository: CRAN
Date/Publication: 2026-04-20 22:30:02 UTC
Built: R 4.5.2; ; 2026-04-20 23:30:37 UTC; unix

Package: assessor
Title: Assessment Tools for Regression Models with Discrete and
        Semicontinuous Outcomes
Version: 1.3.2
Authors@R: c(
    person("Lu", "Yang", email="luyang@umn.edu", role = "aut"),
    person("Jeonghwan", "Lee", email="lee03938@umn.edu", role=c("cre", "aut")))
Description: Provides assessment tools for regression models with discrete and
    semicontinuous outcomes. The implemented methods are described in
    Yang (2021) <doi:10.1080/10618600.2021.1910042>,
    Yang (2024) <doi:10.1080/10618600.2024.2303336>,
    Yang (2024) <doi:10.1093/biomtc/ujae007>, and
    Yang (2026) <doi:10.1002/cjs.70046>. It calculates double probability integral transform (DPIT) residuals and constructs QQ plots, ordered curves, quasi-empirical residual distribution functions, and formal goodness-of-fit tests.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://jhlee1408.github.io/assessor/
BugReports: https://github.com/jhlee1408/assessor/issues
Imports: tweedie, MASS, VGAM, np, pscl
Suggests: statmod, rmarkdown, knitr, AER, testthat (>= 3.0.0), mgcv
Config/testthat/edition: 3
Depends: R (>= 3.5)
LazyData: true
NeedsCompilation: no
Packaged: 2026-08-22 03:40:25 UTC; jeonghwanlee
Author: Lu Yang [aut],
  Jeonghwan Lee [cre, aut]
Maintainer: Jeonghwan Lee <lee03938@umn.edu>
Repository: CRAN
Date/Publication: 2026-08-22 04:00:02 UTC
Built: R 4.5.2; ; 2026-08-22 04:56:36 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(.dpit_glm_key,Tweedie)
S3method(.dpit_glm_key,binomial)
S3method(.dpit_glm_key,default)
S3method(.dpit_glm_key,negbin)
S3method(.dpit_glm_key,poisson)
S3method(.dpit_zeroinfl_key,zeroinfl_default)
S3method(.dpit_zeroinfl_key,zeroinfl_negbin)
S3method(.dpit_zeroinfl_key,zeroinfl_poisson)
S3method(.gof_glm_key, poisson)
S3method(.gof_glm_key,binomial)
S3method(.gof_glm_key,default)
S3method(.gof_glm_key,negbin)
S3method(.gof_zeroinfl_key,default)
S3method(.gof_zeroinfl_key,negbin)
S3method(.gof_zeroinfl_key,poisson)
S3method(.quasi_plot_glm_key,binomial)
S3method(.quasi_plot_glm_key,default)
S3method(.quasi_plot_glm_key,nb)
S3method(.quasi_plot_glm_key,poisson)
S3method(.quasi_plot_zeroinfl_key,default)
S3method(.quasi_plot_zeroinfl_key,negbin)
S3method(.quasi_plot_zeroinfl_key,poisson)
S3method(dpit,default)
S3method(dpit,glm)
S3method(dpit,polr)
S3method(dpit,tobit)
S3method(dpit,vglm)
S3method(dpit,zeroinfl)
S3method(gof_disc,default)
S3method(gof_disc,glm)
S3method(gof_disc,polr)
S3method(gof_disc,zeroinfl)
S3method(ord_curve,default)
S3method(ord_curve,glm)
S3method(ord_curve,lm)
S3method(ord_curve,polr)
S3method(plot,dpit)
S3method(print,dpit)
S3method(print,summary.dpit)
S3method(quasi_plot,default)
S3method(quasi_plot,glm)
S3method(quasi_plot,polr)
S3method(quasi_plot,zeroinfl)
S3method(residuals,dpit)
S3method(summary,dpit)
export(dpit)
export(dpit_2pm)
export(dpit_bin)
export(dpit_nb)
export(dpit_ordi)
export(dpit_pois)
export(dpit_tobit)
export(dpit_tweedie)
export(dpit_znb)
export(dpit_zpois)
export(gof_disc)
export(ord_curve)
export(quasi_plot)
import(graphics)
import(np)
import(stats)
import(tweedie)
importFrom(MASS,gamma.dispersion)
importFrom(MASS,glm.nb)
importFrom(MASS,polr)
importFrom(MASS,rnegbin)
importFrom(graphics,abline)
importFrom(pscl,zeroinfl)
importFrom(stats,ecdf)
importFrom(stats,family)
importFrom(utils,combn)
importFrom(utils,modifyList)

django_assessor
===============

Simple Assessment (Quiz) App with Django, Tastypie and ExtJS

LICENSE: AGPLv3

NOTE: Currently setup to use questions from practice exams for the CA BBS MFT 
certification exams. These are copyrighted and not for public use.

To use, you must create questions, choices and explanations manually. 

TODO:
- Create UI for question/choice/explanation creation
- Create UI for answer history
- Create UI for user authentication and authorization
- Create standalone install for Android
- Optimize AJAX calls, including LoadMask indicator

#' @keywords internal
gof_bin <- function(B, bimodel = NULL, seed=NULL) {
  if (!is.null(seed)) set.seed(seed)
  if( is.null(bimodel)) stop("model object is not given.")
  y <- bimodel$model[, 1]
  n <- length(y)
  p1f <- bimodel$fitted.values
  dfx <- model.matrix(bimodel)
  fam <- bimodel$family
  como <- combn(n,2)
  ind1 <- como[1,]
  ind2 <- como[2,]


  ## F(Y), F(Y-1), P(Y)

  Fy <- ifelse(y==1,1,1-p1f)
  Fy1 <-  ifelse(y==0,0,1-p1f)
  dy <- ifelse(y==1,p1f,1-p1f)

  ## The test statistic is calculated by parts

  int1new <- sum(1/3*(Fy^4+Fy1^4-Fy^3*Fy1-Fy*Fy1^3-2*Fy^3-Fy1^3+3*Fy^2*Fy1+Fy^2-2*Fy*Fy1+Fy1^2)/dy^2)


  Fy1new <- sort(Fy1)
  Fynew <- Fy[sort(Fy1,index=TRUE)$ix]
  dynew <- dy[sort(Fy1,index=TRUE)$ix]




  ###no overlap
  group1 <- which(Fy1new[ind2]>Fynew[ind1])
  ## overlap, and keep order. k=ind1,k'=ind2,m=ind2,m'=ind1
  group2 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]>Fynew[ind1])
  ## overlap, and switch order for on right side. k=ind1,k'=ind2,m=ind1,m'=ind2
  group3 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]<=Fynew[ind1])
  group4 <- which(Fy1new[ind2]<=Fynew[ind1])

  ind2new1 <- sum(((Fynew^3-Fy1new^3)/6/dynew)[ind1[group1]])

  ind2new2 <- sum(((Fynew^3-Fy1new^3-3*Fynew^2+3*Fy1new^2+2*Fynew-2*Fy1new)/dynew/6)[ind2[group1]])


  indnew3p1 <- sum((-Fy1new[ind1[group4]]^3-2*Fy1new[ind2[group4]]^3*(1-dynew[ind1[group4]])+3*Fy1new[ind1[group4]]*Fy1new[ind2[group4]]^2)/6/dynew[ind1[group4]])



  indnew3p2 <- sum((1-dynew[ind1[group2]])*(1-dynew[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^3-Fy1new[ind2[group2]]^3)/3) +
    sum((1-dynew[ind1[group3]])*(1-dynew[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^3-Fy1new[ind2[group3]]^3)/3) -
    sum(((1-dynew[ind1[group2]])*(Fy1new[ind2[group2]])+(1-dynew[ind2[group2]])*(Fy1new[ind1[group2]]))/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^2-Fy1new[ind2[group2]]^2)/2) -
    sum(((1-dynew[ind1[group3]])*(Fy1new[ind2[group3]])+(1-dynew[ind2[group3]])*(Fy1new[ind1[group3]]))/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^2-Fy1new[ind2[group3]]^2)/2) +
    sum((Fy1new[ind1[group2]])*(Fy1new[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]-Fy1new[ind2[group2]])) +
    sum((Fy1new[ind1[group3]])*(Fy1new[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]-Fy1new[ind2[group3]]))

  indnew3p3 <- sum(1/6/dynew[ind2[group2]]*(Fynew[ind2[group2]]^3+2*Fynew[ind1[group2]]^3-2*Fynew[ind1[group2]]^3*Fynew[ind2[group2]]+
                                              2*Fynew[ind1[group2]]^3*Fy1new[ind2[group2]]-
                                              3*Fynew[ind1[group2]]^2-3*Fynew[ind2[group2]]^2+3*Fynew[ind1[group2]]^2*Fynew[ind2[group2]]-
                                              6*Fynew[ind1[group2]]^2*Fy1new[ind2[group2]]+6*Fynew[ind1[group2]]*Fy1new[ind2[group2]]+
                                              2*Fynew[ind2[group2]]-2*Fy1new[ind2[group2]]))+
    sum(1/6/dynew[ind1[group3]]*(Fynew[ind1[group3]]^3+2*Fynew[ind2[group3]]^3-2*Fynew[ind2[group3]]^3*Fynew[ind1[group3]]+
                                   2*Fynew[ind2[group3]]^3*Fy1new[ind1[group3]]-
                                   3*Fynew[ind2[group3]]^2-3*Fynew[ind1[group3]]^2+3*Fynew[ind2[group3]]^2*Fynew[ind1[group3]]-
                                   6*Fynew[ind2[group3]]^2*Fy1new[ind1[group3]]+6*Fynew[ind2[group3]]*Fy1new[ind1[group3]]+
                                   2*Fynew[ind1[group3]]-2*Fy1new[ind1[group3]]))



  ## Test statistic
  disnull <- int1new+2*(ind2new1+ind2new2+indnew3p1+indnew3p2+indnew3p3)

  ## Bootstrap
  disin <- rep(0,B)

  for(repin in 1:B){

    yr <- rbinom(n,size=1,p1f)
    #####!!!changes made
    #####!!!
    dfr <- data.frame(yr=yr,dfx[,-1])

    #fit marginal model
    bimodelr <- glm(yr~.,family=fam,data=dfr)
    p1fr <- bimodelr$fitted.values
    #####!!!
    #####!!!




    Fyr <- ifelse(yr==1,1,1-p1fr)
    Fy1r <-  ifelse(yr==0,0,1-p1fr)
    dyr <- ifelse(yr==1,p1fr,1-p1fr)

    int1newr <- sum(1/3*(Fyr^4+Fy1r^4-Fyr^3*Fy1r-Fyr*Fy1r^3-2*Fyr^3-Fy1r^3+3*Fyr^2*Fy1r+Fyr^2-2*Fyr*Fy1r+Fy1r^2)/dyr^2)



    Fy1new <- sort(Fy1r)
    Fynew <- Fyr[sort(Fy1r,index=TRUE)$ix]
    dynew <- dyr[sort(Fy1r,index=TRUE)$ix]



    group1 <- which(Fy1new[ind2]>Fynew[ind1])
    ## overlap, and keep order. k=ind1,k'=ind2,m=ind2,m'=ind1
    group2 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]>Fynew[ind1])
    ## overlap, and switch order for on right side. k=ind1,k'=ind2,m=ind1,m'=ind2
    group3 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]<=Fynew[ind1])
    group4 <- which(Fy1new[ind2]<=Fynew[ind1])

    ind2new1r <- sum(((Fynew^3-Fy1new^3)/6/dynew)[ind1[group1]])

    ind2new2r <- sum(((Fynew^3-Fy1new^3-3*Fynew^2+3*Fy1new^2+2*Fynew-2*Fy1new)/dynew/6)[ind2[group1]])


    indnew3p1r <- sum((-Fy1new[ind1[group4]]^3-2*Fy1new[ind2[group4]]^3*(1-dynew[ind1[group4]])+3*Fy1new[ind1[group4]]*Fy1new[ind2[group4]]^2)/6/dynew[ind1[group4]])



    indnew3p2r <- sum((1-dynew[ind1[group2]])*(1-dynew[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^3-Fy1new[ind2[group2]]^3)/3) +
      sum((1-dynew[ind1[group3]])*(1-dynew[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^3-Fy1new[ind2[group3]]^3)/3) -
      sum(((1-dynew[ind1[group2]])*(Fy1new[ind2[group2]])+(1-dynew[ind2[group2]])*(Fy1new[ind1[group2]]))/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^2-Fy1new[ind2[group2]]^2)/2) -
      sum(((1-dynew[ind1[group3]])*(Fy1new[ind2[group3]])+(1-dynew[ind2[group3]])*(Fy1new[ind1[group3]]))/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^2-Fy1new[ind2[group3]]^2)/2) +
      sum((Fy1new[ind1[group2]])*(Fy1new[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]-Fy1new[ind2[group2]])) +
      sum((Fy1new[ind1[group3]])*(Fy1new[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]-Fy1new[ind2[group3]]))

    indnew3p3r <- sum(1/6/dynew[ind2[group2]]*(Fynew[ind2[group2]]^3+2*Fynew[ind1[group2]]^3-2*Fynew[ind1[group2]]^3*Fynew[ind2[group2]]+
                                                 2*Fynew[ind1[group2]]^3*Fy1new[ind2[group2]]-
                                                 3*Fynew[ind1[group2]]^2-3*Fynew[ind2[group2]]^2+3*Fynew[ind1[group2]]^2*Fynew[ind2[group2]]-
                                                 6*Fynew[ind1[group2]]^2*Fy1new[ind2[group2]]+6*Fynew[ind1[group2]]*Fy1new[ind2[group2]]+
                                                 2*Fynew[ind2[group2]]-2*Fy1new[ind2[group2]]))+
      sum(1/6/dynew[ind1[group3]]*(Fynew[ind1[group3]]^3+2*Fynew[ind2[group3]]^3-2*Fynew[ind2[group3]]^3*Fynew[ind1[group3]]+
                                     2*Fynew[ind2[group3]]^3*Fy1new[ind1[group3]]-
                                     3*Fynew[ind2[group3]]^2-3*Fynew[ind1[group3]]^2+3*Fynew[ind2[group3]]^2*Fynew[ind1[group3]]-
                                     6*Fynew[ind2[group3]]^2*Fy1new[ind1[group3]]+6*Fynew[ind2[group3]]*Fy1new[ind1[group3]]+
                                     2*Fynew[ind1[group3]]-2*Fy1new[ind1[group3]]))



    disin[repin] <-int1newr+2*(ind2new1r+ind2new2r+indnew3p1r+indnew3p2r+indnew3p3r)

  }
  pvalue <- length(which(disin>disnull))/B
  return(list(test_stat = disnull, p_value = pvalue))
}

#' @importFrom utils combn
#' @importFrom MASS rnegbin
#' @importFrom MASS glm.nb
#' @keywords internal
gof_nb <- function(B,nbmodel=NULL, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  if(is.null(nbmodel)) stop("model object is not given.")
  y <- nbmodel$model[, 1]
  n <- length(y)
  lambda1f <- nbmodel$fitted.values
  size1f <- summary(nbmodel)$theta
  dfx <- model.matrix(nbmodel)
  lin <- nbmodel$family$link

  como <- combn(n,2)
  ind1 <- como[1,]
  ind2 <- como[2,]

  Fy <- pnbinom(y,mu=lambda1f,size=size1f)
  Fy1 <-  pnbinom(y-1,mu=lambda1f,size=size1f)
  dy <-  dnbinom(y,mu=lambda1f,size=size1f)

  int1new <- sum(1/3*(Fy^4+Fy1^4-Fy^3*Fy1-Fy*Fy1^3-2*Fy^3-Fy1^3+3*Fy^2*Fy1+Fy^2-2*Fy*Fy1+Fy1^2)/dy^2)


  Fy1new <- sort(Fy1)
  Fynew <- Fy[sort(Fy1,index=TRUE)$ix]
  dynew <- dy[sort(Fy1,index=TRUE)$ix]



  ###no overlap
  group1 <- which(Fy1new[ind2]>Fynew[ind1])
  ## overlap, and keep order. k=ind1,k'=ind2,m=ind2,m'=ind1
  group2 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]>Fynew[ind1])
  ## overlap, and switch order for on right side. k=ind1,k'=ind2,m=ind1,m'=ind2
  group3 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]<=Fynew[ind1])
  group4 <- which(Fy1new[ind2]<=Fynew[ind1])

  ind2new1 <- sum(((Fynew^3-Fy1new^3)/6/dynew)[ind1[group1]])

  ind2new2 <- sum(((Fynew^3-Fy1new^3-3*Fynew^2+3*Fy1new^2+2*Fynew-2*Fy1new)/dynew/6)[ind2[group1]])


  indnew3p1 <- sum((-Fy1new[ind1[group4]]^3-2*Fy1new[ind2[group4]]^3*(1-dynew[ind1[group4]])+3*Fy1new[ind1[group4]]*Fy1new[ind2[group4]]^2)/6/dynew[ind1[group4]])



  indnew3p2 <- sum((1-dynew[ind1[group2]])*(1-dynew[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^3-Fy1new[ind2[group2]]^3)/3) +
    sum((1-dynew[ind1[group3]])*(1-dynew[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^3-Fy1new[ind2[group3]]^3)/3) -
    sum(((1-dynew[ind1[group2]])*(Fy1new[ind2[group2]])+(1-dynew[ind2[group2]])*(Fy1new[ind1[group2]]))/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^2-Fy1new[ind2[group2]]^2)/2) -
    sum(((1-dynew[ind1[group3]])*(Fy1new[ind2[group3]])+(1-dynew[ind2[group3]])*(Fy1new[ind1[group3]]))/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^2-Fy1new[ind2[group3]]^2)/2) +
    sum((Fy1new[ind1[group2]])*(Fy1new[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]-Fy1new[ind2[group2]])) +
    sum((Fy1new[ind1[group3]])*(Fy1new[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]-Fy1new[ind2[group3]]))

  indnew3p3 <- sum(1/6/dynew[ind2[group2]]*(Fynew[ind2[group2]]^3+2*Fynew[ind1[group2]]^3-2*Fynew[ind1[group2]]^3*Fynew[ind2[group2]]+
                                              2*Fynew[ind1[group2]]^3*Fy1new[ind2[group2]]-
                                              3*Fynew[ind1[group2]]^2-3*Fynew[ind2[group2]]^2+3*Fynew[ind1[group2]]^2*Fynew[ind2[group2]]-
                                              6*Fynew[ind1[group2]]^2*Fy1new[ind2[group2]]+6*Fynew[ind1[group2]]*Fy1new[ind2[group2]]+
                                              2*Fynew[ind2[group2]]-2*Fy1new[ind2[group2]]))+
    sum(1/6/dynew[ind1[group3]]*(Fynew[ind1[group3]]^3+2*Fynew[ind2[group3]]^3-2*Fynew[ind2[group3]]^3*Fynew[ind1[group3]]+
                                   2*Fynew[ind2[group3]]^3*Fy1new[ind1[group3]]-
                                   3*Fynew[ind2[group3]]^2-3*Fynew[ind1[group3]]^2+3*Fynew[ind2[group3]]^2*Fynew[ind1[group3]]-
                                   6*Fynew[ind2[group3]]^2*Fy1new[ind1[group3]]+6*Fynew[ind2[group3]]*Fy1new[ind1[group3]]+
                                   2*Fynew[ind1[group3]]-2*Fy1new[ind1[group3]]))


  ## Test statistic
  disnull <- int1new+2*(ind2new1+ind2new2+indnew3p1+indnew3p2+indnew3p3)


  ## Bootstrap
  disin <- rep(0,B)
  for(repin in 1:B){
    yr <- rnegbin(n, mu=lambda1f, theta=size1f)
    dfr <- data.frame(yr=yr,dfx[,-1])
    if (lin == "log") {
      model1r <- MASS::glm.nb(yr ~ ., data = dfr, link = log)
    } else if (lin == "identity") {
      model1r <- MASS::glm.nb(yr ~ ., data = dfr, link = identity)
    } else if (lin == "sqrt") {
      model1r <- MASS::glm.nb(yr ~ ., data = dfr, link = sqrt)
    } else {
      stop("Unsupported nb link: ", lin)
    }
    lambda1fr <- model1r$fitted.values
    size1fr <- summary(model1r)$theta

    Fyr <- pnbinom(yr,mu=lambda1fr,size=size1fr)
    Fy1r <-  pnbinom(yr-1,mu=lambda1fr,size=size1fr)
    dyr <-  dnbinom(yr,mu=lambda1fr,size=size1fr)

    int1newr <- sum(1/3*(Fyr^4+Fy1r^4-Fyr^3*Fy1r-Fyr*Fy1r^3-2*Fyr^3-Fy1r^3+3*Fyr^2*Fy1r+Fyr^2-2*Fyr*Fy1r+Fy1r^2)/dyr^2)



    Fy1new <- sort(Fy1r)
    Fynew <- Fyr[sort(Fy1r,index=TRUE)$ix]
    dynew <- dyr[sort(Fy1r,index=TRUE)$ix]



    ###no overlap
    group1 <- which(Fy1new[ind2]>Fynew[ind1])
    ## overlap, and keep order. k=ind1,k'=ind2,m=ind2,m'=ind1
    group2 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]>Fynew[ind1])
    ## overlap, and switch order for on right side. k=ind1,k'=ind2,m=ind1,m'=ind2
    group3 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]<=Fynew[ind1])
    group4 <- which(Fy1new[ind2]<=Fynew[ind1])

    ind2new1r <- sum(((Fynew^3-Fy1new^3)/6/dynew)[ind1[group1]])

    ind2new2r <- sum(((Fynew^3-Fy1new^3-3*Fynew^2+3*Fy1new^2+2*Fynew-2*Fy1new)/dynew/6)[ind2[group1]])


    indnew3p1r <- sum((-Fy1new[ind1[group4]]^3-2*Fy1new[ind2[group4]]^3*(1-dynew[ind1[group4]])+3*Fy1new[ind1[group4]]*Fy1new[ind2[group4]]^2)/6/dynew[ind1[group4]])



    indnew3p2r <- sum((1-dynew[ind1[group2]])*(1-dynew[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^3-Fy1new[ind2[group2]]^3)/3) +
      sum((1-dynew[ind1[group3]])*(1-dynew[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^3-Fy1new[ind2[group3]]^3)/3) -
      sum(((1-dynew[ind1[group2]])*(Fy1new[ind2[group2]])+(1-dynew[ind2[group2]])*(Fy1new[ind1[group2]]))/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^2-Fy1new[ind2[group2]]^2)/2) -
      sum(((1-dynew[ind1[group3]])*(Fy1new[ind2[group3]])+(1-dynew[ind2[group3]])*(Fy1new[ind1[group3]]))/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^2-Fy1new[ind2[group3]]^2)/2) +
      sum((Fy1new[ind1[group2]])*(Fy1new[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]-Fy1new[ind2[group2]])) +
      sum((Fy1new[ind1[group3]])*(Fy1new[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]-Fy1new[ind2[group3]]))

    indnew3p3r <- sum(1/6/dynew[ind2[group2]]*(Fynew[ind2[group2]]^3+2*Fynew[ind1[group2]]^3-2*Fynew[ind1[group2]]^3*Fynew[ind2[group2]]+
                                                 2*Fynew[ind1[group2]]^3*Fy1new[ind2[group2]]-
                                                 3*Fynew[ind1[group2]]^2-3*Fynew[ind2[group2]]^2+3*Fynew[ind1[group2]]^2*Fynew[ind2[group2]]-
                                                 6*Fynew[ind1[group2]]^2*Fy1new[ind2[group2]]+6*Fynew[ind1[group2]]*Fy1new[ind2[group2]]+
                                                 2*Fynew[ind2[group2]]-2*Fy1new[ind2[group2]]))+
      sum(1/6/dynew[ind1[group3]]*(Fynew[ind1[group3]]^3+2*Fynew[ind2[group3]]^3-2*Fynew[ind2[group3]]^3*Fynew[ind1[group3]]+
                                     2*Fynew[ind2[group3]]^3*Fy1new[ind1[group3]]-
                                     3*Fynew[ind2[group3]]^2-3*Fynew[ind1[group3]]^2+3*Fynew[ind2[group3]]^2*Fynew[ind1[group3]]-
                                     6*Fynew[ind2[group3]]^2*Fy1new[ind1[group3]]+6*Fynew[ind2[group3]]*Fy1new[ind1[group3]]+
                                     2*Fynew[ind1[group3]]-2*Fy1new[ind1[group3]]))

    disin[repin] <- int1newr+2*(ind2new1r+ind2new2r+indnew3p1r+indnew3p2r+indnew3p3r)

  }

  ##p-value
  pvalue <- length(which(disin>disnull))/B
  return(list(test_stat = disnull, p_value = pvalue))
}

#' @keywords internal
#' @importFrom MASS polr
gof_ordi <- function(B, multimodel=NULL, seed=NULL){
  genemult <- function(p){
    rmultinom(1,size=1,prob=p)
  }
  if (!is.null(seed)) set.seed(seed)
  if( is.null(multimodel )) stop("model object is not given.")

  y <- multimodel$model[, 1]
  n <- length(y)
  fitprob <- multimodel$fitted.values
  k <- length(multimodel$lev)
  ### Convert the levels to 1:k
  out <- as.numeric(factor(multimodel$model[,1],ordered = TRUE))
  dfx <- model.matrix(multimodel)
  met <- multimodel$method

  ### Cumulative probabilities
  q <- t(apply(fitprob,1, cumsum))
  ## F(Y), F(Y-1), P(Y)
  Fy <- q[cbind(1:n,out)]
  out1 <- ifelse(out==1,1,out-1)
  Fy1 <-  ifelse(out==1,0,q[cbind(1:n,out1)])
  dy <-  Fy-Fy1

  como <- combn(n,2)
  ind1 <- como[1,]
  ind2 <- como[2,]

  ## The test statistic is calculated by parts
  int1new <- sum(1/3*(Fy^4+Fy1^4-Fy^3*Fy1-Fy*Fy1^3-2*Fy^3-Fy1^3+3*Fy^2*Fy1+Fy^2-2*Fy*Fy1+Fy1^2)/dy^2)
  Fy1new <- sort(Fy1)
  Fynew <- Fy[sort(Fy1,index=TRUE)$ix]
  dynew <- dy[sort(Fy1,index=TRUE)$ix]

  ###no overlap
  group1 <- which(Fy1new[ind2]>Fynew[ind1])
  ## overlap, and keep order. k=ind1,k'=ind2,m=ind2,m'=ind1
  group2 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]>Fynew[ind1])
  ## overlap, and switch order for on right side. k=ind1,k'=ind2,m=ind1,m'=ind2
  group3 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]<=Fynew[ind1])
  group4 <- which(Fy1new[ind2]<=Fynew[ind1])

  ind2new1 <- sum(((Fynew^3-Fy1new^3)/6/dynew)[ind1[group1]])

  ind2new2 <- sum(((Fynew^3-Fy1new^3-3*Fynew^2+3*Fy1new^2+2*Fynew-2*Fy1new)/dynew/6)[ind2[group1]])


  indnew3p1 <- sum((-Fy1new[ind1[group4]]^3-2*Fy1new[ind2[group4]]^3*(1-dynew[ind1[group4]])+3*Fy1new[ind1[group4]]*Fy1new[ind2[group4]]^2)/6/dynew[ind1[group4]])



  indnew3p2 <- sum((1-dynew[ind1[group2]])*(1-dynew[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^3-Fy1new[ind2[group2]]^3)/3) +
    sum((1-dynew[ind1[group3]])*(1-dynew[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^3-Fy1new[ind2[group3]]^3)/3) -
    sum(((1-dynew[ind1[group2]])*(Fy1new[ind2[group2]])+(1-dynew[ind2[group2]])*(Fy1new[ind1[group2]]))/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^2-Fy1new[ind2[group2]]^2)/2) -
    sum(((1-dynew[ind1[group3]])*(Fy1new[ind2[group3]])+(1-dynew[ind2[group3]])*(Fy1new[ind1[group3]]))/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^2-Fy1new[ind2[group3]]^2)/2) +
    sum((Fy1new[ind1[group2]])*(Fy1new[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]-Fy1new[ind2[group2]])) +
    sum((Fy1new[ind1[group3]])*(Fy1new[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]-Fy1new[ind2[group3]]))

  indnew3p3 <- sum(1/6/dynew[ind2[group2]]*(Fynew[ind2[group2]]^3+2*Fynew[ind1[group2]]^3-2*Fynew[ind1[group2]]^3*Fynew[ind2[group2]]+
                                              2*Fynew[ind1[group2]]^3*Fy1new[ind2[group2]]-
                                              3*Fynew[ind1[group2]]^2-3*Fynew[ind2[group2]]^2+3*Fynew[ind1[group2]]^2*Fynew[ind2[group2]]-
                                              6*Fynew[ind1[group2]]^2*Fy1new[ind2[group2]]+6*Fynew[ind1[group2]]*Fy1new[ind2[group2]]+
                                              2*Fynew[ind2[group2]]-2*Fy1new[ind2[group2]]))+
    sum(1/6/dynew[ind1[group3]]*(Fynew[ind1[group3]]^3+2*Fynew[ind2[group3]]^3-2*Fynew[ind2[group3]]^3*Fynew[ind1[group3]]+
                                   2*Fynew[ind2[group3]]^3*Fy1new[ind1[group3]]-
                                   3*Fynew[ind2[group3]]^2-3*Fynew[ind1[group3]]^2+3*Fynew[ind2[group3]]^2*Fynew[ind1[group3]]-
                                   6*Fynew[ind2[group3]]^2*Fy1new[ind1[group3]]+6*Fynew[ind2[group3]]*Fy1new[ind1[group3]]+
                                   2*Fynew[ind1[group3]]-2*Fy1new[ind1[group3]]))



  ## Test statistic
  disnull <- int1new+2*(ind2new1+ind2new2+indnew3p1+indnew3p2+indnew3p3)

  ## Bootstrap
  disin <- rep(0,B)

  for(repin in 1:B){
    testr <- apply(fitprob,1,genemult)
    yr <- apply(testr,2,which.max)
    dfr <- data.frame(yr=yr,dfx[,-1])
    multimodelr <- polr(as.factor(yr)~.,method=met,data=dfr)
    fitprobr <- multimodelr$fitted.values
    outr <- as.numeric(factor(multimodelr$model[,1],ordered = TRUE))

    ### Cumulative probabilities
    qr <- t(apply(fitprobr,1, cumsum))



    ## F(Y), F(Y-1), P(Y)
    Fyr <- qr[cbind(1:n,outr)]
    out1r <- ifelse(outr==1,1,outr-1)
    Fy1r <-  ifelse(outr==1,0,qr[cbind(1:n,out1r)])
    dyr <-  Fyr-Fy1r


    int1newr <- sum(1/3*(Fyr^4+Fy1r^4-Fyr^3*Fy1r-Fyr*Fy1r^3-2*Fyr^3-Fy1r^3+3*Fyr^2*Fy1r+Fyr^2-2*Fyr*Fy1r+Fy1r^2)/dyr^2)



    Fy1new <- sort(Fy1r)
    Fynew <- Fyr[sort(Fy1r,index=TRUE)$ix]
    dynew <- dyr[sort(Fy1r,index=TRUE)$ix]



    group1 <- which(Fy1new[ind2]>Fynew[ind1])
    ## overlap, and keep order. k=ind1,k'=ind2,m=ind2,m'=ind1
    group2 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]>Fynew[ind1])
    ## overlap, and switch order for on right side. k=ind1,k'=ind2,m=ind1,m'=ind2
    group3 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]<=Fynew[ind1])
    group4 <- which(Fy1new[ind2]<=Fynew[ind1])

    ind2new1r <- sum(((Fynew^3-Fy1new^3)/6/dynew)[ind1[group1]])

    ind2new2r <- sum(((Fynew^3-Fy1new^3-3*Fynew^2+3*Fy1new^2+2*Fynew-2*Fy1new)/dynew/6)[ind2[group1]])


    indnew3p1r <- sum((-Fy1new[ind1[group4]]^3-2*Fy1new[ind2[group4]]^3*(1-dynew[ind1[group4]])+3*Fy1new[ind1[group4]]*Fy1new[ind2[group4]]^2)/6/dynew[ind1[group4]])



    indnew3p2r <- sum((1-dynew[ind1[group2]])*(1-dynew[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^3-Fy1new[ind2[group2]]^3)/3) +
      sum((1-dynew[ind1[group3]])*(1-dynew[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^3-Fy1new[ind2[group3]]^3)/3) -
      sum(((1-dynew[ind1[group2]])*(Fy1new[ind2[group2]])+(1-dynew[ind2[group2]])*(Fy1new[ind1[group2]]))/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^2-Fy1new[ind2[group2]]^2)/2) -
      sum(((1-dynew[ind1[group3]])*(Fy1new[ind2[group3]])+(1-dynew[ind2[group3]])*(Fy1new[ind1[group3]]))/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^2-Fy1new[ind2[group3]]^2)/2) +
      sum((Fy1new[ind1[group2]])*(Fy1new[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]-Fy1new[ind2[group2]])) +
      sum((Fy1new[ind1[group3]])*(Fy1new[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]-Fy1new[ind2[group3]]))

    indnew3p3r <- sum(1/6/dynew[ind2[group2]]*(Fynew[ind2[group2]]^3+2*Fynew[ind1[group2]]^3-2*Fynew[ind1[group2]]^3*Fynew[ind2[group2]]+
                                                 2*Fynew[ind1[group2]]^3*Fy1new[ind2[group2]]-
                                                 3*Fynew[ind1[group2]]^2-3*Fynew[ind2[group2]]^2+3*Fynew[ind1[group2]]^2*Fynew[ind2[group2]]-
                                                 6*Fynew[ind1[group2]]^2*Fy1new[ind2[group2]]+6*Fynew[ind1[group2]]*Fy1new[ind2[group2]]+
                                                 2*Fynew[ind2[group2]]-2*Fy1new[ind2[group2]]))+
      sum(1/6/dynew[ind1[group3]]*(Fynew[ind1[group3]]^3+2*Fynew[ind2[group3]]^3-2*Fynew[ind2[group3]]^3*Fynew[ind1[group3]]+
                                     2*Fynew[ind2[group3]]^3*Fy1new[ind1[group3]]-
                                     3*Fynew[ind2[group3]]^2-3*Fynew[ind1[group3]]^2+3*Fynew[ind2[group3]]^2*Fynew[ind1[group3]]-
                                     6*Fynew[ind2[group3]]^2*Fy1new[ind1[group3]]+6*Fynew[ind2[group3]]*Fy1new[ind1[group3]]+
                                     2*Fynew[ind1[group3]]-2*Fy1new[ind1[group3]]))



    disin[repin] <-int1newr+2*(ind2new1r+ind2new2r+indnew3p1r+indnew3p2r+indnew3p3r)

  }

  pvalue <- length(which(disin>disnull))/B
  return(list(test_stat = disnull, p_value = pvalue))
}

#' @importFrom utils combn
#' @keywords internal
gof_pois <- function(B, poismodel = NULL, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  if(is.null(poismodel)) stop("model object is not given.")
  y <- poismodel$model[, 1]
  n <- length(y)
  lambda1f <- poismodel$fitted.values
  dfx <- model.matrix(poismodel)
  fam <- poismodel$family

  como <- combn(n,2)
  ind1 <- como[1,]
  ind2 <- como[2,]


  ## F(Y), F(Y-1), P(Y)
  Fy <- ppois(y,lambda1f)
  Fy1 <- ppois(y-1,lambda1f)
  dy <- dpois(y,lambda1f)


  ## The test statistic is calculated by parts
  int1new <- sum(1/3*(Fy^4+Fy1^4-Fy^3*Fy1-Fy*Fy1^3-2*Fy^3-Fy1^3+3*Fy^2*Fy1+Fy^2-2*Fy*Fy1+Fy1^2)/dy^2)


  Fy1new <- sort(Fy1)
  Fynew <- Fy[sort(Fy1,index=TRUE)$ix]
  dynew <- dy[sort(Fy1,index=TRUE)$ix]



  ###no overlap
  group1 <- which(Fy1new[ind2]>Fynew[ind1])
  ## overlap, and keep order. k=ind1,k'=ind2,m=ind2,m'=ind1
  group2 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]>Fynew[ind1])
  ## overlap, and switch order for on right side. k=ind1,k'=ind2,m=ind1,m'=ind2
  group3 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]<=Fynew[ind1])
  group4 <- which(Fy1new[ind2]<=Fynew[ind1])


  ind2new1 <- sum(((Fynew^3-Fy1new^3)/6/dynew)[ind1[group1]])

  ind2new2 <- sum(((Fynew^3-Fy1new^3-3*Fynew^2+3*Fy1new^2+2*Fynew-2*Fy1new)/dynew/6)[ind2[group1]])


  indnew3p1 <- sum((-Fy1new[ind1[group4]]^3-2*Fy1new[ind2[group4]]^3*(1-dynew[ind1[group4]])+3*Fy1new[ind1[group4]]*Fy1new[ind2[group4]]^2)/6/dynew[ind1[group4]])



  indnew3p2 <- sum((1-dynew[ind1[group2]])*(1-dynew[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^3-Fy1new[ind2[group2]]^3)/3) +
    sum((1-dynew[ind1[group3]])*(1-dynew[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^3-Fy1new[ind2[group3]]^3)/3) -
    sum(((1-dynew[ind1[group2]])*(Fy1new[ind2[group2]])+(1-dynew[ind2[group2]])*(Fy1new[ind1[group2]]))/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^2-Fy1new[ind2[group2]]^2)/2) -
    sum(((1-dynew[ind1[group3]])*(Fy1new[ind2[group3]])+(1-dynew[ind2[group3]])*(Fy1new[ind1[group3]]))/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^2-Fy1new[ind2[group3]]^2)/2) +
    sum((Fy1new[ind1[group2]])*(Fy1new[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]-Fy1new[ind2[group2]])) +
    sum((Fy1new[ind1[group3]])*(Fy1new[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]-Fy1new[ind2[group3]]))

  indnew3p3 <- sum(1/6/dynew[ind2[group2]]*(Fynew[ind2[group2]]^3+2*Fynew[ind1[group2]]^3-2*Fynew[ind1[group2]]^3*Fynew[ind2[group2]]+
                                              2*Fynew[ind1[group2]]^3*Fy1new[ind2[group2]]-
                                              3*Fynew[ind1[group2]]^2-3*Fynew[ind2[group2]]^2+3*Fynew[ind1[group2]]^2*Fynew[ind2[group2]]-
                                              6*Fynew[ind1[group2]]^2*Fy1new[ind2[group2]]+6*Fynew[ind1[group2]]*Fy1new[ind2[group2]]+
                                              2*Fynew[ind2[group2]]-2*Fy1new[ind2[group2]]))+
    sum(1/6/dynew[ind1[group3]]*(Fynew[ind1[group3]]^3+2*Fynew[ind2[group3]]^3-2*Fynew[ind2[group3]]^3*Fynew[ind1[group3]]+
                                   2*Fynew[ind2[group3]]^3*Fy1new[ind1[group3]]-
                                   3*Fynew[ind2[group3]]^2-3*Fynew[ind1[group3]]^2+3*Fynew[ind2[group3]]^2*Fynew[ind1[group3]]-
                                   6*Fynew[ind2[group3]]^2*Fy1new[ind1[group3]]+6*Fynew[ind2[group3]]*Fy1new[ind1[group3]]+
                                   2*Fynew[ind1[group3]]-2*Fy1new[ind1[group3]]))



  ## Test statistic
  disnull <- int1new+2*(ind2new1+ind2new2+indnew3p1+indnew3p2+indnew3p3)
  ## Bootstrap
  disin <- rep(0,B)

  for(repin in 1:B){

    yr <- rpois(n, lambda1f)
    #####!!!changes made
    #####!!!
    dfr <- data.frame(yr=yr,dfx[,-1])
    #fit marginal model
    poismodelr <- glm(yr~.,family=fam,data=dfr)
    lambda1fr <- poismodelr$fitted.values
    #####!!!changes made
    #####


    Fyr <- ppois(yr,lambda1fr)
    Fy1r <- ppois(yr-1,lambda1fr)
    dyr <- dpois(yr,lambda1fr)


    int1newr <- sum(1/3*(Fyr^4+Fy1r^4-Fyr^3*Fy1r-Fyr*Fy1r^3-2*Fyr^3-Fy1r^3+3*Fyr^2*Fy1r+Fyr^2-2*Fyr*Fy1r+Fy1r^2)/dyr^2)




    Fy1new <- sort(Fy1r)
    Fynew <- Fyr[sort(Fy1r,index=TRUE)$ix]
    dynew <- dyr[sort(Fy1r,index=TRUE)$ix]



    ###no overlap
    group1 <- which(Fy1new[ind2]>Fynew[ind1])
    ## overlap, and keep order. k=ind1,k'=ind2,m=ind2,m'=ind1
    group2 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]>Fynew[ind1])
    ## overlap, and switch order for on right side. k=ind1,k'=ind2,m=ind1,m'=ind2
    group3 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]<=Fynew[ind1])
    group4 <- which(Fy1new[ind2]<=Fynew[ind1])

    ind2new1r <- sum(((Fynew^3-Fy1new^3)/6/dynew)[ind1[group1]])

    ind2new2r <- sum(((Fynew^3-Fy1new^3-3*Fynew^2+3*Fy1new^2+2*Fynew-2*Fy1new)/dynew/6)[ind2[group1]])


    indnew3p1r <- sum((-Fy1new[ind1[group4]]^3-2*Fy1new[ind2[group4]]^3*(1-dynew[ind1[group4]])+3*Fy1new[ind1[group4]]*Fy1new[ind2[group4]]^2)/6/dynew[ind1[group4]])



    indnew3p2r <- sum((1-dynew[ind1[group2]])*(1-dynew[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^3-Fy1new[ind2[group2]]^3)/3) +
      sum((1-dynew[ind1[group3]])*(1-dynew[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^3-Fy1new[ind2[group3]]^3)/3) -
      sum(((1-dynew[ind1[group2]])*(Fy1new[ind2[group2]])+(1-dynew[ind2[group2]])*(Fy1new[ind1[group2]]))/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^2-Fy1new[ind2[group2]]^2)/2) -
      sum(((1-dynew[ind1[group3]])*(Fy1new[ind2[group3]])+(1-dynew[ind2[group3]])*(Fy1new[ind1[group3]]))/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^2-Fy1new[ind2[group3]]^2)/2) +
      sum((Fy1new[ind1[group2]])*(Fy1new[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]-Fy1new[ind2[group2]])) +
      sum((Fy1new[ind1[group3]])*(Fy1new[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]-Fy1new[ind2[group3]]))

    indnew3p3r <- sum(1/6/dynew[ind2[group2]]*(Fynew[ind2[group2]]^3+2*Fynew[ind1[group2]]^3-2*Fynew[ind1[group2]]^3*Fynew[ind2[group2]]+
                                                 2*Fynew[ind1[group2]]^3*Fy1new[ind2[group2]]-
                                                 3*Fynew[ind1[group2]]^2-3*Fynew[ind2[group2]]^2+3*Fynew[ind1[group2]]^2*Fynew[ind2[group2]]-
                                                 6*Fynew[ind1[group2]]^2*Fy1new[ind2[group2]]+6*Fynew[ind1[group2]]*Fy1new[ind2[group2]]+
                                                 2*Fynew[ind2[group2]]-2*Fy1new[ind2[group2]]))+
      sum(1/6/dynew[ind1[group3]]*(Fynew[ind1[group3]]^3+2*Fynew[ind2[group3]]^3-2*Fynew[ind2[group3]]^3*Fynew[ind1[group3]]+
                                     2*Fynew[ind2[group3]]^3*Fy1new[ind1[group3]]-
                                     3*Fynew[ind2[group3]]^2-3*Fynew[ind1[group3]]^2+3*Fynew[ind2[group3]]^2*Fynew[ind1[group3]]-
                                     6*Fynew[ind2[group3]]^2*Fy1new[ind1[group3]]+6*Fynew[ind2[group3]]*Fy1new[ind1[group3]]+
                                     2*Fynew[ind1[group3]]-2*Fy1new[ind1[group3]]))



    disin[repin] <-int1newr+2*(ind2new1r+ind2new2r+indnew3p1r+indnew3p2r+indnew3p3r)

  }

  ##p-value
  pvalue <- length(which(disin>disnull))/B

  return(list(test_stat = disnull, p_value = pvalue))
}

#' @importFrom utils combn
#' @importFrom pscl zeroinfl
#' @importFrom MASS rnegbin
#' @keywords internal
gof_znb <- function(B, model1 =NULL, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  if(is.null(model1)) stop("model object is not given.")
  y <- model1$model[, 1]
  n <- length(y)
  lambda1f <- predict(model1,type="count")
  pzero <- predict(model1,type="zero")
  size1f <- model1$theta
  df <- model1$model
  dis <- model1$dist
  lin <- model1$link
  formula1 <- model1$formula
  #####!!!
  #####!!!

  como <- combn(n,2)
  ind1 <- como[1,]
  ind2 <- como[2,]

  ## F(Y), F(Y-1), P(Y)

  Fy <- pzero + (1 - pzero) * pnbinom(y,mu=lambda1f,size=size1f)
  Fy1 <-  ifelse(y==0,0,(pzero + (1 - pzero) * pnbinom(y-1,mu=lambda1f,size=size1f)))
  dy <-  Fy-Fy1


  int1new <- sum(1/3*(Fy^4+Fy1^4-Fy^3*Fy1-Fy*Fy1^3-2*Fy^3-Fy1^3+3*Fy^2*Fy1+Fy^2-2*Fy*Fy1+Fy1^2)/dy^2)


  Fy1new <- sort(Fy1)
  Fynew <- Fy[sort(Fy1,index=TRUE)$ix]
  dynew <- dy[sort(Fy1,index=TRUE)$ix]




  ###no overlap
  group1 <- which(Fy1new[ind2]>Fynew[ind1])
  ## overlap, and keep order. k=ind1,k'=ind2,m=ind2,m'=ind1
  group2 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]>Fynew[ind1])
  ## overlap, and switch order for on right side. k=ind1,k'=ind2,m=ind1,m'=ind2
  group3 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]<=Fynew[ind1])
  group4 <- which(Fy1new[ind2]<=Fynew[ind1])

  ind2new1 <- sum(((Fynew^3-Fy1new^3)/6/dynew)[ind1[group1]])

  ind2new2 <- sum(((Fynew^3-Fy1new^3-3*Fynew^2+3*Fy1new^2+2*Fynew-2*Fy1new)/dynew/6)[ind2[group1]])


  indnew3p1 <- sum((-Fy1new[ind1[group4]]^3-2*Fy1new[ind2[group4]]^3*(1-dynew[ind1[group4]])+3*Fy1new[ind1[group4]]*Fy1new[ind2[group4]]^2)/6/dynew[ind1[group4]])



  indnew3p2 <- sum((1-dynew[ind1[group2]])*(1-dynew[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^3-Fy1new[ind2[group2]]^3)/3) +
    sum((1-dynew[ind1[group3]])*(1-dynew[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^3-Fy1new[ind2[group3]]^3)/3) -
    sum(((1-dynew[ind1[group2]])*(Fy1new[ind2[group2]])+(1-dynew[ind2[group2]])*(Fy1new[ind1[group2]]))/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^2-Fy1new[ind2[group2]]^2)/2) -
    sum(((1-dynew[ind1[group3]])*(Fy1new[ind2[group3]])+(1-dynew[ind2[group3]])*(Fy1new[ind1[group3]]))/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^2-Fy1new[ind2[group3]]^2)/2) +
    sum((Fy1new[ind1[group2]])*(Fy1new[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]-Fy1new[ind2[group2]])) +
    sum((Fy1new[ind1[group3]])*(Fy1new[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]-Fy1new[ind2[group3]]))

  indnew3p3 <- sum(1/6/dynew[ind2[group2]]*(Fynew[ind2[group2]]^3+2*Fynew[ind1[group2]]^3-2*Fynew[ind1[group2]]^3*Fynew[ind2[group2]]+
                                              2*Fynew[ind1[group2]]^3*Fy1new[ind2[group2]]-
                                              3*Fynew[ind1[group2]]^2-3*Fynew[ind2[group2]]^2+3*Fynew[ind1[group2]]^2*Fynew[ind2[group2]]-
                                              6*Fynew[ind1[group2]]^2*Fy1new[ind2[group2]]+6*Fynew[ind1[group2]]*Fy1new[ind2[group2]]+
                                              2*Fynew[ind2[group2]]-2*Fy1new[ind2[group2]]))+
    sum(1/6/dynew[ind1[group3]]*(Fynew[ind1[group3]]^3+2*Fynew[ind2[group3]]^3-2*Fynew[ind2[group3]]^3*Fynew[ind1[group3]]+
                                   2*Fynew[ind2[group3]]^3*Fy1new[ind1[group3]]-
                                   3*Fynew[ind2[group3]]^2-3*Fynew[ind1[group3]]^2+3*Fynew[ind2[group3]]^2*Fynew[ind1[group3]]-
                                   6*Fynew[ind2[group3]]^2*Fy1new[ind1[group3]]+6*Fynew[ind2[group3]]*Fy1new[ind1[group3]]+
                                   2*Fynew[ind1[group3]]-2*Fy1new[ind1[group3]]))



  disnull <- int1new+2*(ind2new1+ind2new2+indnew3p1+indnew3p2+indnew3p3)
  disin <- rep(0,B)

  for(repin in 1:B){
    y0r <- rbinom(n, size = 1, prob = 1 - pzero)
    y1r <- rnegbin(n, mu=lambda1f, theta=size1f)

    yr <- ifelse(y0r == 0, 0, y1r)

    #####!!!changes made
    #####!!!

    dfr <- df
    dfr[,1] <- yr
    #fit marginal model
    model1r <- zeroinfl(formula1, dist = dis, link = lin,data=dfr)

    lambda1fr <- predict(model1r,type="count")
    pzeror <- predict(model1r,type="zero")

    size1fr <- model1r$theta
    #####!!!
    #####!!!

    Fyr <- (pzeror + (1 - pzeror) *  pnbinom(yr,mu=lambda1fr,size=size1fr))
    Fy1r <-  ifelse(yr==0,0,(pzeror + (1 - pzeror) * pnbinom(yr-1,mu=lambda1fr,size=size1fr)))
    dyr <-  Fyr-Fy1r


    int1newr <- sum(1/3*(Fyr^4+Fy1r^4-Fyr^3*Fy1r-Fyr*Fy1r^3-2*Fyr^3-Fy1r^3+3*Fyr^2*Fy1r+Fyr^2-2*Fyr*Fy1r+Fy1r^2)/dyr^2)



    Fy1new <- sort(Fy1r)
    Fynew <- Fyr[sort(Fy1r,index=TRUE)$ix]
    dynew <- dyr[sort(Fy1r,index=TRUE)$ix]



    group1 <- which(Fy1new[ind2]>Fynew[ind1])
    ## overlap, and keep order. k=ind1,k'=ind2,m=ind2,m'=ind1
    group2 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]>Fynew[ind1])
    ## overlap, and switch order for on right side. k=ind1,k'=ind2,m=ind1,m'=ind2
    group3 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]<=Fynew[ind1])
    group4 <- which(Fy1new[ind2]<=Fynew[ind1])

    ind2new1r <- sum(((Fynew^3-Fy1new^3)/6/dynew)[ind1[group1]])

    ind2new2r <- sum(((Fynew^3-Fy1new^3-3*Fynew^2+3*Fy1new^2+2*Fynew-2*Fy1new)/dynew/6)[ind2[group1]])


    indnew3p1r <- sum((-Fy1new[ind1[group4]]^3-2*Fy1new[ind2[group4]]^3*(1-dynew[ind1[group4]])+3*Fy1new[ind1[group4]]*Fy1new[ind2[group4]]^2)/6/dynew[ind1[group4]])



    indnew3p2r <- sum((1-dynew[ind1[group2]])*(1-dynew[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^3-Fy1new[ind2[group2]]^3)/3) +
      sum((1-dynew[ind1[group3]])*(1-dynew[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^3-Fy1new[ind2[group3]]^3)/3) -
      sum(((1-dynew[ind1[group2]])*(Fy1new[ind2[group2]])+(1-dynew[ind2[group2]])*(Fy1new[ind1[group2]]))/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^2-Fy1new[ind2[group2]]^2)/2) -
      sum(((1-dynew[ind1[group3]])*(Fy1new[ind2[group3]])+(1-dynew[ind2[group3]])*(Fy1new[ind1[group3]]))/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^2-Fy1new[ind2[group3]]^2)/2) +
      sum((Fy1new[ind1[group2]])*(Fy1new[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]-Fy1new[ind2[group2]])) +
      sum((Fy1new[ind1[group3]])*(Fy1new[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]-Fy1new[ind2[group3]]))

    indnew3p3r <- sum(1/6/dynew[ind2[group2]]*(Fynew[ind2[group2]]^3+2*Fynew[ind1[group2]]^3-2*Fynew[ind1[group2]]^3*Fynew[ind2[group2]]+
                                                 2*Fynew[ind1[group2]]^3*Fy1new[ind2[group2]]-
                                                 3*Fynew[ind1[group2]]^2-3*Fynew[ind2[group2]]^2+3*Fynew[ind1[group2]]^2*Fynew[ind2[group2]]-
                                                 6*Fynew[ind1[group2]]^2*Fy1new[ind2[group2]]+6*Fynew[ind1[group2]]*Fy1new[ind2[group2]]+
                                                 2*Fynew[ind2[group2]]-2*Fy1new[ind2[group2]]))+
      sum(1/6/dynew[ind1[group3]]*(Fynew[ind1[group3]]^3+2*Fynew[ind2[group3]]^3-2*Fynew[ind2[group3]]^3*Fynew[ind1[group3]]+
                                     2*Fynew[ind2[group3]]^3*Fy1new[ind1[group3]]-
                                     3*Fynew[ind2[group3]]^2-3*Fynew[ind1[group3]]^2+3*Fynew[ind2[group3]]^2*Fynew[ind1[group3]]-
                                     6*Fynew[ind2[group3]]^2*Fy1new[ind1[group3]]+6*Fynew[ind2[group3]]*Fy1new[ind1[group3]]+
                                     2*Fynew[ind1[group3]]-2*Fy1new[ind1[group3]]))



    disin[repin] <-int1newr+2*(ind2new1r+ind2new2r+indnew3p1r+indnew3p2r+indnew3p3r)


  }
  pvalue <- length(which(disin>disnull))/B

  return(list(test_stat = disnull, p_value = pvalue))
}

#' @importFrom utils combn
#' @importFrom pscl zeroinfl
#' @keywords internal
gof_zpois <- function(B, model1=NULL, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  if(is.null(model1)) stop("model object is not given.")

  y <- model1$model[, 1]
  n <- length(y)
  meanpoisson <- predict(model1,type="count")
  pzero <- predict(model1,type="zero")
  df <- model1$model
  dis <- model1$dist
  lin <- model1$link
  formula1 <- model1$formula
  #####!!!
  #####!!!


  como <- combn(n,2)
  ind1 <- como[1,]
  ind2 <- como[2,]


  ## F(Y), F(Y-1), P(Y)

  Fy <- (pzero + (1 - pzero) * (ppois(y, lambda = meanpoisson)))
  Fy1 <-  ifelse(y==0,0,(pzero + (1 - pzero) * (ppois(y-1, lambda = meanpoisson))))
  dy <-  Fy-Fy1


  int1new <- sum(1/3*(Fy^4+Fy1^4-Fy^3*Fy1-Fy*Fy1^3-2*Fy^3-Fy1^3+3*Fy^2*Fy1+Fy^2-2*Fy*Fy1+Fy1^2)/dy^2)


  Fy1new <- sort(Fy1)
  Fynew <- Fy[sort(Fy1,index=TRUE)$ix]
  dynew <- dy[sort(Fy1,index=TRUE)$ix]




  ###no overlap
  group1 <- which(Fy1new[ind2]>Fynew[ind1])
  ## overlap, and keep order. k=ind1,k'=ind2,m=ind2,m'=ind1
  group2 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]>Fynew[ind1])
  ## overlap, and switch order for on right side. k=ind1,k'=ind2,m=ind1,m'=ind2
  group3 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]<=Fynew[ind1])
  group4 <- which(Fy1new[ind2]<=Fynew[ind1])

  ind2new1 <- sum(((Fynew^3-Fy1new^3)/6/dynew)[ind1[group1]])

  ind2new2 <- sum(((Fynew^3-Fy1new^3-3*Fynew^2+3*Fy1new^2+2*Fynew-2*Fy1new)/dynew/6)[ind2[group1]])


  indnew3p1 <- sum((-Fy1new[ind1[group4]]^3-2*Fy1new[ind2[group4]]^3*(1-dynew[ind1[group4]])+3*Fy1new[ind1[group4]]*Fy1new[ind2[group4]]^2)/6/dynew[ind1[group4]])



  indnew3p2 <- sum((1-dynew[ind1[group2]])*(1-dynew[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^3-Fy1new[ind2[group2]]^3)/3) +
    sum((1-dynew[ind1[group3]])*(1-dynew[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^3-Fy1new[ind2[group3]]^3)/3) -
    sum(((1-dynew[ind1[group2]])*(Fy1new[ind2[group2]])+(1-dynew[ind2[group2]])*(Fy1new[ind1[group2]]))/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^2-Fy1new[ind2[group2]]^2)/2) -
    sum(((1-dynew[ind1[group3]])*(Fy1new[ind2[group3]])+(1-dynew[ind2[group3]])*(Fy1new[ind1[group3]]))/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^2-Fy1new[ind2[group3]]^2)/2) +
    sum((Fy1new[ind1[group2]])*(Fy1new[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]-Fy1new[ind2[group2]])) +
    sum((Fy1new[ind1[group3]])*(Fy1new[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]-Fy1new[ind2[group3]]))

  indnew3p3 <- sum(1/6/dynew[ind2[group2]]*(Fynew[ind2[group2]]^3+2*Fynew[ind1[group2]]^3-2*Fynew[ind1[group2]]^3*Fynew[ind2[group2]]+
                                              2*Fynew[ind1[group2]]^3*Fy1new[ind2[group2]]-
                                              3*Fynew[ind1[group2]]^2-3*Fynew[ind2[group2]]^2+3*Fynew[ind1[group2]]^2*Fynew[ind2[group2]]-
                                              6*Fynew[ind1[group2]]^2*Fy1new[ind2[group2]]+6*Fynew[ind1[group2]]*Fy1new[ind2[group2]]+
                                              2*Fynew[ind2[group2]]-2*Fy1new[ind2[group2]]))+
    sum(1/6/dynew[ind1[group3]]*(Fynew[ind1[group3]]^3+2*Fynew[ind2[group3]]^3-2*Fynew[ind2[group3]]^3*Fynew[ind1[group3]]+
                                   2*Fynew[ind2[group3]]^3*Fy1new[ind1[group3]]-
                                   3*Fynew[ind2[group3]]^2-3*Fynew[ind1[group3]]^2+3*Fynew[ind2[group3]]^2*Fynew[ind1[group3]]-
                                   6*Fynew[ind2[group3]]^2*Fy1new[ind1[group3]]+6*Fynew[ind2[group3]]*Fy1new[ind1[group3]]+
                                   2*Fynew[ind1[group3]]-2*Fy1new[ind1[group3]]))



  disnull <- int1new+2*(ind2new1+ind2new2+indnew3p1+indnew3p2+indnew3p3)
  disin <- rep(0,B)

  for(repin in 1:B){

    y0r <- rbinom(n, size = 1, prob = 1 - pzero)
    y1r <- rpois(n, meanpoisson)

    yr <- ifelse(y0r == 0, 0, y1r)
    #####!!!changes made
    #####!!!

    dfr <- df
    dfr[,1] <- yr
    #fit marginal model
    model1r <- zeroinfl(formula1, dist = dis, link = lin,data=dfr)

    meanpoissonr <- predict(model1r,type="count")
    pzeror <- predict(model1r,type="zero")

    size1fr <- model1r$theta
    #####!!!
    #####!!!

    Fyr <- (pzeror + (1 - pzeror) * (ppois(yr, lambda = meanpoissonr)))
    Fy1r <-  ifelse(yr==0,0,(pzeror + (1 - pzeror) * (ppois(yr-1, lambda = meanpoissonr))))
    dyr <-  Fyr-Fy1r


    int1newr <- sum(1/3*(Fyr^4+Fy1r^4-Fyr^3*Fy1r-Fyr*Fy1r^3-2*Fyr^3-Fy1r^3+3*Fyr^2*Fy1r+Fyr^2-2*Fyr*Fy1r+Fy1r^2)/dyr^2)



    Fy1new <- sort(Fy1r)
    Fynew <- Fyr[sort(Fy1r,index=TRUE)$ix]
    dynew <- dyr[sort(Fy1r,index=TRUE)$ix]



    group1 <- which(Fy1new[ind2]>Fynew[ind1])
    ## overlap, and keep order. k=ind1,k'=ind2,m=ind2,m'=ind1
    group2 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]>Fynew[ind1])
    ## overlap, and switch order for on right side. k=ind1,k'=ind2,m=ind1,m'=ind2
    group3 <- which(Fy1new[ind2]<=Fynew[ind1]&Fynew[ind2]<=Fynew[ind1])
    group4 <- which(Fy1new[ind2]<=Fynew[ind1])

    ind2new1r <- sum(((Fynew^3-Fy1new^3)/6/dynew)[ind1[group1]])

    ind2new2r <- sum(((Fynew^3-Fy1new^3-3*Fynew^2+3*Fy1new^2+2*Fynew-2*Fy1new)/dynew/6)[ind2[group1]])


    indnew3p1r <- sum((-Fy1new[ind1[group4]]^3-2*Fy1new[ind2[group4]]^3*(1-dynew[ind1[group4]])+3*Fy1new[ind1[group4]]*Fy1new[ind2[group4]]^2)/6/dynew[ind1[group4]])



    indnew3p2r <- sum((1-dynew[ind1[group2]])*(1-dynew[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^3-Fy1new[ind2[group2]]^3)/3) +
      sum((1-dynew[ind1[group3]])*(1-dynew[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^3-Fy1new[ind2[group3]]^3)/3) -
      sum(((1-dynew[ind1[group2]])*(Fy1new[ind2[group2]])+(1-dynew[ind2[group2]])*(Fy1new[ind1[group2]]))/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]^2-Fy1new[ind2[group2]]^2)/2) -
      sum(((1-dynew[ind1[group3]])*(Fy1new[ind2[group3]])+(1-dynew[ind2[group3]])*(Fy1new[ind1[group3]]))/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]^2-Fy1new[ind2[group3]]^2)/2) +
      sum((Fy1new[ind1[group2]])*(Fy1new[ind2[group2]])/dynew[ind1[group2]]/dynew[ind2[group2]]*(Fynew[ind1[group2]]-Fy1new[ind2[group2]])) +
      sum((Fy1new[ind1[group3]])*(Fy1new[ind2[group3]])/dynew[ind1[group3]]/dynew[ind2[group3]]*(Fynew[ind2[group3]]-Fy1new[ind2[group3]]))

    indnew3p3r <- sum(1/6/dynew[ind2[group2]]*(Fynew[ind2[group2]]^3+2*Fynew[ind1[group2]]^3-2*Fynew[ind1[group2]]^3*Fynew[ind2[group2]]+
                                                 2*Fynew[ind1[group2]]^3*Fy1new[ind2[group2]]-
                                                 3*Fynew[ind1[group2]]^2-3*Fynew[ind2[group2]]^2+3*Fynew[ind1[group2]]^2*Fynew[ind2[group2]]-
                                                 6*Fynew[ind1[group2]]^2*Fy1new[ind2[group2]]+6*Fynew[ind1[group2]]*Fy1new[ind2[group2]]+
                                                 2*Fynew[ind2[group2]]-2*Fy1new[ind2[group2]]))+
      sum(1/6/dynew[ind1[group3]]*(Fynew[ind1[group3]]^3+2*Fynew[ind2[group3]]^3-2*Fynew[ind2[group3]]^3*Fynew[ind1[group3]]+
                                     2*Fynew[ind2[group3]]^3*Fy1new[ind1[group3]]-
                                     3*Fynew[ind2[group3]]^2-3*Fynew[ind1[group3]]^2+3*Fynew[ind2[group3]]^2*Fynew[ind1[group3]]-
                                     6*Fynew[ind2[group3]]^2*Fy1new[ind1[group3]]+6*Fynew[ind2[group3]]*Fy1new[ind1[group3]]+
                                     2*Fynew[ind1[group3]]-2*Fy1new[ind1[group3]]))



    disin[repin] <-int1newr+2*(ind2new1r+ind2new2r+indnew3p1r+indnew3p2r+indnew3p3r)


  }
  ##p-value
  pvalue <- length(which(disin>disnull))/B


  return(list(test_stat = disnull, p_value = pvalue))
}

#' Quasi empirical residuals functions
#'
#' Draw the quasi-empirical residual distribution functions for regression models with discrete outcomes.
#' Specifically, the model assumption of GLMs with binary, ordinal, Poisson, negative binomial,
#' zero-inflated Poisson, and zero-inflated negative binomial outcomes can be assessed using `quasi_plot()`.
#' A plot far apart from the diagonal indicates lack of fit.
#'
#'
#' @usage quasi_plot(model, line_args=list(), ...)
#'
#' @param model Model object (e.g., `glm`, `glm.nb`, `polr`, `zeroinfl`)
#' @param line_args A named list of graphical parameters passed to
#'   \code{graphics::abline()} to modify the reference (red) 45° line
#'   in the QQ plot. If left empty, a default red dashed line is drawn.
#' @param ... Additional graphical arguments passed to
#'   \code{stats::qqplot()} for customizing the QQ plot (e.g., \code{lty},
#'   \code{col}, \code{lwd}, \code{xlab}, \code{ylab}).
#'
#' @import np
#'
#' @returns A plot of quasi-empirial residual distribution function \eqn{\hat{U}(s;\beta)} against \eqn{s}.
#'
#' @details
#' The quasi-empirical residual distribution function is defined as follows:
#' \deqn{\hat{U}(s; \beta) = \sum_{i=1}^{n} W_{n}(s;\mathbf{X}_{i},\beta) 1[F(Y_{i}| X_{i}) < H(s;X_{i})]}
#' where
#' \deqn{W_n(s; \mathbf{X}_i, \beta) = \frac{K[(H(s; \mathbf{X}_i)-s)/ \epsilon_n]}{\sum_{j=1}^{n} K[(H(s; \mathbf{X}_j)-s)/ \epsilon_n]},}
#' \eqn{\epsilon_n} is the bandwidth; \eqn{H(s, X_i) = \mathrm{argmin}_{F(k \mid X_i)} |F(k \mid X_i) - s|} and \eqn{K} is a bounded, symmetric, and Lipschitz continuous kernel.
#'
#' @references Lu Yang (2021). Assessment of Regression Models with Discrete Outcomes Using Quasi-Empirical Residual Distribution Functions, Journal of Computational and Graphical Statistics, 30(4), 1019-1035.
#' @export
#'
#' @examples
#' ## Negative Binomial example
#' library(MASS)
#' # Covariates
#' n <- 500
#' x1 <- rnorm(n)
#' x2 <- rbinom(n, 1, 0.7)
#' ### Parameters
#' beta0 <- -2
#' beta1 <- 2
#' beta2 <- 1
#' size1 <- 2
#' lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
#' # generate outcomes
#' y <- rnbinom(n, mu = lambda1, size = size1)
#'
#' # True model
#' model1 <- glm.nb(y ~ x1 + x2)
#' resid.nb1 <- quasi_plot(model1)
#'
#' # Overdispersion
#' model2 <- glm(y ~ x1 + x2, family = poisson(link = "log"))
#' resid.nb2 <- quasi_plot(model2)
#'
#' ## Zero inflated Poisson example
#' library(pscl)
#' n <- 500
#' set.seed(1234)
#' # Covariates
#' x1 <- rnorm(n)
#' x2 <- rbinom(n, 1, 0.7)
#' # Coefficients
#' beta0 <- -2
#' beta1 <- 2
#' beta2 <- 1
#' beta00 <- -2
#' beta10 <- 2
#'
#' # Mean of Poisson part
#' lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
#' # Excess zero probability
#' p0 <- 1 / (1 + exp(-(beta00 + beta10 * x1)))
#' ## simulate outcomes
#' y0 <- rbinom(n, size = 1, prob = 1 - p0)
#' y1 <- rpois(n, lambda1)
#' y <- ifelse(y0 == 0, 0, y1)
#' ## True model
#' modelzero1 <- zeroinfl(y ~ x1 + x2 | x1, dist = "poisson", link = "logit")
#' resid.zero1 <- quasi_plot(modelzero1)
quasi_plot <- function(model, line_args = list(), ...) {
  UseMethod("quasi_plot")
}

#' @rawNamespace S3method(quasi_plot,glm)
quasi_plot.glm <- function(model, line_args = list(), ...) {
  .quasi_plot_glm(model, line_args = line_args, ...)
}

.quasi_plot_glm <- function(model, line_args = list(), ...) {
  fam_obj <- stats::family(model)
  fam_raw <- fam_obj$family
  link_raw <- fam_obj$link

  fam_key <- fam_raw
  if (startsWith(fam_raw, "Negative Binomial")) {
    fam_key <- "nb"
  }

  key <- list(
    model     = model,
    family    = fam_raw,
    link      = link_raw,
    line_args = line_args
  )

  class(key) <- fam_key
  .quasi_plot_glm_key(key, ...)
}

.quasi_plot_glm_key <- function(key, ...) {
  UseMethod(".quasi_plot_glm_key")
}

#' @rawNamespace S3method(.quasi_plot_glm_key,nb)
.quasi_plot_glm_key.nb <- function(key, ...) {
  if (!exists("resid.nb_quasi", mode = "function", inherits = TRUE)) {
    stop("resid.nb_quasi() not found.", call. = FALSE)
  }
  resid.nb_quasi(key$model, line_args = key$line_args, ...)
}

#' @rawNamespace S3method(.quasi_plot_glm_key,poisson)
.quasi_plot_glm_key.poisson <- function(key, ...) {
  if (!exists("resid.pois_quasi", mode = "function", inherits = TRUE)) {
    stop("resid.pois_quasi() not found.", call. = FALSE)
  }
  resid.pois_quasi(key$model, line_args = key$line_args, ...)
}

#' @rawNamespace S3method(.quasi_plot_glm_key,binomial)
.quasi_plot_glm_key.binomial <- function(key, ...) {
  if (!exists("resid.bin_quasi", mode = "function", inherits = TRUE)) {
    stop("resid.bin_quasi() not found.", call. = FALSE)
  }
  resid.bin_quasi(key$model, line_args = key$line_args, ...)
}

#' @rawNamespace S3method(.quasi_plot_glm_key,default)
.quasi_plot_glm_key.default <- function(key, ...) {
  stop(
    sprintf("Unsupported GLM family/link for quasi_plot(): %s / %s", key$family, key$link),
    call. = FALSE
  )
}


#' @rawNamespace S3method(quasi_plot,zeroinfl)
quasi_plot.zeroinfl <- function(model, line_args = list(), ...) {
  .quasi_plot_zeroinfl(model, line_args = line_args, ...)
}

.quasi_plot_zeroinfl <- function(model, line_args = list(), ...) {
  dist_raw <- model$dist

  key <- list(
    model     = model,
    dist      = dist_raw,
    line_args = line_args
  )

  class(key) <- dist_raw
  .quasi_plot_zeroinfl_key(key, ...)
}

.quasi_plot_zeroinfl_key <- function(key, ...) {
  UseMethod(".quasi_plot_zeroinfl_key")
}

#' @rawNamespace S3method(.quasi_plot_zeroinfl_key,poisson)
.quasi_plot_zeroinfl_key.poisson <- function(key, ...) {
  if (!exists("resid.zpois_quasi", mode = "function", inherits = TRUE)) {
    stop("resid.zpois_quasi() not found.", call. = FALSE)
  }
  resid.zpois_quasi(key$model, line_args = key$line_args, ...)
}

#' @rawNamespace S3method(.quasi_plot_zeroinfl_key,negbin)
.quasi_plot_zeroinfl_key.negbin <- function(key, ...) {
  if (!exists("resid.znb_quasi", mode = "function", inherits = TRUE)) {
    stop("resid.znb_quasi() not found.", call. = FALSE)
  }
  resid.znb_quasi(key$model, line_args = key$line_args, ...)
}

#' @rawNamespace S3method(.quasi_plot_zeroinfl_key,default)
.quasi_plot_zeroinfl_key.default <- function(key, ...) {
  stop(sprintf("Unsupported zeroinfl dist for quasi_plot(): %s", key$dist), call. = FALSE)
}


#' @rawNamespace S3method(quasi_plot,polr)
quasi_plot.polr <- function(model, line_args = list(), ...) {
  if (!exists("resid.ordi_quasi", mode = "function", inherits = TRUE)) {
    stop("resid.ordi_quasi() not found.", call. = FALSE)
  }
  resid.ordi_quasi(model, line_args = line_args, ...)
}


#' @rawNamespace S3method(quasi_plot,default)
quasi_plot.default <- function(model, line_args = list(), ...) {
  cls <- paste(class(model), collapse = ", ")
  stop(
    sprintf("Unsupported model class for quasi_plot(): %s. Supported: glm, zeroinfl, polr.", cls),
    call. = FALSE
  )
}

#' Residuals for regression models with negative binomial outcomes
#'
#' Computes DPIT residuals for regression models with negative binomial
#' outcomes using the observed counts (`y`) and their fitted distributional
#' parameters (`mu`, `size`).
#'
#' @usage dpit_nb(y, mu, size, plot=TRUE, scale="normal", line_args=list(), ...)
#' @param y An observed outcome vector.
#' @param mu A vector of fitted mean values.
#' @param size A dispersion parameter of the negative binomial distribution.
#' @param plot A logical value indicating whether or not to return QQ-plot
#' @param scale You can choose the scale of the residuals among `normal` and `uniform`.
#' The sample quantiles of the residuals are plotted against
#' the theoretical quantiles of a standard normal distribution under the normal scale,
#' and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
#'  The default scale is `normal`.
#' @param line_args A named list of graphical parameters passed to
#'   \code{graphics::abline()} to modify the reference (red) 45° line
#'   in the QQ plot. If left empty, a default red dashed line is drawn.
#' @param ... Additional graphical arguments passed to
#'   \code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
#'   \code{col}, \code{cex}, \code{xlab}, \code{ylab}).
#' @returns DPIT residuals.
#'
#' @details
#' For formulation details on discrete outcomes, see \code{\link{dpit}}.
#'
#' @examples
#' ## Negative Binomial example
#' library(MASS)
#' n <- 500
#' x1 <- rnorm(n)
#' x2 <- rbinom(n, 1, 0.7)
#' ### Parameters
#' beta0 <- -2
#' beta1 <- 2
#' beta2 <- 1
#' size1 <- 2
#' lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
#' # generate outcomes
#' y <- rnbinom(n, mu = lambda1, size = size1)
#'
#' # True model
#' model1 <- glm.nb(y ~ x1 + x2)
#' y1 <- model1$y
#' fitted1 <- fitted(model1)
#' size1 <- model1$theta
#' resid.nb1 <- dpit_nb(y=y1, mu=fitted1, size=size1)
#'
#' # Overdispersion
#' model2 <- glm(y ~ x1 + x2, family = poisson(link = "log"))
#' y2 <- model2$y
#' fitted2 <- fitted(model2)
#' resid.nb2 <- dpit_pois(y=y2, mu=fitted2)
#' @export
dpit_nb <- function(y, mu, size,
                    plot=TRUE, scale="normal", line_args=list(), ...){
  lambda1f <- mu
  size1f <- size
  n <- length(y)
  res <- pnbinom(y, mu = lambda1f, size = size1f)

  empcdf <- rep(NA,n)
  for(i in 1:n){
    qres <- qnbinom(res[i], mu=lambda1f, size=size1f)-1
    pres <- pnbinom(qres,mu=lambda1f,size=size1f)
    pres[i] <- 0
    empcdf[i] <-sum(pres)/(n-1)
  }
  .dpit_finalize(empcdf, plot=plot, scale=scale, line_args=line_args, ...)
}

#' @keywords internal
inv.znb <- function(s, pzero, mu.hat, size1f) {
  qres <- ifelse(s < (pzero + (1 - pzero) * (pnbinom(0, size = size1f, mu = mu.hat))), 0,
                 qnbinom(pmax((s - pzero) / (1 - pzero), 0), mu = mu.hat, size = size1f)
  ) - 1
  pres <- ifelse(qres == -1, 0, (pzero + (1 - pzero) * (pnbinom(qres, size = size1f, mu = mu.hat))))
  return(pres)
}

#' Residuals for regression models with zero-inflated negative binomial outcomes
#'
#' Computes DPIT residuals for regression models with zero-inflated negative
#' binomial outcomes using the observed counts (`y`) and their fitted distributional
#' parameters (`mu`, `pzero`, `size`).
#'
#' @usage dpit_znb(y, mu, pzero, size, plot=TRUE, scale="normal", line_args=list(), ...)
#' @param y An observed outcome vector.
#' @param mu A vector of fitted mean values for the count (non-zero) component.
#' @param pzero A vector of fitted probabilities for the zero-inflation component.
#' @param size A dispersion parameter of the negative binomial distribution.
#' @param plot A logical value indicating whether or not to return QQ-plot
#' @param scale You can choose the scale of the residuals among `normal` and `uniform`.
#' The sample quantiles of the residuals are plotted against
#' the theoretical quantiles of a standard normal distribution under the normal scale,
#' and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
#'  The default scale is `normal`.
#' @param line_args A named list of graphical parameters passed to
#'   \code{graphics::abline()} to modify the reference (red) 45° line
#'   in the QQ plot. If left empty, a default red dashed line is drawn.
#' @param ... Additional graphical arguments passed to
#'   \code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
#'   \code{col}, \code{cex}, \code{xlab}, \code{ylab}).
#' @returns DPIT residuals.
#' @importFrom stats family
#' @export
#'
#'
#' @details
#' For formulation details on discrete outcomes, see \code{\link{dpit}}.
#'
#' @examples
#' ## Zero-Inflated Negative Binomial
#' library(pscl)
#' n <- 500
#' set.seed(1234)
#'
#' # Covariates
#' x1 <- rnorm(n)
#' x2 <- rbinom(n, 1, 0.7)
#'
#' # Coefficients
#' beta0 <- -2
#' beta1 <-  2
#' beta2 <-  1
#' beta00 <- -2
#' beta10 <-  2
#'
#' # NB dispersion (size = theta; larger => closer to Poisson)
#' theta_true <- 1.2
#'
#' # Mean of NB count part
#' mu_true <- exp(beta0 + beta1 * x1 + beta2 * x2)
#'
#' # Excess zero probability (logit)
#' p0 <- 1 / (1 + exp(-(beta00 + beta10 * x1)))
#'
#' ## simulate outcomes
#' z  <- rbinom(n, size = 1, prob = 1 - p0)              # 1 => from NB, 0 => structural zero
#' y1 <- rnbinom(n, size = theta_true, mu = mu_true)     # NB count draw
#' y  <- ifelse(z == 0, 0, y1)
#'
#' ## True model
#' modelzero1 <- zeroinfl(y ~ x1 + x2 | x1, dist = "negbin", link = "logit")
#' y1 <- modelzero1$y
#' mu1    <- stats::predict(modelzero1, type = "count")
#' pzero1 <- stats::predict(modelzero1, type = "zero")
#' theta1 <- modelzero1$theta
#' resid.zero1 <- dpit_znb(y = y1, pzero = pzero1, mu = mu1, size = theta1)
#'
#' ## Ignoring zero-inflation: NB only
#' modelzero2 <- MASS::glm.nb(y ~ x1 + x2)
#' y2 <- modelzero2$y
#' mu2    <- fitted(modelzero2)
#' theta2 <- modelzero2$theta
#' resid.zero2 <- dpit_nb(y = y2, mu = mu2, size = theta2)
#' @export
dpit_znb <- function(y, mu, pzero, size, plot=TRUE, scale="normal", line_args=list(), ...) {
  mu.hat <- mu
  pzero <- pzero
  size1f <- size
  n <- length(y)
  res <- (pzero + (1 - pzero) * (pnbinom(y, size = size1f, mu = mu.hat)))

  # residuals
  empcdf <- rep(NA,0)
  for(i in 1:n){
    pres <- inv.znb(res[i], pzero, mu.hat, size1f)
    pres[i] <-0
    empcdf[i] <- sum(pres)/(n-1)
  }
  .dpit_finalize(empcdf, plot=plot, scale=scale, line_args=line_args, ...)
}

#' @keywords internal
.resid_quasi_finalize <- function(x,
                                  y,
                                  main,
                                  xlab = "s",
                                  ylab = expression(hat(U) * "(s)"),
                                  xlim = range(x, na.rm = TRUE),
                                  line_args = list(),
                                  ...) {

  defaults <- list(
    type     = "l",
    main     = main,
    ylab     = ylab,
    xlab     = xlab,
    cex.lab  = 2,
    cex.axis = 2,
    cex.main = 2,
    lwd      = 2,
    xlim     = xlim
  )

  qq_args <- modifyList(defaults, list(...))

  do.call(plot, c(list(x = x, y = y), qq_args))

  default_abline <- list(
    a   = 0,
    b   = 1,
    col = "red",
    lty = 5,
    lwd = 2
  )
  ab_args <- modifyList(default_abline, line_args)
  do.call(abline, ab_args)

  invisible(NULL)
}


#' @keywords internal
listvec <- function(x) {
  x[1]:x[2]
}

## Binary
margin01 <- function(u, y, q0, h) {
  wei <- 1 * ((q0 - u)^2 < 5 * h^2) * (1 - ((q0 - u)^2) / h^2 / 5)
  l <- sum(wei * 1 * (y == 0)) / sum(wei)
  l
}
margin01 <- Vectorize(margin01, "u")

## bandwidth selector
bandwidth01 <- function(y, q0) {
  bw <- npregbw(ydat = 1 * (y == 0), xdat = q0, ckertype = "epanechnikov")
  return(bw$bw)
}

#' @keywords internal
#' @importFrom utils modifyList
resid.bin_quasi <- function(model, line_args, ...){
  y   <- model$y
  q10 <- 1 - model$fitted.values

  h       <- bandwidth01(y = y, q0 = q10)
  x.input <- seq(0, 1, length.out = 101)
  y_curve <- margin01(x.input, y = y, q0 = q10, h = h)

  xlim_vec <- c(
    min(pbinom(0, size = 1, prob = q10)),
    max(pbinom(0, size = 1, prob = q10))
  )

  .resid_quasi_finalize(
    x         = x.input,
    y         = y_curve,
    main      = "Quasi, Binary",
    xlim      = xlim_vec,
    line_args = line_args,
    ...
  )
}


## Poisson
marginesti <- function(u, y, lambdaf, h) {
  n <- length(y)
  # F^{-1}(u)
  inv1 <- qpois(u, lambdaf)
  inv1m <- ifelse(inv1 > 0, inv1 - 1, 0)
  n1 <- cbind(inv1, inv1m)
  # H^+ and H^_
  p1 <- ppois(n1, lambdaf)
  # find out which one is closer to u
  ind1 <- apply(abs(p1 - u), 1, which.min)
  # weight function
  wei <- 1 * ((p1[cbind(1:n, ind1)] - u)^2 < 5 * h^2) *
    (1 - ((p1[cbind(1:n, ind1)] - u)^2) / h^2 / 5)
  l <- sum(wei * 1 * (y <= n1[cbind(1:n, ind1)])) / sum(wei)
  l
}
marginesti <- Vectorize(marginesti, "u")

####### bandwidth selector
bandwidthp <- function(y, lambdaf){
  n <- length(y)
  newout <- unlist(sapply(split(cbind(qpois(0.1, lambdaf), qpois(0.9, lambdaf)), 1:n), listvec))
  newlambda <- rep(lambdaf, times = qpois(0.9, lambdaf) - qpois(0.1, lambdaf) + 1)
  newx <- ppois(newout, newlambda)
  newy <- 1 * (rep(y, times = qpois(0.9, lambdaf) - qpois(0.1, lambdaf) + 1) <= newout)
  bws <- npregbw(ydat = newy[which(newx <= 0.9)], xdat = newx[which(newx <= 0.9)], ckertype = "epanechnikov")
  return(bws$bw)
}

#' @keywords internal
resid.pois_quasi <- function(model, line_args, ...){
  y        <- model$y
  lambda1f <- model$fitted.values

  h       <- bandwidthp(y = y, lambdaf = lambda1f)
  x.input <- seq(0, 1, length.out = 101)
  y_curve <- marginesti(u = x.input, y = y, lambdaf = lambda1f, h = h)

  xlim_vec <- c(min(ppois(0, lambda = lambda1f)), 1)

  .resid_quasi_finalize(
    x         = x.input,
    y         = y_curve,
    main      = "Quasi, Poisson",
    xlim      = xlim_vec,
    line_args = line_args,
    ...
  )
}

############
##   NB   ##
############
marginnb <- function(u, y, lambdaf, sizef,h) {
  n <- length(y)
  inv1 <- qnbinom(u, mu = lambdaf, size = sizef)
  inv1m <- ifelse(inv1 > 0, inv1 - 1, 0)
  n1 <- cbind(inv1, inv1m)
  p1 <- pnbinom(n1, mu = lambdaf, size = sizef)
  ind1 <- apply(abs(p1 - u), 1, which.min)
  wei <- 1 * ((p1[cbind(1:n, ind1)] - u)^2 < 5 * h^2) *
    (1 - ((p1[cbind(1:n, ind1)] - u)^2) / h^2 / 5)
  l <- sum(wei * 1 * (y <= n1[cbind(1:n, ind1)])) / sum(wei)
  l
}

marginnb <- Vectorize(marginnb, "u")

## bandwidth selector
bandwidthnb <- function(y, lambdaf, sizef) {
  n <- length(y)
  newout <- unlist(sapply(split(cbind(qnbinom(0.1, mu = lambdaf, size = sizef), qnbinom(0.9, mu = lambdaf, size = sizef)), 1:n), listvec))
  newlambda <- rep(lambdaf, times = qnbinom(0.9, mu = lambdaf, size = sizef) - qnbinom(0.1, mu = lambdaf, size = sizef) + 1)
  newx <- pnbinom(newout, mu = newlambda, size = sizef)
  newy <- 1 * (rep(y, times = qnbinom(0.9, mu = lambdaf, size = sizef) - qnbinom(0.1, mu = lambdaf, size = sizef) + 1) <= newout)

  newy1 <- newy[which(newx <= 0.9)]
  newx1 <- newx[which(newx <= 0.9)]
  bws <- npregbw(ydat = newy1, xdat = newx1, ckertype = "epanechnikov")
  return(bws$bw)
}

#' @keywords internal
resid.nb_quasi <- function(model, line_args, ...){
  y        <- model$y
  lambda1f <- model$fitted.values
  size1f   <- summary(model)$theta

  h       <- bandwidthnb(y = y, lambdaf = lambda1f, sizef = size1f)
  x.input <- seq(0, 1, length.out = 101)
  y_curve <- marginnb(u = x.input, y = y,
                      lambdaf = lambda1f, sizef = size1f, h = h)

  xlim_vec <- c(min(pnbinom(0, size = size1f, mu = lambda1f)), 1)

  .resid_quasi_finalize(
    x         = x.input,
    y         = y_curve,
    main      = "Quasi, NB",
    xlim      = xlim_vec,
    line_args = line_args,
    ...
  )
}

###########
## Ord  ###
###########
### \hat{U} y is the outcome, q0=P(y<=1), q1=P(y<=2)
marginm <- function(x, y, p1, h) { # p1 = t(apply(model$fitted.values,1 ,cumsum))[,-n]
  n <- length(y)
  ind1 <- apply(abs(p1 - x), 1, which.min)
  wei <- 1 * ((p1[cbind(1:n, ind1)] - x)^2 < 5 * h^2) *
    (1 - ((p1[cbind(1:n, ind1)] - x)^2) / h^2 / 5)
  l <- sum(wei * 1 * (y <= (ind1 - 1))) / sum(wei)
  l
}

marginm <- Vectorize(marginm, "x")

### Bandwidth selection
bandwidthord <- function(y, p1) {
  k <- max(y)
  n <- length(y)
  xdat. <- rep(NA, n*k)
  for(i in 0:(k-1)) xdat.[(i*n+1):((i+1)*n)] <- p1[, (i+1)]
  ydat. <- rep(NA,n*k)
  for(i in 0:(k-1)) ydat.[(i*n+1):((i+1)*n)] <- 1*(y <= i)
  bw <- npregbw(ydat = ydat., xdat = xdat., ckertype = "epanechnikov")
  return(bw$bw)
}

#' @keywords internal
resid.ordi_quasi <- function(model, line_args, ...){
  y  <- as.numeric(model$model[,1]) - 1
  p1 <- t(apply(model$fitted.values, 1, cumsum))

  h       <- bandwidthord(y = y, p1 = p1)
  x.input <- seq(0, 1, length.out = 101)
  y_curve <- marginm(x = x.input, y = y, p1 = p1, h = h)

  xlim_vec <- c(0, 1)

  .resid_quasi_finalize(
    x         = x.input,
    y         = y_curve,
    main      = "Quasi, Ordinal",
    xlim      = xlim_vec,
    line_args = line_args,
    ...
  )
}

###########
## zpois ##
###########
marginzerop <- function(u, y, pzero, meanpoisson,h) {
  n <- length(y)
  inv1 <- ifelse(u < (pzero + (1 - pzero) * (ppois(0, lambda = meanpoisson))), 0,
                 qpois(pmax((u - pzero) / (1 - pzero), 0), lambda = meanpoisson)
  )
  inv1m <- ifelse(inv1 > 0, inv1 - 1, 0)
  n1 <- cbind(inv1, inv1m)
  p1 <- cbind(
    (pzero + (1 - pzero) * (ppois(inv1, meanpoisson))),
    (pzero + (1 - pzero) * (ppois(inv1m, meanpoisson)))
  )
  ind1 <- apply(abs(p1 - u), 1, which.min)
  wei <- 1 * ((p1[cbind(1:n, ind1)] - u)^2 < 5 * h^2) *
    (1 - ((p1[cbind(1:n, ind1)] - u)^2) / h^2 / 5)
  l <- sum(wei * 1 * (y <= n1[cbind(1:n, ind1)])) / sum(wei)
  l
}
marginzerop <- Vectorize(marginzerop, "u")

### Bandwidth selection
bandwidth0p <- function(y, pzero, meanpoisson) {
  n <- length(y)
  newout <- unlist(sapply(split(cbind(qpois(0.1, meanpoisson), qpois(0.9, meanpoisson)), 1:n), listvec))
  newlambda <- rep(meanpoisson, times = qpois(0.9, meanpoisson) - qpois(0.1, meanpoisson) + 1)
  newzero <- rep(pzero, times = qpois(0.9, meanpoisson) - qpois(0.1, meanpoisson) + 1)
  newx <- newzero + (1 - newzero) * ppois(newout, newlambda)
  newy <- 1 * (rep(y, times = qpois(0.9, meanpoisson) - qpois(0.1, meanpoisson) + 1) <= newout)
  return(npregbw(ydat = newy[which(newx <= 0.9 & newx >= 0.1)], xdat = newx[which(newx <= 0.9 & newx > 0.1)], ckertype = "epanechnikov")$bw)
}

#' @keywords internal
resid.zpois_quasi <- function(model, line_args, ...){
  y           <- model$model[,1]
  meanpoisson <- predict(model, type = "count")
  pzero       <- predict(model, type = "zero")

  h       <- bandwidth0p(y = y, pzero = pzero, meanpoisson = meanpoisson)
  x.input <- seq(0, 1, length.out = 101)

  y_curve <- marginzerop(
    u           = x.input,
    y           = y,
    pzero       = pzero,
    meanpoisson = meanpoisson,
    h           = h
  )

  xlim_vec <- c(
    min(pzero + (1 - pzero) * ppois(0, meanpoisson)),
    1
  )

  .resid_quasi_finalize(
    x         = x.input,
    y         = y_curve,
    main      = "Quasi, 0-Inflated Poisson",
    xlim      = xlim_vec,
    line_args = line_args,
    ...
  )
}


###########
##  znb  ##
###########
marginzerop.nb <- function(u, y, pzero, size1f, mu.hat, h) {
  n <- length(y)
  inv1 <- ifelse(u < (pzero + (1 - pzero) * (pnbinom(0, size = size1f, mu = mu.hat))), 0,
                 qnbinom(pmax((u - pzero) / (1 - pzero), 0), mu = mu.hat, size = size1f)
  )
  inv1m <- ifelse(inv1 > 0, inv1 - 1, 0)
  n1 <- cbind(inv1, inv1m)
  p1 <- cbind(
    (pzero + (1 - pzero) * (pnbinom(inv1, size=size1f, mu=mu.hat))),
    (pzero + (1 - pzero) * (pnbinom(inv1m, size=size1f, mu=mu.hat)))
  )
  ind1 <- apply(abs(p1 - u), 1, which.min)
  wei <- 1 * ((p1[cbind(1:n, ind1)] - u)^2 < 5 * h^2) *
    (1 - ((p1[cbind(1:n, ind1)] - u)^2) / h^2 / 5)
  l <- sum(wei * 1 * (y <= n1[cbind(1:n, ind1)])) / sum(wei)
  l
}

marginzerop.nb <- Vectorize(marginzerop.nb, "u")

### Bandwidth selection
bandwidth0p.nb <- function(y, pzero, mu.hat, size1f) {
  n <- length(y)
  newout <- unlist(sapply(split(cbind(qnbinom(0.1, mu=mu.hat, size=size1f), qnbinom(0.9, mu=mu.hat, size=size1f)), 1:n), listvec))
  newlambda <- rep(mu.hat, times = qnbinom(0.9, mu=mu.hat, size=size1f) - qnbinom(0.1, mu=mu.hat, size=size1f) + 1)
  newzero <- rep(pzero, times = qnbinom(0.9, mu=mu.hat, size=size1f) - qnbinom(0.1, mu=mu.hat, size=size1f) + 1)
  newx <- newzero + (1 - newzero) * ppois(newout, newlambda)
  newy <- 1 * (rep(y, times = qnbinom(0.9, mu=mu.hat, size=size1f) - qnbinom(0.1, mu=mu.hat, size=size1f) + 1) <= newout)

  return(npregbw(ydat = newy[which(newx <= 0.9 & newx >= 0.1)], xdat = newx[which(newx <= 0.9 & newx > 0.1)], ckertype = "epanechnikov")$bw)
}

#' @keywords internal
resid.znb_quasi <- function(model, line_args, ...){
  y      <- model$model[,1]
  mu.hat <- predict(model, type = "count")
  size1f <- model$theta
  pzero  <- predict(model, type = "zero")

  h       <- bandwidth0p.nb(y = y, pzero = pzero, mu.hat = mu.hat, size1f = size1f)
  x.input <- seq(0,1, length.out=101)

  y_curve <- marginzerop.nb(
    u      = x.input,
    y      = y,
    pzero  = pzero,
    size1f = size1f,
    mu.hat = mu.hat,
    h      = h
  )

  xlim_vec <- c(
    min(pzero + (1 - pzero) * pnbinom(0, size = size1f, mu = mu.hat)),
    1
  )

  .resid_quasi_finalize(
    x         = x.input,
    y         = y_curve,
    main      = "Quasi, 0-Inflated NB",
    xlim      = xlim_vec,
    line_args = line_args,
    ...
  )
}

#' @title Healthcare expenditure data
#' @description
#' Healthcare expenditure data set.
#' @source http://www.meps.ahrq.gov/mepsweb/
#' @format A data frame with 29784 rows and 29 variables:
#' \describe{
#'   \item{\code{EXP}}{the aggregate annual office based expenditure per participants, semicontinuous outcomes}
#'   \item{\code{AGE}}{Age}
#'   \item{\code{GENDER}}{1 if female}
#'   \item{\code{ASIAN}}{1 if Asian}
#'   \item{\code{BLACK}}{1 if Black}
#'   \item{\code{NORTHEAST}}{1 if Northeast}
#'   \item{\code{MIDWEST}}{1 if Midwest}
#'   \item{\code{SOUTH}}{1 if South}
#'   \item{\code{USC}}{1 if have usual source of care}
#'   \item{\code{COLLEGE}}{1 if colleage or higher degrees}
#'   \item{\code{HIGHSCH}}{1 if high school degree}
#'   \item{\code{MARRIED}}{1 if married}
#'   \item{\code{WIDIVSEP}}{1 if widowed or divorced or separated}
#'   \item{\code{FAMSIZE}}{Family Size}
#'   \item{\code{HINCOME}}{1 if high income}
#'   \item{\code{MINCOME}}{1 if middle income}
#'   \item{\code{LINCOME}}{1 if low income}
#'   \item{\code{NPOOR}}{1 if near poor}
#'   \item{\code{POOR}}{1 if poor}
#'   \item{\code{FAIR}}{1 if fair}
#'   \item{\code{GOOD}}{1 if good}
#'   \item{\code{VGOOD}}{1 if very good}
#'   \item{\code{MNHPOOR}}{1 if poor or fair mental health}
#'   \item{\code{ANYLIMIT}}{1 if any functional or activity limitation}
#'   \item{\code{unemployed}}{1 if unemployed at the beginning of 2006}
#'   \item{\code{EDUCHEALTH}}{1 if education, health and social services}
#'   \item{\code{PUBADMIN}}{1 if public administration}
#'   \item{\code{insured}}{1 if is insured at the beginning of the year 2006}
#'   \item{\code{MANAGEDCARE}}{if enrolled in an HMO or a gatekeeper plan}
#'}
"MEPS"

#' @rawNamespace S3method(dpit,zeroinfl)
dpit.zeroinfl <- function(model,
                          plot = TRUE,
                          scale = "normal",
                          line_args = list(),
                          ...) {
  res_u <- .dpit_zeroinfl(model, ...)
  .dpit_finalize(res_u, plot = plot, scale = scale, line_args = line_args, ...)
}

.dpit_zeroinfl <- function(model, ...) {
  key <- list(model = model)
  dist <- NULL
  if (!is.null(model$dist)) {
    dist <- model$dist
  } else if (!is.null(model$call$dist)) {
    dist <- eval(model$call$dist, parent.frame())
  } else if (!is.null(model$call$distribution)) {
    dist <- eval(model$call$distribution, parent.frame())
  } else if (!is.null(model$call$family)) {
    dist <- eval(model$call$family, parent.frame())
  }

  dist_chr <- if (length(dist) == 1L && is.character(dist)) tolower(dist) else NA_character_

  class(key) <- if (!is.na(dist_chr) && identical(dist_chr, "poisson")) {
    "zeroinfl_poisson"
  } else if (!is.na(dist_chr) && dist_chr %in% c("negbin", "negbinom", "negative binomial", "nbinom", "nb")) {
    "zeroinfl_negbin"
  } else {
    "zeroinfl_default"
  }

  .dpit_zeroinfl_key(key, ...)
}

.dpit_zeroinfl_key <- function(key, ...) {
  UseMethod(".dpit_zeroinfl_key")
}

#' @rawNamespace S3method(.dpit_zeroinfl_key,zeroinfl_poisson)
.dpit_zeroinfl_key.zeroinfl_poisson <- function(key, ...) {
  model <- key$model

  # response y
  mf <- try(stats::model.frame(model), silent = TRUE)
  if (inherits(mf, "try-error")) {
    # fallback: older objects may store model matrix in model$model
    if (!is.null(model$model)) {
      y <- model$model[, 1]
    } else {
      stop("Cannot extract response y from zeroinfl model.", call. = FALSE)
    }
  } else {
    y <- stats::model.response(mf)
  }

  mu    <- stats::predict(model, type = "count")
  pzero <- stats::predict(model, type = "zero")

  dpit_zpois(y = y, pzero = pzero, mu = mu)
}

#' @rawNamespace S3method(.dpit_zeroinfl_key,zeroinfl_negbin)
.dpit_zeroinfl_key.zeroinfl_negbin <- function(key, ...) {
  model <- key$model
  mf <- try(stats::model.frame(model), silent = TRUE)
  if (inherits(mf, "try-error")) {
    if (!is.null(model$model)) {
      y <- model$model[, 1]
    } else {
      stop("Cannot extract response y from zeroinfl model.", call. = FALSE)
    }
  } else {
    y <- stats::model.response(mf)
  }

  mu    <- stats::predict(model, type = "count")
  pzero <- stats::predict(model, type = "zero")
  size <- model$theta
  if (is.null(size)) {
    stop("Cannot extract theta/size from zeroinfl(negbin) model (model$theta is NULL).", call. = FALSE)
  }
  dpit_znb(y = y, pzero = pzero, mu = mu, size = size)
}

#' @rawNamespace S3method(.dpit_zeroinfl_key,zeroinfl_default)
.dpit_zeroinfl_key.zeroinfl_default <- function(key, ...) {
  model <- key$model
  dist <- if (!is.null(model$dist)) model$dist else NULL
  msg <- if (is.null(dist)) {
    "Unsupported zeroinfl distribution for dpit() (expected poisson or negbin)."
  } else {
    sprintf("Unsupported zeroinfl distribution for dpit(): %s (expected poisson or negbin).", as.character(dist))
  }
  stop(msg, call. = FALSE)
}

#' Residuals for regression models with tweedie outcomes
#'
#' Computes DPIT residuals for Tweedie-distributed outcomes using the observed responses (\code{y}),
#' their fitted mean values (\code{mu}), the variance power parameter
#' (\eqn{\xi}), and the dispersion parameter (\eqn{\phi}).
#'
#' @usage dpit_tweedie(y, mu, xi, phi, plot=TRUE, scale="normal", line_args=list(), ...)
#' @param y Observed outcome vector.
#' @param mu Vector of fitted mean values of each outcomes.
#' @param xi Value of \eqn{\xi} such that the variance is \eqn{Var[Y] = \phi\mu^\xi}
#' @param phi Dispersion parameter \eqn{\phi}.
#' @param plot A logical value indicating whether or not to return QQ-plot
#' The sample quantiles of the residuals are plotted against
#' @param scale You can choose the scale of the residuals among `normal` and `uniform`.
#' the theoretical quantiles of a standard normal distribution under the normal scale,
#' and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
#'  The default scale is `normal`.
#' @param line_args A named list of graphical parameters passed to
#'   \code{graphics::abline()} to modify the reference (red) 45° line
#'   in the QQ plot. If left empty, a default red dashed line is drawn.
#' @param ... Additional graphical arguments passed to
#'   \code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
#'   \code{col}, \code{cex}, \code{xlab}, \code{ylab}).
#'
#' @returns DPIT residuals.
#'
#' @details
#' For formulation details on semicontinuous outcomes, see \code{\link{dpit}}.
#' @import tweedie
#'
#'
#' @examples
#' ## Tweedie model
#' library(tweedie)
#' library(statmod)
#' n <- 300
#' x11 <- rnorm(n)
#' x12 <- rnorm(n)
#' beta0 <- 5
#' beta1 <- 1
#' beta2 <- 1
#' lambda1 <- exp(beta0 + beta1 * x11 + beta2 * x12)
#' y1 <- rtweedie(n, mu = lambda1, xi = 1.6, phi = 10)
#' # Choose parameter p
#' # True model
#' model1 <-
#'   glm(y1 ~ x11 + x12,
#'     family = tweedie(var.power = 1.6, link.power = 0)
#'   )
#' y1 <- model1$y
#' p.max <- get("p", envir = environment(model1$family$variance))
#' lambda1f <- model1$fitted.values
#' phi1f <- summary(model1)$dis
#' resid.tweedie <- dpit_tweedie(y= y1, mu=lambda1f, xi=p.max, phi=phi1f)
#' @export
dpit_tweedie <- function(y, mu, xi, phi, plot=TRUE, scale="normal", line_args=list(), ...){
  n <- length(y)
  p.max <- xi
  lambda1f <- mu
  phi1f <- phi
  p1f <- dtweedie(rep(0, n), mu = lambda1f, xi = p.max, phi = phi1f)
  cdf1 <- ptweedie(y, mu = lambda1f, xi = p.max, phi = phi1f)
  func <- ecdf(p1f)
  newp <- cdf1 * func(cdf1)
  .dpit_finalize(newp, plot=plot, scale=scale, line_args=line_args, ...)
}

#' DPIT residuals for regression models with various non-continuous outcomes
#'
#' Calculates DPIT residuals for regression models with non-continuous outcomes.
#' In particular, model assumptions for GLMs with discrete outcomes (e.g., binary, Poisson, and negative binomial), ordinal
#' regression models, zero-inflated regression models, and semicontinuous outcome
#' models can be assessed using \code{dpit()}.
#'
#' @usage dpit(model, plot=TRUE, scale="normal", line_args=list(), ...)
#' @param model A model object.
#' @param plot A logical value indicating whether or not to return QQ-plot
#' @param scale You can choose the scale of the residuals among `normal` and `uniform`.
#' The sample quantiles of the residuals are plotted against
#' the theoretical quantiles of a standard normal distribution under the normal scale,
#' and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
#'  The default scale is `normal`.
#' @param line_args A named list of graphical parameters passed to
#'   \code{graphics::abline()} to modify the reference (red) 45° line
#'   in the QQ plot. If left empty, a default red dashed line is drawn.
#' @param ... Additional graphical arguments passed to
#'   \code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
#'   \code{col}, \code{cex}, \code{xlab}, \code{ylab}).
#'
#' @details
#' This function determines the appropriate computation based on the class of
#' \code{model}. The supported model objects and outcome types are listed below.
#'
#' In addition to the class-based interface, the package also provides
#' distribution-specific DPIT calculators. If a fitted model comes from a
#' different class but has a supported outcome distribution, users can call
#' the corresponding distribution-based function directly.
#' For instance, for a regression model with Poisson outcomes,
#' one can use dpit to calculate the residuals if
#' the model is fit using `glm` function, or to use `dpit_pois` upon supplying fitted mean values.
#'
#' \itemize{
#' \item \strong{Discrete outcomes}
#'   \itemize{
#'   \item \code{\link[stats]{glm}} with \code{family = binomial()} (see \code{\link{dpit_bin}}).
#'   \item \code{\link[stats]{glm}} with \code{family = poisson()}
#'         (see \code{\link{dpit_pois}}).
#'   \item \code{\link[MASS]{glm.nb}} for negative binomial regression
#'         (see \code{\link{dpit_nb}}).
#'   \item \code{\link[MASS]{polr}} for ordinal outcomes
#'         (see \code{\link{dpit_ordi}}).
#'   }
#'
#' \item \strong{Zero-inflated discrete outcomes}
#'   \itemize{
#'   \item \code{\link[pscl]{zeroinfl}} with \code{dist = "poisson"}
#'         (see \code{\link{dpit_zpois}}).
#'   \item \code{\link[pscl]{zeroinfl}} with \code{dist = "negbin"}
#'         (see \code{\link{dpit_znb}}).
#'   }
#'
#' \item \strong{Semicontinuous outcomes}
#'   \itemize{
#'   \item Tobit regression via \code{\link[AER]{tobit}} from `AER` or \code{\link[VGAM]{vglm}} from `VGAM`
#'         (see \code{\link{dpit_tobit}}).
#'   \item Tweedie regression via \code{\link[stats]{glm}} with a Tweedie family
#'         (see \code{\link{dpit_tweedie}}).
#'   }
#' }
#'
#' \strong{Formulation for Discrete and Zero-Inflated Outcomes:}
#' \cr
#' The DPIT residual for the \eqn{i}th observation is defined as follows:
#' \deqn{\hat{r}(Y_i|X_i) = \hat{G}\bigg(\hat{F}(Y_i|\mathbf{X}_i)\bigg)}
#' where
#' \deqn{\hat{G}(s) = \frac{1}{n-1}\sum_{j=1, j \neq i}^{n}\hat{F}\bigg(\hat{F}^{(-1)}(\mathbf{X}_j)\bigg|\mathbf{X}_j\bigg)}
#' and \eqn{\hat{F}} refers to the fitted cumulative distribution function.
#' When `scale="uniform"`, DPIT residuals should closely follow a uniform distribution, otherwise it implies model deficiency.
#' When `scale="normal"`, it applies the normal quantile transformation to the DPIT residuals
#' \deqn{\Phi^{-1}\left[\hat{r}(Y_i|\mathbf{X}_i)\right],i=1,\ldots,n.} The null pattern is the standard normal distribution in this case.
#' \cr
#'
#' \strong{Formulation for Semicontinuous Outcomes:}
#' \cr
#' The DPIT residuals for regression models with semi-continuous outcomes are \deqn{\hat{r}_i=\frac{\hat{F}(Y_i|\mathbf{X}_i)}{n}\sum_{j=1}^n1\left(\hat{p}_0(\mathbf{X}_j)\leq \hat{F}(Y_i|\mathbf{X}_i)\right), i=1,\ldots,n,}
#' where \eqn{\hat{p}_0(\mathbf{X}_i)} is the fitted probability of zero, and \eqn{\hat{F}(\cdot|\mathbf{X}_i)} is the  fitted cumulative distribution function for the \eqn{i}th observation. Furthermore, \deqn{\hat{F}(y|\mathbf{x})=\hat{p}_0(\mathbf{x})+\left(1-\hat{p}_0(\mathbf{x})\right)\hat{G}(y|\mathbf{x})}
#' where \eqn{\hat{G}} is the fitted cumulative distribution for the positive data.
#'
#' @returns DPIT residuals. If `plot=TRUE`, also produces a QQ plot.
#'
#'
#' @import stats
#' @import graphics
#' @export
#'
#'
#' @references
#' L. Yang. Double probability integral transform residuals for regression models with discrete outcomes. Journal of Computational and Graphical Statistics, 33(3), pp.787–803, 2024. \cr
#' L. Yang. Diagnostics for regression models with semicontinuous outcomes. Biometrics, 80(1), ujae007, 2024.
#'
#' @examples
#' library(MASS)
#' n <- 500
#' set.seed(1234)
#' ## Negative Binomial example
#' # Covariates
#' x1 <- rnorm(n)
#' x2 <- rbinom(n, 1, 0.7)
#' ### Parameters
#' beta0 <- -2
#' beta1 <- 2
#' beta2 <- 1
#' size1 <- 2
#' lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
#' # generate outcomes
#' y <- rnbinom(n, mu = lambda1, size = size1)
#'
#' # True model
#' model1 <- glm.nb(y ~ x1 + x2)
#' resid.nb1 <- dpit(model1, plot = TRUE, scale = "uniform")
#'
#' # Overdispersion
#' model2 <- glm(y ~ x1 + x2, family = poisson(link = "log"))
#' resid.nb2 <- dpit(model2, plot = TRUE, scale = "normal")
#'
#' ## Binary example
#' n <- 500
#' set.seed(1234)
#' # Covariates
#' x1 <- rnorm(n, 1, 1)
#' x2 <- rbinom(n, 1, 0.7)
#' # Coefficients
#' beta0 <- -5
#' beta1 <- 2
#' beta2 <- 1
#' beta3 <- 3
#' q1 <- 1 / (1 + exp(beta0 + beta1 * x1 + beta2 * x2 + beta3 * x1 * x2))
#' y1 <- rbinom(n, size = 1, prob = 1 - q1)
#'
#' # True model
#' model01 <- glm(y1 ~ x1 * x2, family = binomial(link = "logit"))
#' resid.bin1 <- dpit(model01, plot = TRUE)
#'
#' # Missing covariates
#' model02 <- glm(y1 ~ x1, family = binomial(link = "logit"))
#' resid.bin2 <- dpit(model02, plot = TRUE)
#'
#' ## Poisson example
#' n <- 500
#' set.seed(1234)
#' # Covariates
#' x1 <- rnorm(n)
#' x2 <- rbinom(n, 1, 0.7)
#' # Coefficients
#' beta0 <- -2
#' beta1 <- 2
#' beta2 <- 1
#' lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
#' y <- rpois(n, lambda1)
#'
#' # True model
#' poismodel1 <- glm(y ~ x1 + x2, family = poisson(link = "log"))
#' resid.poi1 <- dpit(poismodel1, plot = TRUE)
#'
#' # Enlarge three outcomes
#' y <- rpois(n, lambda1) + c(rep(0, (n - 3)), c(10, 15, 20))
#' poismodel2 <- glm(y ~ x1 + x2, family = poisson(link = "log"))
#' resid.poi2 <- dpit(poismodel2, plot = TRUE)
#'
#' ## Ordinal example
#' n <- 500
#' set.seed(1234)
#' # Covariates
#' x1 <- rnorm(n, mean = 2)
#' # Coefficient
#' beta1 <- 3
#'
#' # True model
#' p0 <- plogis(1, location = beta1 * x1)
#' p1 <- plogis(4, location = beta1 * x1) - p0
#' p2 <- 1 - p0 - p1
#' genemult <- function(p) {
#'   rmultinom(1, size = 1, prob = c(p[1], p[2], p[3]))
#' }
#' test <- apply(cbind(p0, p1, p2), 1, genemult)
#' y1 <- rep(0, n)
#' y1[which(test[1, ] == 1)] <- 0
#' y1[which(test[2, ] == 1)] <- 1
#' y1[which(test[3, ] == 1)] <- 2
#' multimodel <- polr(as.factor(y1) ~ x1, method = "logistic")
#' resid.ord1 <- dpit(multimodel, plot = TRUE)
#'
#' ## Non-Proportionality
#' n <- 500
#' set.seed(1234)
#' x1 <- rnorm(n, mean = 2)
#' beta1 <- 3
#' beta2 <- 1
#' p0 <- plogis(1, location = beta1 * x1)
#' p1 <- plogis(4, location = beta2 * x1) - p0
#' p2 <- 1 - p0 - p1
#' genemult <- function(p) {
#'   rmultinom(1, size = 1, prob = c(p[1], p[2], p[3]))
#' }
#' test <- apply(cbind(p0, p1, p2), 1, genemult)
#' y1 <- rep(0, n)
#' y1[which(test[1, ] == 1)] <- 0
#' y1[which(test[2, ] == 1)] <- 1
#' y1[which(test[3, ] == 1)] <- 2
#' multimodel <- polr(as.factor(y1) ~ x1, method = "logistic")
#' resid.ord2 <- dpit(multimodel, plot = TRUE)
dpit <- function(model,
                 plot = TRUE,
                 scale = "normal",
                 line_args = list(),
                 ...) {
  UseMethod("dpit")
}

#' Ordered curve for assessing mean structures
#'
#' Creates a plot to assess the mean structure of regression models. The
#' plot compares the cumulative sum of the response variable and its hypothesized value.
#' Deviation from the diagonal suggests the possibility that the mean structure of the model is incorrect.
#'
#'
#' @details
#' The ordered curve plots \deqn{\hat{L}_1(t)=\frac{\sum_{i=1}^n\left[Y_i1(Z_i\leq t)\right]}{\sum_{i=1}^nY_i}} against
#'  \deqn{\hat{L}_2(t)=\frac{\sum_{i=1}^n\left[\hat{\lambda}_i1(Z_i\leq t)\right]}{\sum_{i=1}^n\hat{\lambda}_i},}
#' where \eqn{\hat{\lambda}_i} is the fitted mean, and \eqn{Z_i} is the threshold variable. \cr
#'
#'  If the mean structure is correctly specified in the model,
#' \eqn{\hat L_1(t)} and \eqn{\hat L_2(t)} should be close to each other.
#'
#' If the curve is distant from the diagonal, it suggests incorrectness in the mean structure.
#' Moreover, if the curve is above the diagonal, the summation of the response is larger than
#' the fitted mean, which implies that the mean is underestimated, and vice versa. \cr
#'
#' The role of `thr` (threshold variable \eqn{Z}) is to determine the rule  for accumulating \eqn{\hat{\lambda}_i} and \eqn{Y_i}, \eqn{i=1,\ldots,n}
#' for the ordered curve.
#' The candidate for `thr` could be any function of predictors such as a single predictor (e.g., `x1`),
#' a linear combination of predictor (e.g., `x1+x2`), or fitted values (e.g., `fitted(model)`).
#' It can also be a variable being considered to be included in the mean function.
#' If a variable  leads to a large discrepancy between the ordered curve and the diagonal,
#'  including this variable in the mean function should be considered.
#'
#' For more details, see the reference paper.
#'
#' @references Yang, Lu. "Double Probability Integral Transform Residuals for Regression Models with Discrete Outcomes." arXiv preprint arXiv:2308.15596 (2023).
#'
#' @usage ord_curve(model, thr, line_args=list(), ...)
#'
#' @param model Regression model object (e.g.,`lm`, `glm`, `glm.nb`, `polr`, `lm`)
#' @param thr Threshold variable (e.g., predictor, fitted values, or variable to be included as a covariate)
#' @param line_args A named list of graphical parameters passed to
#'   \code{graphics::abline()} to modify the reference (red) 45° line
#'   in the QQ plot. If left empty, a default red dashed line is drawn.
#' @param ... Additional graphical arguments passed to
#'   \code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
#'   \code{col}, \code{cex}, \code{xlab}, \code{ylab}).
#'
#' @returns
#' \itemize{
#'  \item x-axis: \eqn{\hat L_1(t)}
#'  \item y-axis: \eqn{\hat L_2(t)}
#' }
#' which are defined in Details.
#'
#' @importFrom graphics abline
#'
#' @examples
#' ## Binary example of ordered curve
#' n <- 500
#' set.seed(1234)
#' x1 <- rnorm(n, 1, 1)
#' x2 <- rbinom(n, 1, 0.7)
#' beta0 <- -5
#' beta1 <- 2
#' beta2 <- 1
#' beta3 <- 3
#' q1 <- 1 / (1 + exp(beta0 + beta1 * x1 + beta2 * x2 + beta3 * x1 * x2))
#' y1 <- rbinom(n, size = 1, prob = 1 - q1)
#'
#' ## True Model
#' model0 <- glm(y1 ~ x1 * x2, family = binomial(link = "logit"))
#' ord_curve(model0, thr = model0$fitted.values) # set the threshold as fitted values
#'
#' ## Missing a covariate
#' model1 <- glm(y1 ~ x1, family = binomial(link = "logit"))
#' ord_curve(model1, thr = x2) # set the threshold as a covariate
#'
#' ## Poisson example of ordered curve
#' n <- 500
#' set.seed(1234)
#' x1 <- rnorm(n)
#' x2 <- rnorm(n)
#' beta0 <- 0
#' beta1 <- 2
#' beta2 <- 1
#' lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
#'
#' y <- rpois(n, lambda1)
#'
#' ## True Model
#' poismodel1 <- glm(y ~ x1 + x2, family = poisson(link = "log"))
#' ord_curve(poismodel1, thr = poismodel1$fitted.values)
#'
#' ## Missing a covariate
#' poismodel2 <- glm(y ~ x1, family = poisson(link = "log"))
#' ord_curve(poismodel2, thr = poismodel2$fitted.values)
#' ord_curve(poismodel2, thr = x2)
#'
#' @export
ord_curve <- function(model, thr, line_args = list(), ...) {
  UseMethod("ord_curve")
}

#' @rawNamespace S3method(ord_curve,glm)
ord_curve.glm <- function(model, thr, line_args = list(), ...) {
  y1 <- model$y
  .ord_curve_core(y1 = y1, q10 = stats::fitted.values(model), thr = thr, line_args = line_args, ...)
}

#' @rawNamespace S3method(ord_curve,negbin)
ord_curve.negbin <- function(model, thr, line_args = list(), ...) {
  y1 <- model$y
  .ord_curve_core(y1 = y1, q10 = stats::fitted.values(model), thr = thr, line_args = line_args, ...)
}

#' @rawNamespace S3method(ord_curve,polr)
ord_curve.polr <- function(model, thr, line_args = list(), ...) {
  y1 <- model$y
  .ord_curve_core(y1 = y1, q10 = stats::fitted.values(model), thr = thr, line_args = line_args, ...)
}

#' @rawNamespace S3method(ord_curve,lm)
ord_curve.lm <- function(model, thr, line_args = list(), ...) {
  y1 <- model$model[, 1]
  .ord_curve_core(y1 = y1, q10 = stats::fitted.values(model), thr = thr, line_args = line_args, ...)
}

#' @rawNamespace S3method(ord_curve,default)
ord_curve.default <- function(model, thr, line_args = list(), ...) {
  cls <- paste(class(model), collapse = ", ")
  stop(
    sprintf("Unsupported model class for ord_curve(): %s. Supported: glm, negbin, polr, lm.", cls),
    call. = FALSE
  )
}


.ord_curve_core <- function(y1, q10, thr, line_args = list(), ...) {
  if (length(thr) != length(y1)) {
    stop("Length of thr and response has to match.", call. = FALSE)
  }

  ord <- order(thr)

  plot_defaults <- list(
    main = paste("Z:", deparse(substitute(thr))),
    xlab = expression(L[2](t)),
    ylab = expression(L[1](t)),
    cex.lab = 1,
    cex.axis = 1,
    cex.main = 1.5,
    lwd = 2,
    type = "l",
    ylim = c(0, 1),
    xlim = c(0, 1)
  )

  plot_args <- utils::modifyList(plot_defaults, list(...))

  do.call(graphics::plot, c(
    list(
      x = cumsum(q10[ord]) / sum(q10),
      y = cumsum(y1[ord]) / sum(y1)
    ),
    plot_args
  ))

  line_defaults <- list(a = 0, b = 1, col = "red", lty = 5, lwd = 2)
  ab_args <- utils::modifyList(line_defaults, line_args)
  do.call(graphics::abline, ab_args)
  invisible(NULL)
}

#' Residuals for regression models with binary outcomes
#'
#'
#' Computes DPIT residuals for regression models with binary outcomes
#' using the observed responses (`y`) and their fitted distributional parameters(`prob`).
#'
#' @usage dpit_bin(y, prob, plot=TRUE, scale="normal", line_args=list(), ...)
#' @param y An observed outcome vector.
#' @param prob A vector of fitted probabilities of one.
#' @param plot A logical value indicating whether or not to return QQ-plot
#' @param scale You can choose the scale of the residuals among `normal` and `uniform`.
#' The sample quantiles of the residuals are plotted against
#' the theoretical quantiles of a standard normal distribution under the normal scale,
#' and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
#'  The default scale is `normal`.
#' @param line_args A named list of graphical parameters passed to
#'   \code{graphics::abline()} to modify the reference (red) 45° line
#'   in the QQ plot. If left empty, a default red dashed line is drawn.
#' @param ... Additional graphical arguments passed to
#'   \code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
#'   \code{col}, \code{cex}, \code{xlab}, \code{ylab}).
#'
#' @returns DPIT residuals.
#'
#' @details
#' For formulation details on discrete outcomes, see \code{\link{dpit_pois}}.
#'
#' @examples
#' ## Binary example
#' n <- 500
#' set.seed(1234)
#' # Covariates
#' x1 <- rnorm(n, 1, 1)
#' x2 <- rbinom(n, 1, 0.7)
#' # Coefficients
#' beta0 <- -5
#' beta1 <- 2
#' beta2 <- 1
#' beta3 <- 3
#' q1 <- 1 / (1 + exp(beta0 + beta1 * x1 + beta2 * x2 + beta3 * x1 * x2))
#' y1 <- rbinom(n, size = 1, prob = 1 - q1)
#'
#' # True model
#' model01 <- glm(y1 ~ x1 * x2, family = binomial(link = "logit"))
#' fitted1 <- fitted(model01)
#' y1 <- model01$y
#' resid.bin1 <- dpit_bin(y=y1, prob=fitted1)
#'
#' # Missing covariates
#' model02 <- glm(y1 ~ x1, family = binomial(link = "logit"))
#' y2 <- model02$y
#' fitted2 <- fitted(model02)
#' resid.bin2 <- dpit_bin(y=y2, prob=fitted2)
#' @export
dpit_bin <- function(y, prob,
                     plot=TRUE,
                     scale="normal",
                     line_args=list(), ...) {
  n <- length(y)
  prob <- 1-prob
  # fitted.values
  res <- ifelse(y == 0, prob, 1)
  ses <- ifelse(y == 1, prob, 0)
  # residuals
  empcdf <- rep(NA,n)
  for(i in 1:n){
    if(i %in% which(y==1)) next
    qres <- 1*(res[i] >= prob)*(res[i] < 1)*0 + 1 * (res[i]<prob) * (res[i]<1)*(-1)+1*(res[i]==1)
    pres <- ifelse(qres ==0, prob, ifelse(qres == 1, 1, 0))
    pres[i] <- 0
    empcdf[i] <- sum(pres)/ (n-1)
  }

  for(i in 1:n){
    if(i %in% which(y==0)) next
    qses <- (1*(ses[i]==0)*(-2)+1*(ses[i]>=prob)*(ses[i]<1)*0+1*(ses[i] < prob)*(ses[i]<1)*(-1)+1*(ses[i]==1))+1
    pses <- ifelse(qses ==0, prob, ifelse(qses == 1, 1, 0))
    pses[i] <- 0
    empcdf[i] <- sum(pses)/ (n-1)
  }
  .dpit_finalize(empcdf, plot=plot, scale=scale, line_args = line_args,...)
}

#' Goodness-of-fit test for discrete outcome regression models
#'
#' Goodness-of-fit test for discrete-outcome regression models.
#' Works with GLMs (Poisson, binomial/logistic, negative binomial),
#' ordinal outcome regression (`MASS::polr`), and
#' zero-inflated regressions (zero-inflated Poisson and negative binomial via `pscl::zeroinfl()`).
#'
#' @usage gof_disc(model, B=1e2, seed=NULL)
#' @param model A fitted model object (e.g., from `glm()`, `polr()`, `glm.nb()` or `zeroinfl()`).
#' @param B Number of bootstrap samples. Default is 1e2.
#' @param seed random seed for bootstrap.
#'
#' @details
#' Let \eqn{(Y_i,\mathbf{X}_i),\ i=1,\ldots,n} denote independent observations, and let
#' \eqn{\hat F_M(\cdot \mid \mathbf{X}_i)} be the fitted model-based CDF.
#' It was shown in \emph{Yang (2025)} that under the correctly specified model,
#' \deqn{\hat{H}(u) = \frac{1}{n}\sum_{i=1}^n \hat{h}(u, Y_i, \mathbf{X}_i)}
#' should be close to the identity function, where
#' \deqn{\hat{h}(u, y, \mathbf{x}) =
#' \frac{u - \hat{F}_M (y-1 \mid \mathbf{x})}
#' {\hat{F}_M (y \mid \mathbf{x}) - \hat{F}_M (y-1 \mid \mathbf{x})}
#' \,\mathbf{1}\{ \hat{F}_M (y-1 \mid \mathbf{x}) < u < \hat{F}_M (y \mid \mathbf{x}) \}
#' + \mathbf{1}\{ u \ge \hat{F}_M (y \mid \mathbf{x}) \}.}
#' The test statistic
#' \deqn{S_n = \int_0^1 \{ \hat{H}(u) - u \}^2 du} measures the deviation of \eqn{\hat{h}(u,y,\mathbf{x})} from the
#' identity function, with p-values obtained by bootstrap. This method
#' complements residual-based diagnostics by providing a formal check of model adequacy.
#' @references Yang L, Genest C, Neslehova J (2025). “A goodness-of-fit test for regression models with discrete outcomes.” Canadian Journal of Statistics
#'
#' @returns Test statistics and p-values
#' @importFrom stats family
#' @import tweedie
#' @export
#' @examples
#' library(MASS)
#' library(pscl)
#' n <- 100
#' beta1 <- 1;  beta2 <- 1
#' beta0 <- -2; beta00 <- -2; beta10 <- 2
#' size1 <- 2
#' set.seed(1)
#' x1 <- rnorm(n)
#' x2 <- rbinom(n,1,0.7)
#' lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
#' p0 <- 1 / (1 + exp(-(beta00 + beta10 * x1)))
#' y0 <- rbinom(n, size = 1, prob = 1 - p0)
#' y1 <- rnegbin(n, mu=lambda1, theta=size1)
#' y <- ifelse(y0 == 0, 0, y1)
#' model1 <- zeroinfl(y ~ x1 + x2 | x1, dist = "negbin", link = "logit")
#' gof_disc(model1, B=50)
gof_disc <- function(model, B = 1e2, seed = NULL) UseMethod("gof_disc")

#' @rawNamespace S3method(gof_disc,default)
gof_disc.default <- function(model, B = 1e2, seed = NULL) {
  cls <- paste(class(model), collapse = ", ")
  stop(
    sprintf(
      "Unsupported model class for gof_disc(): %s. Supported classes include glm, zeroinfl, and polr.",
      cls
    ),
    call. = FALSE
  )
}

#' @rawNamespace S3method(gof_disc,glm)
gof_disc.glm <- function(model, B = 1e2, seed = NULL) {
  .gof_glm(model, B = B, seed = seed)
}

.gof_glm <- function(model, B = 1e2, seed = NULL) {
  fam_obj <- stats::family(model)
  fam_raw  <- fam_obj$family
  link_raw <- fam_obj$link
  fam_key <- fam_raw
  if (startsWith(fam_raw, "Negative Binomial")) {
    fam_key <- "negbin"
  }
  key <- list(
    model  = model,
    family = fam_raw,
    link   = link_raw,
    B      = B,
    seed   = seed
  )
  class(key) <- fam_key
  .gof_glm_key(key)
}

.gof_glm_key <- function(key) {
  UseMethod(".gof_glm_key")
}


#' @rawNamespace S3method(.gof_glm_key,default)
.gof_glm_key.default <- function(key) {
  stop(
    sprintf(
      "Unsupported GLM family/link for gof_disc(): %s / %s",
      key$family, key$link
    ),
    call. = FALSE
  )
}

#' @rawNamespace S3method(.gof_glm_key, poisson)
.gof_glm_key.poisson <- function(key) {
  if (!exists("gof_pois", mode = "function", inherits = TRUE)) {
    stop("gof_pois() not found.", call. = FALSE)
  }
  gof_pois(B = key$B, poismodel = key$model, seed = key$seed)
}

#' @rawNamespace S3method(.gof_glm_key,binomial)
.gof_glm_key.binomial <- function(key) {
  if (!exists("gof_bin", mode = "function", inherits = TRUE)) {
    stop("gof_bin() not found.", call. = FALSE)
  }
  gof_bin(B = key$B, bimodel = key$model, seed = key$seed)
}

#' @rawNamespace S3method(.gof_glm_key,negbin)
.gof_glm_key.negbin <- function(key) {
  if (!exists("gof_nb", mode = "function", inherits = TRUE)) {
    stop("gof_nb() not found.", call. = FALSE)
  }
  gof_nb(B = key$B, nbmodel = key$model, seed = key$seed)
}

#' @rawNamespace S3method(gof_disc,zeroinfl)
gof_disc.zeroinfl <- function(model, B = 1e2, seed = NULL) {
  .gof_zeroinfl(model, B = B, seed = seed)
}

.gof_zeroinfl <- function(model, B = 1e2, seed = NULL) {
  dist_raw <- model$dist

  key <- list(
    model = model,
    dist  = dist_raw,
    B     = B,
    seed  = seed
  )

  class(key) <- dist_raw
  .gof_zeroinfl_key(key)
}

.gof_zeroinfl_key <- function(key) {
  UseMethod(".gof_zeroinfl_key")
}

#' @rawNamespace S3method(.gof_zeroinfl_key,poisson)
.gof_zeroinfl_key.poisson <- function(key) {
  if (!exists("gof_zpois", mode = "function", inherits = TRUE)) {
    stop("gof_zpois() not found.", call. = FALSE)
  }
  gof_zpois(B = key$B, model1 = key$model, seed = key$seed)
}

#' @rawNamespace S3method(.gof_zeroinfl_key,negbin)
.gof_zeroinfl_key.negbin <- function(key) {
  if (!exists("gof_znb", mode = "function", inherits = TRUE)) {
    stop("gof_znb() not found.", call. = FALSE)
  }
  gof_znb(B = key$B, model1 = key$model, seed = key$seed)
}

#' @rawNamespace S3method(.gof_zeroinfl_key,default)
.gof_zeroinfl_key.default <- function(key) {
  stop(
    sprintf("Unsupported zeroinfl dist for gof_disc(): %s", key$dist),
    call. = FALSE
  )
}

#' @rawNamespace S3method(gof_disc,polr)
gof_disc.polr <- function(model, B = 1e2, seed = NULL) {
  if (!exists("gof_ordi", mode = "function", inherits = TRUE)) {
    stop("gof_ordi() not found.", call. = FALSE)
  }
  gof_ordi(B = B, multimodel = model, seed = seed)
}

#' MLB Players' Home Run and Batted Ball Statistics with Red Zone Metrics (2017-2019)
#'
#' This dataset provides annual statistics for Major League Baseball (MLB) players, including home run counts, at-bats, mean exit velocities, launch angles, quantile statistics of exit velocities and launch angles, and red zone metrics. It is intended for analyzing batted ball performance, with additional variables on the red zone, which is defined as balls in play with a launch angle between 20 and 35 degrees and an exit velocity of at least 95 mph.
#'
#' @format A data frame with the following columns:
#' \describe{
#'   \item{name}{Player's full name (character).}
#'   \item{playerID}{Player's unique identifier in the Lahman database (character).}
#'   \item{teamID}{Team abbreviation (character).}
#'
#'   \item{year}{Season year (numeric).}
#'   \item{HR}{Home runs hit during the season (integer).}
#'   \item{AB}{At-bats during the season (integer).}
#'   \item{mean_exit_velo}{Average exit velocity (mph) over the season (numeric).}
#'   \item{mean_launch_angle}{Average launch angle (degrees) over the season (numeric).}
#'   \item{launch_angle_75}{Launch angle at the 75th percentile of the player's distribution (numeric).}
#'   \item{launch_angle_70}{Launch angle at the 70th percentile of the player's distribution (numeric).}
#'   \item{launch_angle_65}{Launch angle at the 65th percentile of the player's distribution (numeric).}
#'   \item{exit_velo_75}{Exit velocity at the 75th percentile of the player's distribution (numeric).}
#'   \item{exit_velo_80}{Exit velocity at the 80th percentile of the player's distribution (numeric).}
#'   \item{exit_velo_85}{Exit velocity at the 85th percentile of the player's distribution (numeric).}
#'   \item{count_red_zone}{Seasonal count of batted balls in the red zone, defined as a launch angle between 20 and 35 degrees and an exit velocity greater than or equal to 95 mph (integer).}
#'   \item{prop_red_zone}{Proportion of batted balls that fall into the red zone (numeric).}
#'   \item{BPF}{Ballpark factor, indicating the effect of the player's home ballpark on offensive statistics (integer).}
#' }
#'
#' @details
#' \describe{
#'   \item{Mean Metrics}{\code{mean_exit_velo} and \code{mean_launch_angle} represent the player's average exit velocities and launch angles, respectively, over the course of a season.}
#'   \item{Quantile Metrics}{The \code{launch_angle_x} and \code{exit_velo_x} columns denote the upper \eqn{x}-percentiles (e.g., 75th percentile) of the player's launch angle and exit velocity distributions for that year.}
#'   \item{Red Zone Metrics}{\code{count_red_zone} gives the number of balls in play that fall into the red zone, while \code{prop_red_zone} represents the proportion of balls in play in this category.}
#'   \item{BPF}{The Ballpark Factor (BPF) quantifies the influence of the player's home ballpark on offensive performance, with values above 100 indicating a hitter-friendly environment.}
#' }
#'
#' @source
#' - Player statistics: [Lahman R Package](https://CRAN.R-project.org/package=Lahman)
#' - Batted ball data: [Baseball Savant](https://baseballsavant.mlb.com/)
#' - Additional analysis: [Patterns of Home Run Hitting in the Statcast Era](https://bayesball.github.io/BLOG/homeruns.html) by Jim Albert
#'
#' @examples
#' data(bballHR)
#' head(bballHR)
#'
"bballHR"

#' Residuals for regression models with ordinal outcomes
#'
#' Computes DPIT residuals for regression models with ordinal outcomes
#' using observed outcomes (`y`), ordinal outcome levels (`level`) and their fitted category
#' probabilities (`fitprob`).
#'
#'
#' @usage dpit_ordi(y, level, fitprob, plot=TRUE, scale="normal", line_args=list(), ...)
#' @param y An observed ordinal outcome vector.
#' @param level The names of the response levels. For instance, c(0,1,2).
#' @param fitprob A matrix of fitted category probabilities. Each row corresponds to an observation, and column j contains the fitted probability P(Y_i = j).
#' @param plot A logical value indicating whether or not to return QQ-plot
#' @param scale You can choose the scale of the residuals among `normal` and `uniform`.
#' The sample quantiles of the residuals are plotted against
#' the theoretical quantiles of a standard normal distribution under the normal scale,
#' and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
#'  The default scale is `normal`.
#' @param line_args A named list of graphical parameters passed to
#'   \code{graphics::abline()} to modify the reference (red) 45° line
#'   in the QQ plot. If left empty, a default red dashed line is drawn.
#' @param ... Additional graphical arguments passed to
#'   \code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
#'   \code{col}, \code{cex}, \code{xlab}, \code{ylab}).
#' @returns DPIT residuals.
#'
#' @details
#' For formulation details on discrete outcomes, see \code{\link{dpit}}.
#'
#' @examples
#' ## Ordinal example
#' library(MASS)
#' n <- 500
#' x1 <- rnorm(n, mean = 2)
#' beta1 <- 3
#' # True model
#' p0 <- plogis(1, location = beta1 * x1)
#' p1 <- plogis(4, location = beta1 * x1) - p0
#' p2 <- 1 - p0 - p1
#' genemult <- function(p) {
#'  rmultinom(1, size = 1, prob = c(p[1], p[2], p[3]))
#' }
#' test <- apply(cbind(p0, p1, p2), 1, genemult)
#' y1 <- rep(0, n)
#' y1[which(test[1, ] == 1)] <- 0
#' y1[which(test[2, ] == 1)] <- 1
#' y1[which(test[3, ] == 1)] <- 2
#' multimodel <- polr(as.factor(y1) ~ x1, method = "logistic")
#'
#' y1 <- multimodel$model[,1]
#' lev1 <- multimodel$lev
#' fitprob1 <- fitted(multimodel)
#'
#' resid.ord <- dpit_ordi(y=y1, level=lev1, fitprob=fitprob1)
#' @export
dpit_ordi <- function(y, level, fitprob,plot=TRUE, scale="normal", line_args=list(), ...) {
  k <- length(level)
  out <- as.numeric(factor(y, ordered = TRUE))
  n <- length(out)
  q <- t(apply(fitprob, 1, cumsum))
  inde <- cbind(1:n, out)
  res <- q[inde]

  empcdf <- rep(NA, n)
  for(i in 1:n){
    if(i %in% which(out==k)) next
    note <- matrix(NA, ncol=k, nrow=n)
    for(p in 1:k){
        note[,p] <- fitprob[, p] * (res[i] > q[, p])
    }
    note.sum <- rowSums(note)
    note.sum[i] <- 0
    empcdf[i] <- sum(note.sum)/(n-1)
  }

  # for loop with max values
  ses <- ifelse(out == k, q[, 1], 0)
  for(i in 1:n){
    if(i %in% which(out != k)) next
    pses <- (ses[i] < q[,1])*q[,k-1]
    pses[pses==0] <- 1
    pses[i] <- 0
    empcdf[i] <- sum(pses)/(n-1)
  }
  .dpit_finalize(empcdf, plot=plot, scale=scale, line_args=line_args,...)
}

#' @rawNamespace S3method(dpit,default)
dpit.default <- function(model, ...) {
  cls <- paste(class(model), collapse = ", ")
  stop(sprintf("Unsupported model class for dpit(): %s", cls), call. = FALSE)
}

.dpit_finalize <- function(res,
                           plot,
                           scale,
                           line_args,
                           ...) {
  scale <- match.arg(scale, c("normal", "uniform"))
  eps <- .Machine$double.eps
  res <- pmin(pmax(res, eps), 1 - eps)
  if (isTRUE(plot)) {
    qqplot.resid(res, scale = scale, line_args = line_args, ...)
  }
  out <- if (scale == "normal") stats::qnorm(res) else res
  return(out)
}

#' @keywords internal
qqplot.resid <- function(empcdf, scale, line_args, ...) {
  keep <- is.finite(empcdf)
  u <- empcdf[keep]
  n <- length(u)

  if (n == 0L) {
    stop("No finite residuals available for QQ plot.", call. = FALSE)
  }

  if (scale == "normal") {
    y <- stats::qnorm(u)
    x <- stats::qnorm(stats::ppoints(n))
  } else {
    y <- u
    x <- stats::ppoints(n)
  }

  qq_defaults <- list(
    main = "QQ plot",
    xlab = "Theoretical Quantiles",
    ylab = "Sample Quantiles",
    cex.lab = 1, cex.axis = 1, cex.main = 1.5, lwd = 1.5
  )
  qq_args <- utils::modifyList(qq_defaults, list(...))
  do.call(stats::qqplot, c(list(x = x, y = y), qq_args))

  abline_defaults <- list(a = 0, b = 1, col = "red", lty = 5, lwd = 1.5)
  ab_args <- utils::modifyList(abline_defaults, line_args)
  do.call(graphics::abline, ab_args)
}

#' @title LGPIF Data
#' @description
#' Data from the Wisconsin Local Government Property Insurance Fund (LGPIF).
#' The LGPIF was established to provide property insurance for local
#' government entities that include counties, cities, towns,
#' villages, school districts, fire departments, and other
#' miscellaneous entities, and is administered by the Wisconsin Office of the Insurance Commissioner. Properties covered under this fund include government buildings, vehicles, and equipment.
#' @source https://sites.google.com/a/wisc.edu/jed-frees/
#' @format A data frame with 5677 rows and 41 variables:
#' \describe{
#' \item{\code{PolicyNum}}{Policy number}
#' \item{\code{Year}}{Policy year}
#' \item{\code{ClaimBC}}{Total building and contents (BC) claims in the year}
#' \item{\code{ClaimIM}}{Total inland marine (IM) claims in the year (contractor’s equipment)}
#' \item{\code{ClaimPN}}{Total comprehensive claims from new motor vehicles in the year}
#' \item{\code{ClaimPO}}{Total comprehensive claims from old motor vehicles in the year}
#' \item{\code{ClaimCN}}{Total collision claims from new vehicles in the year}
#' \item{\code{ClaimCO}}{Total collision claims from old vehicles in the year}
#' \item{\code{TypeCity}}{Indicator for city entity}
#' \item{\code{TypeCounty}}{Indicator for county entity}
#' \item{\code{TypeMisc}}{Indicator for miscellaneous entity}
#' \item{\code{TypeSchool}}{Indicator for school entity}
#' \item{\code{TypeTown}}{Indicator for town entity}
#' \item{\code{TypeVillage}}{Indicator for village entity}
#' \item{\code{IsRC}}{Indicator for replacement cost (motor vehicles)}
#' \item{\code{CoverageBC}}{Log coverage amount for building and contents (in millions of dollars)}
#' \item{\code{lnDeductBC}}{Log deductible amount for building and contents}
#' \item{\code{NoClaimCreditBC}}{Indicator for no BC claims in prior year}
#' \item{\code{yAvgBC}}{Average BC claim amount}
#' \item{\code{FreqBC}}{Frequency of BC claims}
#' \item{\code{CoverageIM}}{Log coverage amount for inland marine (in millions of dollars)}
#' \item{\code{lnDeductIM}}{Log deductible amount for inland marine}
#' \item{\code{NoClaimCreditIM}}{Indicator for no IM claims in prior year}
#' \item{\code{yAvgIM}}{Average IM claim amount}
#' \item{\code{FreqIM}}{Frequency of IM claims}
#' \item{\code{CoveragePN}}{Log coverage amount for comprehensive new vehicles (in millions of dollars)}
#' \item{\code{NoClaimCreditPN}}{Indicator for no PN claims in prior year}
#' \item{\code{yAvgPN}}{Average PN claim amount}
#' \item{\code{FreqPN}}{Frequency of PN claims}
#' \item{\code{CoveragePO}}{Log coverage amount for comprehensive old vehicles (in millions of dollars)}
#' \item{\code{NoClaimCreditPO}}{Indicator for no PO claims in prior year}
#' \item{\code{yAvgPO}}{Average PO claim amount}
#' \item{\code{FreqPO}}{Frequency of PO claims}
#' \item{\code{CoverageCN}}{Log coverage amount for collision of new vehicles (in millions of dollars)}
#' \item{\code{NoClaimCreditCN}}{Indicator for no CN claims in prior year}
#' \item{\code{yAvgCN}}{Average CN claim amount}
#' \item{\code{FreqCN}}{Frequency of CN claims}
#' \item{\code{CoverageCO}}{Log coverage amount for collision of old vehicles (in millions of dollars)}
#' \item{\code{NoClaimCreditCO}}{Indicator for no CO claims in prior year}
#' \item{\code{yAvgCO}}{Average CO claim amount}
#' \item{\code{FreqCO}}{Frequency of CO claims}
#' }
#' @references Frees, E. W., Lee, G., & Yang, L. (2016). Multivariate frequency-severity regression models in insurance. Risks, 4(1), 4.
"LGPIF"

#' Residuals for regression models with two-part outcomes
#'
#' Calculates DPIT residuals with model for semi-continuous outcomes.
#' `dpit_2pm` can be used either with `model0` and `model1` or with `part0` and `part1` as arguments.
#'
#' @usage dpit_2pm(model0, model1, y, part0, part1, plot=TRUE, scale = "normal",
#'  line_args= list(), ...)
#'
#'
#' @param model0 Model object for 0 outcomes (e.g., logistic regression)
#' @param model1 Model object for the continuous part (gamma regression)
#' @param y Semicontinuous outcomes.
#' @param part0 Alternative argument to `model0`. One can supply the sequence of probabilities \eqn{P(Y_i=0),~i=1,\ldots,n}.
#' @param part1 Alternative argument to `model1`. One can fit a regression model on the positive data and supply their probability integral transform. Note that the length of `part1` is the number of positive values in `y` and can be shorter than `part0`.
#' @param plot A logical value indicating whether or not to return QQ-plot
#' @param scale You can choose the scale of the residuals among `normal` and `uniform`. The default scale is `normal`.
#' @param line_args A named list of graphical parameters passed to
#'   \code{graphics::abline()} to modify the reference (red) 45° line
#'   in the QQ plot. If left empty, a default red dashed line is drawn.
#' @param ... Additional graphical arguments passed to
#'   \code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
#'   \code{col}, \code{cex}, \code{xlab}, \code{ylab}).
#'
#'
#' @details
#' For formulation details on semicontinuous outcomes, see \code{\link{dpit}}.
#'
#' In two-part models, the probability of zero can be modeled using a logistic regression, `model0`,
#' while the positive observations can be modeled using a gamma regression, `model1.`
#' Users can choose to use different models and supply the resulting probabilities of zero and probability integral transforms.
#'  `part0` should be the sequence of fitted probabilities of zeros \eqn{\hat{p}_0(\mathbf{X}_i) ,~i=1,\ldots,n}.
#'  `part1` should be the probability integral transform of the positive part \eqn{\hat{G}(Y_i|\mathbf{X}_i)}.
#'  Note that the length of `part1` is the number of positive values in `y` and can be shorter than `part0`.
#'
#'
#' @returns Residuals. If plot=TRUE, also produces a QQ plot.
#'
#' @importFrom stats ecdf
#' @importFrom MASS gamma.dispersion
#' @export
#'
#' @examples
#' library(MASS)
#' n <- 500
#' beta10 <- 1
#' beta11 <- -2
#' beta12 <- -1
#' beta13 <- -1
#' beta14 <- -1
#' beta15 <- -2
#' x11 <- rnorm(n)
#' x12 <- rbinom(n, size = 1, prob = 0.4)
#'
#' p1 <- 1 / (1 + exp(-(beta10 + x11 * beta11 + x12 * beta12)))
#' lambda1 <- exp(beta13 + beta14 * x11 + beta15 * x12)
#' y2 <- rgamma(n, scale = lambda1 / 2, shape = 2)
#' y <- rep(0, n)
#' u <- runif(n, 0, 1)
#' ind1 <- which(u >= p1)
#' y[ind1] <- y2[ind1]
#'
#' # models as input
#' mgamma <- glm(y[ind1] ~ x11[ind1] + x12[ind1], family = Gamma(link = "log"))
#' m10 <- glm(y == 0 ~ x12 + x11, family = binomial(link = "logit"))
#' resid.model <- dpit_2pm(model0 = m10, model1 = mgamma, y = y)
#'
#' # PIT as input
#' cdfgamma <- pgamma(y[ind1],
#'   scale = mgamma$fitted.values * gamma.dispersion(mgamma),
#'   shape = 1 / gamma.dispersion(mgamma)
#' )
#' p1f <- m10$fitted.values
#' resid.pit <- dpit_2pm(y = y, part0 = p1f, part1 = cdfgamma)

# dpit_2pm <- function(model0, model1, y, part0, part1, plot = TRUE, scale = "normal", line_args=list(), ...){
#   UseMethod("dpit_2pm")
# }

dpit_2pm <- function(model0, model1, y, part0, part1, plot = TRUE, scale = "normal", line_args=list(), ...) {
  if (!(scale %in% c("normal", "uniform"))) stop("scale has to be either normal or uniform")
  if (missing(y)) stop("argument y is missing, with no default")
  if (sum(!(y >= 0)) != 0) stop("y has to be nonnegative")


  if (!missing(model0) && !missing(model1) && !missing(y)) {
    if (model0$family[[1]] != "binomial") stop("model0 has to be a logistic regression")
    if (model1$family[[1]] != "Gamma") stop("model1 has to be a gamma regression")
    if (length(model1$fitted.values) != sum(y > 0)) stop("Length of the fitted values of model1 has to be the same as the number of positive values in y")
    if (length(model0$fitted.values) != length(y)) {
      stop("Length of the fitted values of model0 has to be the same as the length of y")
    } else {
      n <- length(y)
      cdfgamma <- pgamma(y[y > 0],
        scale = model1$fitted.values * gamma.dispersion(model1),
        shape = 1 / gamma.dispersion(model1)
      )
      p1f <- model0$fitted.values
      cdf1 <- rep(0, n)
      cdf1[y == 0] <- model0$fitted.values[y == 0]
      cdf1[y > 0] <- model0$fitted.values[which(y > 0)] + (1 - model0$fitted.values[which(y > 0)]) * cdfgamma
      newp <- cdf1 * ecdf(p1f)(cdf1)
    }
  }

  if (!missing(part0) && !missing(part1) && !missing(y)) {
    if (length(part0) != length(y)) stop("Length of part0 has to be the same as the length of y")
    if (sum(y != 0) != length(part1)) stop("Length of part1 has to be the same as the number of positive values in y")
    if (sum((part1 < 0) + (part1 > 1)) != 0) stop("Values of part0 and part1 have to be between 0 and 1")
    if (sum((part0 < 0) + (part0 > 1)) != 0) stop("Values of part0 and part1 have to be between 0 and 1")
    n <- length(y)
    cdf1 <- rep(0, n)
    cdf1[y == 0] <- part0[y == 0]
    cdf1[y > 0] <- part0[y > 0] + (1 - part0[y > 0]) * part1
    newp <- cdf1 * ecdf(part0)(cdf1)
  }

  if (!missing(model0) && !missing(part1) && !missing(y)) {
    if (length(model0$fitted.values) != length(y)) stop("Length of the fitted values of model0 has to be the same as the length of y")
    if (sum(y != 0) != length(part1)) stop("Length of part1 has to be the same as the number of positive values in y")
    if (model0$family[[1]] != "binomial") stop("model0 has to be a logistic regression")
    if (sum((part1 < 0) + (part1 > 1)) != 0) stop("Values of part0 and part1 have to be between 0 and 1")
    n <- length(y)
    cdf1 <- rep(0, n)
    part0 <- model0$fitted.values
    cdf1[y == 0] <- part0[y == 0]
    cdf1[y > 0] <- part0[y > 0] + (1 - part0[y > 0]) * part1
    newp <- cdf1 * ecdf(part0)(cdf1)
  }

  if (!missing(part0) && !missing(model1) && !missing(y)) {
    if (length(model1$fitted.values) != sum(y > 0)) stop("Length of the fitted values of model1 has to be the same as the number of positive values in y")
    if (length(part0) != length(y)) stop("Length of part0 has to be the same as the length of y")
    if (sum((part0 < 0) + (part0 > 1)) != 0) stop("Values of part0 and part1 have to be between 0 and 1")
    if (model1$family[[1]] != "Gamma") stop("model1 has to be a gamma regression")
    n <- length(y)
    cdf1 <- rep(0, n)
    cdfgamma <- pgamma(y[y > 0],
      scale = model1$fitted.values * gamma.dispersion(model1),
      shape = 1 / gamma.dispersion(model1)
    )
    cdf1[y == 0] <- part0[y == 0]
    cdf1[y > 0] <- part0[y > 0] + (1 - part0[y > 0]) * cdfgamma
    newp <- cdf1 * ecdf(part0)(cdf1)
  }

  if (plot == TRUE) {
    qqplot.resid(newp, scale, line_args,...)
  } else {
    if (scale == "normal") newp <- qnorm(newp)
    if (scale == "uniform") newp <- newp
  }
  return(newp)
}

#' Residuals for a tobit model
#'
#' Computes DPIT residuals for tobit regression models using the observed
#' responses (`y`) and their corresponding fitted distributional parameters (`mu`, `sd`).
#'
#' @usage dpit_tobit(y, mu, sd, plot=TRUE, scale="normal", line_args=list(), ...)
#' @param y An observed outcome vector.
#' @param mu A vector of fitted mean values of latent variables.
#' @param sd A standard deviation of latent variables.
#' @param plot A logical value indicating whether or not to return QQ-plot
#' @param scale You can choose the scale of the residuals among `normal` and `uniform`.
#' The sample quantiles of the residuals are plotted against
#' the theoretical quantiles of a standard normal distribution under the normal scale,
#' and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
#'  The default scale is `normal`.
#' @param line_args A named list of graphical parameters passed to
#'   \code{graphics::abline()} to modify the reference (red) 45° line
#'   in the QQ plot. If left empty, a default red dashed line is drawn.
#' @param ... Additional graphical arguments passed to
#'   \code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
#'   \code{col}, \code{cex}, \code{xlab}, \code{ylab}).
#' @returns DPIT residuals.
#'
#' @details
#' For formulation details on semicontinuous outcomes, see \code{\link{dpit}}.
#'
#' @examples
#' ## Tobit regression model
#' library(VGAM)
#' n <- 500
#' beta13 <- 1
#' beta14 <- -3
#' beta15 <- 3
#'
#' set.seed(1234)
#' x11 <- runif(n)
#' x12 <- runif(n)
#' lambda1 <- beta13 + beta14 * x11 + beta15 * x12
#' sd0 <- 0.3
#' yun <- rnorm(n, mean = lambda1, sd = sd0)
#' y <- ifelse(yun >= 0, yun, 0)
#'
#' # Using VGAM package
#' # True model
#' fit1 <- vglm(formula = y ~ x11 + x12,
#'              tobit(Upper = Inf, Lower = 0, lmu = "identitylink"))
#' # Missing covariate
#' fit1miss <- vglm(formula = y ~ x11,
#'                  tobit(Upper = Inf, Lower = 0, lmu = "identitylink"))
#'
#' resid.tobit1 <- dpit_tobit(y = y, mu = VGAM::fitted(fit1), sd = sd0)
#' resid.tobit2 <- dpit_tobit(y = y, mu = VGAM::fitted(fit1miss), sd = sd0)
#'
#' # Using AER package
#' library(AER)
#' # True model
#' fit2 <- tobit(y ~ x11 + x12, left = 0, right = Inf, dist = "gaussian")
#' # Missing covariate
#' fit2miss <- tobit(y ~ x11, left = 0, right = Inf, dist = "gaussian")
#'
#' resid.aer1 <- dpit_tobit(y = y, mu = fitted(fit2), sd = sd0)
#' resid.aer2 <- dpit_tobit(y = y, mu = fitted(fit2miss), sd = sd0)
#' @export
dpit_tobit <- function(y, mu, sd,
                       plot = TRUE,
                       scale = "normal",
                       line_args = list(),
                       ...) {
  y  <- as.numeric(y)
  mu <- as.numeric(mu)
  sd <- as.numeric(sd)
  p1f  <- pnorm(0, mean = mu, sd = sd)
  cdf1 <- pnorm(y, mean = mu, sd = sd)
  Fhat <- stats::ecdf(p1f)
  newp <- as.vector(cdf1 * Fhat(cdf1))

  .dpit_finalize(newp, plot = plot, scale = scale, line_args = line_args, ...)
}

#' @rawNamespace S3method(dpit,vglm)
dpit.vglm <- function(model,
                      plot = TRUE,
                      scale = "normal",
                      line_args = list(),
                      ...) {
  y <- model@y
  fitted <- VGAM::fitted(model)
  sd <- do.call(model@misc$link[2],
                args = list(theta = coef(model)[2], inverse = TRUE))

  dpit_tobit(y = y, mu = fitted, sd = sd,
             plot = plot, scale = scale, line_args = line_args, ...)
}

#' @rawNamespace S3method(dpit,tobit)
dpit.tobit <- function(model,
                       plot = TRUE,
                       scale = "normal",
                       line_args = list(),
                       ...) {
  y <- as.numeric(model$y)
  fitted <- VGAM::fitted(model)
  sd <- summary(model)$scale

  dpit_tobit(y = y, mu = fitted, sd = sd,
             plot = plot, scale = scale, line_args = line_args, ...)
}

#' @keywords internal
inv.zpois <- function(s, pzero, meanpoisson) {
  qres <- ifelse(s < (pzero + (1 - pzero) * (ppois(0, lambda = meanpoisson))), 0,
    qpois(pmax((s - pzero)/(1 - pzero), 0), lambda = meanpoisson)
  ) - 1
  pres <- ifelse(qres == -1, 0, (pzero + (1 - pzero) * (ppois(qres, meanpoisson))))
  return(pres)
}


#' Residuals for regression models with zero-inflated Poisson outcomes
#'
#' Computes DPIT residuals for regression models with zero-inflated Poisson
#' outcomes using the observed counts(`y`) and their fitted distributional
#' parameters(`mu`, `pzero`).
#'
#'
#' @usage dpit_zpois(y, mu, pzero, plot=TRUE, scale="normal", line_args=list(), ...)
#' @param y An observed outcome vector.
#' @param mu A vector of fitted mean values for the count (non-zero) component.
#' @param pzero A vector of fitted probabilities for the zero-inflation component.
#' @param plot A logical value indicating whether or not to return QQ-plot
#' @param scale You can choose the scale of the residuals among `normal` and `uniform`.
#' The sample quantiles of the residuals are plotted against
#' the theoretical quantiles of a standard normal distribution under the normal scale,
#' and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
#'  The default scale is `normal`.
#' @param line_args A named list of graphical parameters passed to
#'   \code{graphics::abline()} to modify the reference (red) 45° line
#'   in the QQ plot. If left empty, a default red dashed line is drawn.
#' @param ... Additional graphical arguments passed to
#'   \code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
#'   \code{col}, \code{cex}, \code{xlab}, \code{ylab}).
#' @returns DPIT residuals.
#' @importFrom stats family
#' @export
#'
#' @details
#' For formulation details on discrete outcomes, see \code{\link{dpit}}.
#'
#'
#' @examples
#' ## Zero-Inflated Poisson
#' library(pscl)
#' n <- 500
#' set.seed(1234)
#' # Covariates
#' x1 <- rnorm(n)
#' x2 <- rbinom(n, 1, 0.7)
#' # Coefficients
#' beta0 <- -2
#' beta1 <- 2
#' beta2 <- 1
#' beta00 <- -2
#' beta10 <- 2
#'
#' # Mean of Poisson part
#' lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
#' # Excess zero probability
#' p0 <- 1 / (1 + exp(-(beta00 + beta10 * x1)))
#' ## simulate outcomes
#' y0 <- rbinom(n, size = 1, prob = 1 - p0)
#' y1 <- rpois(n, lambda1)
#' y <- ifelse(y0 == 0, 0, y1)
#' ## True model
#' modelzero1 <- zeroinfl(y ~ x1 + x2 | x1, dist = "poisson", link = "logit")
#' y1 <- modelzero1$y
#' mu1    <- stats::predict(modelzero1, type = "count")
#' pzero1 <- stats::predict(modelzero1, type = "zero")
#' resid.zero1 <- dpit_zpois(y= y1, pzero=pzero1, mu=mu1)
#'
#' ## Zero inflation
#' modelzero2 <- glm(y ~ x1 + x2, family = poisson(link = "log"))
#' y2 <- modelzero2$y
#' mu2    <- fitted(modelzero2)
#' resid.zero2 <- dpit_pois(y= y2, mu=mu2)
#'
#'
#' @export
dpit_zpois <- function(y, mu, pzero,
                       plot=TRUE, scale="normal", line_args=list(), ...) {
  if (missing(y) || missing(pzero) || missing(mu)) {
    stop("y, pzero, and mu are required.", call. = FALSE)
  }
  y <- as.numeric(y)
  n <- length(y)

  if (length(pzero) != n) stop("length(pzero) must equal length(y).", call. = FALSE)
  if (length(mu)    != n) stop("length(mu) must equal length(y).", call. = FALSE)
  if (any(y < 0)) stop("y must be nonnegative.", call. = FALSE)

  res <- pzero + (1 - pzero) * stats::ppois(y, lambda = mu)

  empcdf <- rep(NA_real_, n)
  for (i in seq_len(n)) {
    pres <- inv.zpois(res[i], pzero, mu)  # inv.zpois is assumed to exist
    pres[i] <- 0
    empcdf[i] <- sum(pres) / (n - 1)
  }
  .dpit_finalize(empcdf, plot=plot, scale=scale, line_args=line_args,...)
}

#' Residuals for regression models with poisson outcomes
#'
#' Computes DPIT residuals for Poisson outcomes regression using the observed counts (`y`) and their
#' corresponding fitted mean values (`mu`).
#'
#' @usage dpit_pois(y, mu, plot=TRUE, scale="normal", line_args=list(), ...)
#' @param y An observed outcome vector.
#' @param mu A vector of fitted mean values.
#' @param plot A logical value indicating whether or not to return QQ-plot
#' @param scale You can choose the scale of the residuals among `normal` and `uniform`.
#' The sample quantiles of the residuals are plotted against
#' the theoretical quantiles of a standard normal distribution under the normal scale,
#' and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
#'  The default scale is `normal`.
#' @param line_args A named list of graphical parameters passed to
#'   \code{graphics::abline()} to modify the reference (red) 45° line
#'   in the QQ plot. If left empty, a default red dashed line is drawn.
#' @param ... Additional graphical arguments passed to
#'   \code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
#'   \code{col}, \code{cex}, \code{xlab}, \code{ylab}).
#' @returns DPIT residuals.
#'
#' @details
#' For formulation details on discrete outcomes, see \code{\link{dpit}}.
#'
#' @examples
#' ## Poisson example
#' n <- 500
#' set.seed(1234)
#' # Covariates
#' x1 <- rnorm(n)
#' x2 <- rbinom(n, 1, 0.7)
#' # Coefficients
#' beta0 <- -2
#' beta1 <- 2
#' beta2 <- 1
#' lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
#' y <- rpois(n, lambda1)
#'
#' # True model
#' poismodel <- glm(y ~ x1 + x2, family = poisson(link = "log"))
#' y1 <- poismodel$y
#' p1f <- fitted(poismodel)
#' resid.poi <- dpit_pois(y=y1, mu=p1f)
#'
#' @export
dpit_pois <- function(y, mu, plot=TRUE, scale="normal", line_args=list(), ...) {
  n <- length(y)
  lambda1f <- mu
  res <- ppois(y, lambda = lambda1f)
  empcdf <- rep(NA,n)

  for(i in 1:n){
    qres <- qpois(res[i], lambda=lambda1f)-1
    pres <- ppois(qres,lambda = lambda1f)
    pres[i] <- 0
    empcdf[i] <-sum(pres)/(n-1)
  }
  .dpit_finalize(empcdf, plot=plot, scale=scale, line_args = line_args, ...)
}

#' @rawNamespace S3method(dpit,glm)
dpit.glm <- function(model,
                     plot = TRUE,
                     scale = c("normal", "uniform"),
                     line_args = list(),
                     ...) {
  scale <- match.arg(scale)

  .dpit_glm(
    model,
    plot = plot,
    scale = scale,
    line_args = line_args,
    ...
  )
}


.dpit_glm_key <- function(key,
                          plot = TRUE,
                          scale = c("normal", "uniform"),
                          line_args = list(),
                          ...) {
  UseMethod(".dpit_glm_key")
}


.dpit_glm <- function(model,
                      plot,
                      scale,
                      line_args,
                      ...) {
  fam_obj <- stats::family(model)
  fam_raw <- fam_obj$family
  link_raw <- fam_obj$link

  fam_key <- fam_raw
  if (startsWith(fam_raw, "Negative Binomial")) {
    fam_key <- "negbin"
  }

  key <- list(
    model  = model,
    family = fam_raw,
    link   = link_raw
  )

  class(key) <- fam_key

  .dpit_glm_key(
    key,
    plot = plot,
    scale = scale,
    line_args = line_args,
    ...
  )
}


.dpit_glm_key <- function(key, ...) {
  UseMethod(".dpit_glm_key")
}

#' @rawNamespace S3method(.dpit_glm_key,poisson)
.dpit_glm_key.poisson <- function(key, plot, scale, line_args, ...) {
  model <- key$model
  y <- stats::model.response(stats::model.frame(model))
  mu <- stats::fitted.values(model)

  if (!exists("dpit_pois", mode = "function", inherits = TRUE)) {
    stop("dpit_pois() not found. Please define it in the package namespace.", call. = FALSE)
  }
  dpit_pois(
    y = y,
    mu = mu,
    plot = plot,
    scale = scale,
    line_args = line_args,
    ...
  )
}

#' @rawNamespace S3method(.dpit_glm_key,binomial)
.dpit_glm_key.binomial <- function(key, plot, scale, line_args, ...) {
  model <- key$model
  y <- stats::model.response(stats::model.frame(model))
  mu <- stats::fitted.values(model)

  if (!exists("dpit_bin", mode = "function", inherits = TRUE)) {
    stop("dpit_bin() not found. Please define it in the package namespace.", call. = FALSE)
  }

  dpit_bin(
    y = y,
    prob = mu,
    plot = plot,
    scale = scale,
    line_args = line_args,
    ...
  )
}

#' @rawNamespace S3method(.dpit_glm_key,negbin)
.dpit_glm_key.negbin <- function(key, plot, scale, line_args, ...) {
  model <- key$model
  y <- stats::model.response(stats::model.frame(model))
  mu <- stats::fitted.values(model)

  size <- if (!is.null(model$theta)) model$theta else summary(model)$theta

  if (!exists("dpit_nb", mode = "function", inherits = TRUE)) {
    stop("dpit_nb() not found. Please define it in the package namespace.", call. = FALSE)
  }

  dpit_nb(
    y = y,
    mu = mu,
    size = size,
    plot = plot,
    scale = scale,
    line_args = line_args,
    ...
  )
}

#' @rawNamespace S3method(.dpit_glm_key,Tweedie)
.dpit_glm_key.Tweedie <- function(key, plot, scale, line_args, ...) {
  model <- key$model
  y1 <- model$y
  p.max <- get("p", envir = environment(model$family$variance))
  lambda1f <- model$fitted.values
  phi1f <- summary(model)$dis

  dpit_tweedie(
    y = y1,
    mu = lambda1f,
    xi = p.max,
    phi = phi1f,
    plot = plot,
    scale = scale,
    line_args = line_args,
    ...
  )
}

#' @rawNamespace S3method(.dpit_glm_key,default)
.dpit_glm_key.default <- function(key,
                                  plot = plot,
                                  scale = scale,
                                  line_args = line_args,
                                  ...) {
  fam  <- key$family
  link <- key$link
  stop(sprintf("Unsupported GLM family/link for dpit(): %s / %s", fam, link),
       call. = FALSE)
}

#' @rawNamespace S3method(dpit,polr)
dpit.polr <- function(model,
                      plot = TRUE,
                      scale = "normal",
                      line_args = line_args,
                      ...) {
  if (!exists("dpit_ordi", mode = "function", inherits = TRUE)) {
    stop("dpit_ordi() not found. Please define it in the package namespace.", call. = FALSE)
  }
  y <- model$model[,1]
  lev <- model$lev
  p1f <- fitted(model)
  dpit_ordi(y=y, scale=scale, level=lev, fitprob=p1f, plot=plot)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ord_curve.R
\name{ord_curve}
\alias{ord_curve}
\title{Ordered curve for assessing mean structures}
\usage{
ord_curve(model, thr, line_args=list(), ...)
}
\arguments{
\item{model}{Regression model object (e.g.,\code{lm}, \code{glm}, \code{glm.nb}, \code{polr}, \code{lm})}

\item{thr}{Threshold variable (e.g., predictor, fitted values, or variable to be included as a covariate)}

\item{line_args}{A named list of graphical parameters passed to
\code{graphics::abline()} to modify the reference (red) 45° line
in the QQ plot. If left empty, a default red dashed line is drawn.}

\item{...}{Additional graphical arguments passed to
\code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
\code{col}, \code{cex}, \code{xlab}, \code{ylab}).}
}
\value{
\itemize{
\item x-axis: \eqn{\hat L_1(t)}
\item y-axis: \eqn{\hat L_2(t)}
}
which are defined in Details.
}
\description{
Creates a plot to assess the mean structure of regression models. The
plot compares the cumulative sum of the response variable and its hypothesized value.
Deviation from the diagonal suggests the possibility that the mean structure of the model is incorrect.
}
\details{
The ordered curve plots \deqn{\hat{L}_1(t)=\frac{\sum_{i=1}^n\left[Y_i1(Z_i\leq t)\right]}{\sum_{i=1}^nY_i}} against
\deqn{\hat{L}_2(t)=\frac{\sum_{i=1}^n\left[\hat{\lambda}_i1(Z_i\leq t)\right]}{\sum_{i=1}^n\hat{\lambda}_i},}
where \eqn{\hat{\lambda}_i} is the fitted mean, and \eqn{Z_i} is the threshold variable. \cr

If the mean structure is correctly specified in the model,
\eqn{\hat L_1(t)} and \eqn{\hat L_2(t)} should be close to each other.

If the curve is distant from the diagonal, it suggests incorrectness in the mean structure.
Moreover, if the curve is above the diagonal, the summation of the response is larger than
the fitted mean, which implies that the mean is underestimated, and vice versa. \cr

The role of \code{thr} (threshold variable \eqn{Z}) is to determine the rule  for accumulating \eqn{\hat{\lambda}_i} and \eqn{Y_i}, \eqn{i=1,\ldots,n}
for the ordered curve.
The candidate for \code{thr} could be any function of predictors such as a single predictor (e.g., \code{x1}),
a linear combination of predictor (e.g., \code{x1+x2}), or fitted values (e.g., \code{fitted(model)}).
It can also be a variable being considered to be included in the mean function.
If a variable  leads to a large discrepancy between the ordered curve and the diagonal,
including this variable in the mean function should be considered.

For more details, see the reference paper.
}
\examples{
## Binary example of ordered curve
n <- 500
set.seed(1234)
x1 <- rnorm(n, 1, 1)
x2 <- rbinom(n, 1, 0.7)
beta0 <- -5
beta1 <- 2
beta2 <- 1
beta3 <- 3
q1 <- 1 / (1 + exp(beta0 + beta1 * x1 + beta2 * x2 + beta3 * x1 * x2))
y1 <- rbinom(n, size = 1, prob = 1 - q1)

## True Model
model0 <- glm(y1 ~ x1 * x2, family = binomial(link = "logit"))
ord_curve(model0, thr = model0$fitted.values) # set the threshold as fitted values

## Missing a covariate
model1 <- glm(y1 ~ x1, family = binomial(link = "logit"))
ord_curve(model1, thr = x2) # set the threshold as a covariate

## Poisson example of ordered curve
n <- 500
set.seed(1234)
x1 <- rnorm(n)
x2 <- rnorm(n)
beta0 <- 0
beta1 <- 2
beta2 <- 1
lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)

y <- rpois(n, lambda1)

## True Model
poismodel1 <- glm(y ~ x1 + x2, family = poisson(link = "log"))
ord_curve(poismodel1, thr = poismodel1$fitted.values)

## Missing a covariate
poismodel2 <- glm(y ~ x1, family = poisson(link = "log"))
ord_curve(poismodel2, thr = poismodel2$fitted.values)
ord_curve(poismodel2, thr = x2)

}
\references{
Yang, Lu. "Double Probability Integral Transform Residuals for Regression Models with Discrete Outcomes." arXiv preprint arXiv:2308.15596 (2023).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dpit_main.R
\name{dpit}
\alias{dpit}
\title{DPIT residuals for regression models with various non-continuous outcomes}
\usage{
dpit(model, plot=TRUE, scale="normal", line_args=list(), ...)
}
\arguments{
\item{model}{A model object.}

\item{plot}{A logical value indicating whether or not to return QQ-plot}

\item{scale}{You can choose the scale of the residuals among \code{normal} and \code{uniform}.
The sample quantiles of the residuals are plotted against
the theoretical quantiles of a standard normal distribution under the normal scale,
and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
The default scale is \code{normal}.}

\item{line_args}{A named list of graphical parameters passed to
\code{graphics::abline()} to modify the reference (red) 45° line
in the QQ plot. If left empty, a default red dashed line is drawn.}

\item{...}{Additional graphical arguments passed to
\code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
\code{col}, \code{cex}, \code{xlab}, \code{ylab}).}
}
\value{
DPIT residuals. If \code{plot=TRUE}, also produces a QQ plot.
}
\description{
Calculates DPIT residuals for regression models with non-continuous outcomes.
In particular, model assumptions for GLMs with discrete outcomes (e.g., binary, Poisson, and negative binomial), ordinal
regression models, zero-inflated regression models, and semicontinuous outcome
models can be assessed using \code{dpit()}.
}
\details{
This function determines the appropriate computation based on the class of
\code{model}. The supported model objects and outcome types are listed below.

In addition to the class-based interface, the package also provides
distribution-specific DPIT calculators. If a fitted model comes from a
different class but has a supported outcome distribution, users can call
the corresponding distribution-based function directly.
For instance, for a regression model with Poisson outcomes,
one can use dpit to calculate the residuals if
the model is fit using \code{glm} function, or to use \code{dpit_pois} upon supplying fitted mean values.

\itemize{
\item \strong{Discrete outcomes}
\itemize{
\item \code{\link[stats]{glm}} with \code{family = binomial()} (see \code{\link{dpit_bin}}).
\item \code{\link[stats]{glm}} with \code{family = poisson()}
(see \code{\link{dpit_pois}}).
\item \code{\link[MASS]{glm.nb}} for negative binomial regression
(see \code{\link{dpit_nb}}).
\item \code{\link[MASS]{polr}} for ordinal outcomes
(see \code{\link{dpit_ordi}}).
}

\item \strong{Zero-inflated discrete outcomes}
\itemize{
\item \code{\link[pscl]{zeroinfl}} with \code{dist = "poisson"}
(see \code{\link{dpit_zpois}}).
\item \code{\link[pscl]{zeroinfl}} with \code{dist = "negbin"}
(see \code{\link{dpit_znb}}).
}

\item \strong{Semicontinuous outcomes}
\itemize{
\item Tobit regression via \code{\link[AER]{tobit}} from \code{AER} or \code{\link[VGAM]{vglm}} from \code{VGAM}
(see \code{\link{dpit_tobit}}).
\item Tweedie regression via \code{\link[stats]{glm}} with a Tweedie family
(see \code{\link{dpit_tweedie}}).
}
}

\strong{Formulation for Discrete and Zero-Inflated Outcomes:}
\cr
The DPIT residual for the \eqn{i}th observation is defined as follows:
\deqn{\hat{r}(Y_i|X_i) = \hat{G}\bigg(\hat{F}(Y_i|\mathbf{X}_i)\bigg)}
where
\deqn{\hat{G}(s) = \frac{1}{n-1}\sum_{j=1, j \neq i}^{n}\hat{F}\bigg(\hat{F}^{(-1)}(\mathbf{X}_j)\bigg|\mathbf{X}_j\bigg)}
and \eqn{\hat{F}} refers to the fitted cumulative distribution function.
When \code{scale="uniform"}, DPIT residuals should closely follow a uniform distribution, otherwise it implies model deficiency.
When \code{scale="normal"}, it applies the normal quantile transformation to the DPIT residuals
\deqn{\Phi^{-1}\left[\hat{r}(Y_i|\mathbf{X}_i)\right],i=1,\ldots,n.} The null pattern is the standard normal distribution in this case.
\cr

\strong{Formulation for Semicontinuous Outcomes:}
\cr
The DPIT residuals for regression models with semi-continuous outcomes are \deqn{\hat{r}_i=\frac{\hat{F}(Y_i|\mathbf{X}_i)}{n}\sum_{j=1}^n1\left(\hat{p}_0(\mathbf{X}_j)\leq \hat{F}(Y_i|\mathbf{X}_i)\right), i=1,\ldots,n,}
where \eqn{\hat{p}_0(\mathbf{X}_i)} is the fitted probability of zero, and \eqn{\hat{F}(\cdot|\mathbf{X}_i)} is the  fitted cumulative distribution function for the \eqn{i}th observation. Furthermore, \deqn{\hat{F}(y|\mathbf{x})=\hat{p}_0(\mathbf{x})+\left(1-\hat{p}_0(\mathbf{x})\right)\hat{G}(y|\mathbf{x})}
where \eqn{\hat{G}} is the fitted cumulative distribution for the positive data.
}
\examples{
library(MASS)
n <- 500
set.seed(1234)
## Negative Binomial example
# Covariates
x1 <- rnorm(n)
x2 <- rbinom(n, 1, 0.7)
### Parameters
beta0 <- -2
beta1 <- 2
beta2 <- 1
size1 <- 2
lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
# generate outcomes
y <- rnbinom(n, mu = lambda1, size = size1)

# True model
model1 <- glm.nb(y ~ x1 + x2)
resid.nb1 <- dpit(model1, plot = TRUE, scale = "uniform")

# Overdispersion
model2 <- glm(y ~ x1 + x2, family = poisson(link = "log"))
resid.nb2 <- dpit(model2, plot = TRUE, scale = "normal")

## Binary example
n <- 500
set.seed(1234)
# Covariates
x1 <- rnorm(n, 1, 1)
x2 <- rbinom(n, 1, 0.7)
# Coefficients
beta0 <- -5
beta1 <- 2
beta2 <- 1
beta3 <- 3
q1 <- 1 / (1 + exp(beta0 + beta1 * x1 + beta2 * x2 + beta3 * x1 * x2))
y1 <- rbinom(n, size = 1, prob = 1 - q1)

# True model
model01 <- glm(y1 ~ x1 * x2, family = binomial(link = "logit"))
resid.bin1 <- dpit(model01, plot = TRUE)

# Missing covariates
model02 <- glm(y1 ~ x1, family = binomial(link = "logit"))
resid.bin2 <- dpit(model02, plot = TRUE)

## Poisson example
n <- 500
set.seed(1234)
# Covariates
x1 <- rnorm(n)
x2 <- rbinom(n, 1, 0.7)
# Coefficients
beta0 <- -2
beta1 <- 2
beta2 <- 1
lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
y <- rpois(n, lambda1)

# True model
poismodel1 <- glm(y ~ x1 + x2, family = poisson(link = "log"))
resid.poi1 <- dpit(poismodel1, plot = TRUE)

# Enlarge three outcomes
y <- rpois(n, lambda1) + c(rep(0, (n - 3)), c(10, 15, 20))
poismodel2 <- glm(y ~ x1 + x2, family = poisson(link = "log"))
resid.poi2 <- dpit(poismodel2, plot = TRUE)

## Ordinal example
n <- 500
set.seed(1234)
# Covariates
x1 <- rnorm(n, mean = 2)
# Coefficient
beta1 <- 3

# True model
p0 <- plogis(1, location = beta1 * x1)
p1 <- plogis(4, location = beta1 * x1) - p0
p2 <- 1 - p0 - p1
genemult <- function(p) {
  rmultinom(1, size = 1, prob = c(p[1], p[2], p[3]))
}
test <- apply(cbind(p0, p1, p2), 1, genemult)
y1 <- rep(0, n)
y1[which(test[1, ] == 1)] <- 0
y1[which(test[2, ] == 1)] <- 1
y1[which(test[3, ] == 1)] <- 2
multimodel <- polr(as.factor(y1) ~ x1, method = "logistic")
resid.ord1 <- dpit(multimodel, plot = TRUE)

## Non-Proportionality
n <- 500
set.seed(1234)
x1 <- rnorm(n, mean = 2)
beta1 <- 3
beta2 <- 1
p0 <- plogis(1, location = beta1 * x1)
p1 <- plogis(4, location = beta2 * x1) - p0
p2 <- 1 - p0 - p1
genemult <- function(p) {
  rmultinom(1, size = 1, prob = c(p[1], p[2], p[3]))
}
test <- apply(cbind(p0, p1, p2), 1, genemult)
y1 <- rep(0, n)
y1[which(test[1, ] == 1)] <- 0
y1[which(test[2, ] == 1)] <- 1
y1[which(test[3, ] == 1)] <- 2
multimodel <- polr(as.factor(y1) ~ x1, method = "logistic")
resid.ord2 <- dpit(multimodel, plot = TRUE)
}
\references{
L. Yang. Double probability integral transform residuals for regression models with discrete outcomes. Journal of Computational and Graphical Statistics, 33(3), pp.787–803, 2024. \cr
L. Yang. Diagnostics for regression models with semicontinuous outcomes. Biometrics, 80(1), ujae007, 2024.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dpit_binomial.R
\name{dpit_bin}
\alias{dpit_bin}
\title{Residuals for regression models with binary outcomes}
\usage{
dpit_bin(y, prob, plot=TRUE, scale="normal", line_args=list(), ...)
}
\arguments{
\item{y}{An observed outcome vector.}

\item{prob}{A vector of fitted probabilities of one.}

\item{plot}{A logical value indicating whether or not to return QQ-plot}

\item{scale}{You can choose the scale of the residuals among \code{normal} and \code{uniform}.
The sample quantiles of the residuals are plotted against
the theoretical quantiles of a standard normal distribution under the normal scale,
and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
The default scale is \code{normal}.}

\item{line_args}{A named list of graphical parameters passed to
\code{graphics::abline()} to modify the reference (red) 45° line
in the QQ plot. If left empty, a default red dashed line is drawn.}

\item{...}{Additional graphical arguments passed to
\code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
\code{col}, \code{cex}, \code{xlab}, \code{ylab}).}
}
\value{
DPIT residuals.
}
\description{
Computes DPIT residuals for regression models with binary outcomes
using the observed responses (\code{y}) and their fitted distributional parameters(\code{prob}).
}
\details{
For formulation details on discrete outcomes, see \code{\link{dpit_pois}}.
}
\examples{
## Binary example
n <- 500
set.seed(1234)
# Covariates
x1 <- rnorm(n, 1, 1)
x2 <- rbinom(n, 1, 0.7)
# Coefficients
beta0 <- -5
beta1 <- 2
beta2 <- 1
beta3 <- 3
q1 <- 1 / (1 + exp(beta0 + beta1 * x1 + beta2 * x2 + beta3 * x1 * x2))
y1 <- rbinom(n, size = 1, prob = 1 - q1)

# True model
model01 <- glm(y1 ~ x1 * x2, family = binomial(link = "logit"))
fitted1 <- fitted(model01)
y1 <- model01$y
resid.bin1 <- dpit_bin(y=y1, prob=fitted1)

# Missing covariates
model02 <- glm(y1 ~ x1, family = binomial(link = "logit"))
y2 <- model02$y
fitted2 <- fitted(model02)
resid.bin2 <- dpit_bin(y=y2, prob=fitted2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dpit_pois.R
\name{dpit_pois}
\alias{dpit_pois}
\title{Residuals for regression models with poisson outcomes}
\usage{
dpit_pois(y, mu, plot=TRUE, scale="normal", line_args=list(), ...)
}
\arguments{
\item{y}{An observed outcome vector.}

\item{mu}{A vector of fitted mean values.}

\item{plot}{A logical value indicating whether or not to return QQ-plot}

\item{scale}{You can choose the scale of the residuals among \code{normal} and \code{uniform}.
The sample quantiles of the residuals are plotted against
the theoretical quantiles of a standard normal distribution under the normal scale,
and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
The default scale is \code{normal}.}

\item{line_args}{A named list of graphical parameters passed to
\code{graphics::abline()} to modify the reference (red) 45° line
in the QQ plot. If left empty, a default red dashed line is drawn.}

\item{...}{Additional graphical arguments passed to
\code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
\code{col}, \code{cex}, \code{xlab}, \code{ylab}).}
}
\value{
DPIT residuals.
}
\description{
Computes DPIT residuals for Poisson outcomes regression using the observed counts (\code{y}) and their
corresponding fitted mean values (\code{mu}).
}
\details{
For formulation details on discrete outcomes, see \code{\link{dpit}}.
}
\examples{
## Poisson example
n <- 500
set.seed(1234)
# Covariates
x1 <- rnorm(n)
x2 <- rbinom(n, 1, 0.7)
# Coefficients
beta0 <- -2
beta1 <- 2
beta2 <- 1
lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
y <- rpois(n, lambda1)

# True model
poismodel <- glm(y ~ x1 + x2, family = poisson(link = "log"))
y1 <- poismodel$y
p1f <- fitted(poismodel)
resid.poi <- dpit_pois(y=y1, mu=p1f)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dpit_ordinal.R
\name{dpit_ordi}
\alias{dpit_ordi}
\title{Residuals for regression models with ordinal outcomes}
\usage{
dpit_ordi(y, level, fitprob, plot=TRUE, scale="normal", line_args=list(), ...)
}
\arguments{
\item{y}{An observed ordinal outcome vector.}

\item{level}{The names of the response levels. For instance, c(0,1,2).}

\item{fitprob}{A matrix of fitted category probabilities. Each row corresponds to an observation, and column j contains the fitted probability P(Y_i = j).}

\item{plot}{A logical value indicating whether or not to return QQ-plot}

\item{scale}{You can choose the scale of the residuals among \code{normal} and \code{uniform}.
The sample quantiles of the residuals are plotted against
the theoretical quantiles of a standard normal distribution under the normal scale,
and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
The default scale is \code{normal}.}

\item{line_args}{A named list of graphical parameters passed to
\code{graphics::abline()} to modify the reference (red) 45° line
in the QQ plot. If left empty, a default red dashed line is drawn.}

\item{...}{Additional graphical arguments passed to
\code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
\code{col}, \code{cex}, \code{xlab}, \code{ylab}).}
}
\value{
DPIT residuals.
}
\description{
Computes DPIT residuals for regression models with ordinal outcomes
using observed outcomes (\code{y}), ordinal outcome levels (\code{level}) and their fitted category
probabilities (\code{fitprob}).
}
\details{
For formulation details on discrete outcomes, see \code{\link{dpit}}.
}
\examples{
## Ordinal example
library(MASS)
n <- 500
x1 <- rnorm(n, mean = 2)
beta1 <- 3
# True model
p0 <- plogis(1, location = beta1 * x1)
p1 <- plogis(4, location = beta1 * x1) - p0
p2 <- 1 - p0 - p1
genemult <- function(p) {
 rmultinom(1, size = 1, prob = c(p[1], p[2], p[3]))
}
test <- apply(cbind(p0, p1, p2), 1, genemult)
y1 <- rep(0, n)
y1[which(test[1, ] == 1)] <- 0
y1[which(test[2, ] == 1)] <- 1
y1[which(test[3, ] == 1)] <- 2
multimodel <- polr(as.factor(y1) ~ x1, method = "logistic")

y1 <- multimodel$model[,1]
lev1 <- multimodel$lev
fitprob1 <- fitted(multimodel)

resid.ord <- dpit_ordi(y=y1, level=lev1, fitprob=fitprob1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dpit_znb.R
\name{dpit_znb}
\alias{dpit_znb}
\title{Residuals for regression models with zero-inflated negative binomial outcomes}
\usage{
dpit_znb(y, mu, pzero, size, plot=TRUE, scale="normal", line_args=list(), ...)
}
\arguments{
\item{y}{An observed outcome vector.}

\item{mu}{A vector of fitted mean values for the count (non-zero) component.}

\item{pzero}{A vector of fitted probabilities for the zero-inflation component.}

\item{size}{A dispersion parameter of the negative binomial distribution.}

\item{plot}{A logical value indicating whether or not to return QQ-plot}

\item{scale}{You can choose the scale of the residuals among \code{normal} and \code{uniform}.
The sample quantiles of the residuals are plotted against
the theoretical quantiles of a standard normal distribution under the normal scale,
and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
The default scale is \code{normal}.}

\item{line_args}{A named list of graphical parameters passed to
\code{graphics::abline()} to modify the reference (red) 45° line
in the QQ plot. If left empty, a default red dashed line is drawn.}

\item{...}{Additional graphical arguments passed to
\code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
\code{col}, \code{cex}, \code{xlab}, \code{ylab}).}
}
\value{
DPIT residuals.
}
\description{
Computes DPIT residuals for regression models with zero-inflated negative
binomial outcomes using the observed counts (\code{y}) and their fitted distributional
parameters (\code{mu}, \code{pzero}, \code{size}).
}
\details{
For formulation details on discrete outcomes, see \code{\link{dpit}}.
}
\examples{
## Zero-Inflated Negative Binomial
library(pscl)
n <- 500
set.seed(1234)

# Covariates
x1 <- rnorm(n)
x2 <- rbinom(n, 1, 0.7)

# Coefficients
beta0 <- -2
beta1 <-  2
beta2 <-  1
beta00 <- -2
beta10 <-  2

# NB dispersion (size = theta; larger => closer to Poisson)
theta_true <- 1.2

# Mean of NB count part
mu_true <- exp(beta0 + beta1 * x1 + beta2 * x2)

# Excess zero probability (logit)
p0 <- 1 / (1 + exp(-(beta00 + beta10 * x1)))

## simulate outcomes
z  <- rbinom(n, size = 1, prob = 1 - p0)              # 1 => from NB, 0 => structural zero
y1 <- rnbinom(n, size = theta_true, mu = mu_true)     # NB count draw
y  <- ifelse(z == 0, 0, y1)

## True model
modelzero1 <- zeroinfl(y ~ x1 + x2 | x1, dist = "negbin", link = "logit")
y1 <- modelzero1$y
mu1    <- stats::predict(modelzero1, type = "count")
pzero1 <- stats::predict(modelzero1, type = "zero")
theta1 <- modelzero1$theta
resid.zero1 <- dpit_znb(y = y1, pzero = pzero1, mu = mu1, size = theta1)

## Ignoring zero-inflation: NB only
modelzero2 <- MASS::glm.nb(y ~ x1 + x2)
y2 <- modelzero2$y
mu2    <- fitted(modelzero2)
theta2 <- modelzero2$theta
resid.zero2 <- dpit_nb(y = y2, mu = mu2, size = theta2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dpit_tweedie.R
\name{dpit_tweedie}
\alias{dpit_tweedie}
\title{Residuals for regression models with tweedie outcomes}
\usage{
dpit_tweedie(y, mu, xi, phi, plot=TRUE, scale="normal", line_args=list(), ...)
}
\arguments{
\item{y}{Observed outcome vector.}

\item{mu}{Vector of fitted mean values of each outcomes.}

\item{xi}{Value of \eqn{\xi} such that the variance is \eqn{Var[Y] = \phi\mu^\xi}}

\item{phi}{Dispersion parameter \eqn{\phi}.}

\item{plot}{A logical value indicating whether or not to return QQ-plot
The sample quantiles of the residuals are plotted against}

\item{scale}{You can choose the scale of the residuals among \code{normal} and \code{uniform}.
the theoretical quantiles of a standard normal distribution under the normal scale,
and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
The default scale is \code{normal}.}

\item{line_args}{A named list of graphical parameters passed to
\code{graphics::abline()} to modify the reference (red) 45° line
in the QQ plot. If left empty, a default red dashed line is drawn.}

\item{...}{Additional graphical arguments passed to
\code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
\code{col}, \code{cex}, \code{xlab}, \code{ylab}).}
}
\value{
DPIT residuals.
}
\description{
Computes DPIT residuals for Tweedie-distributed outcomes using the observed responses (\code{y}),
their fitted mean values (\code{mu}), the variance power parameter
(\eqn{\xi}), and the dispersion parameter (\eqn{\phi}).
}
\details{
For formulation details on semicontinuous outcomes, see \code{\link{dpit}}.
}
\examples{
## Tweedie model
library(tweedie)
library(statmod)
n <- 300
x11 <- rnorm(n)
x12 <- rnorm(n)
beta0 <- 5
beta1 <- 1
beta2 <- 1
lambda1 <- exp(beta0 + beta1 * x11 + beta2 * x12)
y1 <- rtweedie(n, mu = lambda1, xi = 1.6, phi = 10)
# Choose parameter p
# True model
model1 <-
  glm(y1 ~ x11 + x12,
    family = tweedie(var.power = 1.6, link.power = 0)
  )
y1 <- model1$y
p.max <- get("p", envir = environment(model1$family$variance))
lambda1f <- model1$fitted.values
phi1f <- summary(model1)$dis
resid.tweedie <- dpit_tweedie(y= y1, mu=lambda1f, xi=p.max, phi=phi1f)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dpit_nb.R
\name{dpit_nb}
\alias{dpit_nb}
\title{Residuals for regression models with negative binomial outcomes}
\usage{
dpit_nb(y, mu, size, plot=TRUE, scale="normal", line_args=list(), ...)
}
\arguments{
\item{y}{An observed outcome vector.}

\item{mu}{A vector of fitted mean values.}

\item{size}{A dispersion parameter of the negative binomial distribution.}

\item{plot}{A logical value indicating whether or not to return QQ-plot}

\item{scale}{You can choose the scale of the residuals among \code{normal} and \code{uniform}.
The sample quantiles of the residuals are plotted against
the theoretical quantiles of a standard normal distribution under the normal scale,
and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
The default scale is \code{normal}.}

\item{line_args}{A named list of graphical parameters passed to
\code{graphics::abline()} to modify the reference (red) 45° line
in the QQ plot. If left empty, a default red dashed line is drawn.}

\item{...}{Additional graphical arguments passed to
\code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
\code{col}, \code{cex}, \code{xlab}, \code{ylab}).}
}
\value{
DPIT residuals.
}
\description{
Computes DPIT residuals for regression models with negative binomial
outcomes using the observed counts (\code{y}) and their fitted distributional
parameters (\code{mu}, \code{size}).
}
\details{
For formulation details on discrete outcomes, see \code{\link{dpit}}.
}
\examples{
## Negative Binomial example
library(MASS)
n <- 500
x1 <- rnorm(n)
x2 <- rbinom(n, 1, 0.7)
### Parameters
beta0 <- -2
beta1 <- 2
beta2 <- 1
size1 <- 2
lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
# generate outcomes
y <- rnbinom(n, mu = lambda1, size = size1)

# True model
model1 <- glm.nb(y ~ x1 + x2)
y1 <- model1$y
fitted1 <- fitted(model1)
size1 <- model1$theta
resid.nb1 <- dpit_nb(y=y1, mu=fitted1, size=size1)

# Overdispersion
model2 <- glm(y ~ x1 + x2, family = poisson(link = "log"))
y2 <- model2$y
fitted2 <- fitted(model2)
resid.nb2 <- dpit_pois(y=y2, mu=fitted2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gof_main.R
\name{gof_disc}
\alias{gof_disc}
\title{Goodness-of-fit test for discrete outcome regression models}
\usage{
gof_disc(model, B=1e2, seed=NULL)
}
\arguments{
\item{model}{A fitted model object (e.g., from \code{glm()}, \code{polr()}, \code{glm.nb()} or \code{zeroinfl()}).}

\item{B}{Number of bootstrap samples. Default is 1e2.}

\item{seed}{random seed for bootstrap.}
}
\value{
Test statistics and p-values
}
\description{
Goodness-of-fit test for discrete-outcome regression models.
Works with GLMs (Poisson, binomial/logistic, negative binomial),
ordinal outcome regression (\code{MASS::polr}), and
zero-inflated regressions (zero-inflated Poisson and negative binomial via \code{pscl::zeroinfl()}).
}
\details{
Let \eqn{(Y_i,\mathbf{X}_i),\ i=1,\ldots,n} denote independent observations, and let
\eqn{\hat F_M(\cdot \mid \mathbf{X}_i)} be the fitted model-based CDF.
It was shown in \emph{Yang (2025)} that under the correctly specified model,
\deqn{\hat{H}(u) = \frac{1}{n}\sum_{i=1}^n \hat{h}(u, Y_i, \mathbf{X}_i)}
should be close to the identity function, where
\deqn{\hat{h}(u, y, \mathbf{x}) =
\frac{u - \hat{F}_M (y-1 \mid \mathbf{x})}
{\hat{F}_M (y \mid \mathbf{x}) - \hat{F}_M (y-1 \mid \mathbf{x})}
\,\mathbf{1}\{ \hat{F}_M (y-1 \mid \mathbf{x}) < u < \hat{F}_M (y \mid \mathbf{x}) \}
+ \mathbf{1}\{ u \ge \hat{F}_M (y \mid \mathbf{x}) \}.}
The test statistic
\deqn{S_n = \int_0^1 \{ \hat{H}(u) - u \}^2 du} measures the deviation of \eqn{\hat{h}(u,y,\mathbf{x})} from the
identity function, with p-values obtained by bootstrap. This method
complements residual-based diagnostics by providing a formal check of model adequacy.
}
\examples{
library(MASS)
library(pscl)
n <- 100
beta1 <- 1;  beta2 <- 1
beta0 <- -2; beta00 <- -2; beta10 <- 2
size1 <- 2
set.seed(1)
x1 <- rnorm(n)
x2 <- rbinom(n,1,0.7)
lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
p0 <- 1 / (1 + exp(-(beta00 + beta10 * x1)))
y0 <- rbinom(n, size = 1, prob = 1 - p0)
y1 <- rnegbin(n, mu=lambda1, theta=size1)
y <- ifelse(y0 == 0, 0, y1)
model1 <- zeroinfl(y ~ x1 + x2 | x1, dist = "negbin", link = "logit")
gof_disc(model1, B=50)
}
\references{
Yang L, Genest C, Neslehova J (2025). “A goodness-of-fit test for regression models with discrete outcomes.” Canadian Journal of Statistics
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dpit_tobit.R
\name{dpit_tobit}
\alias{dpit_tobit}
\title{Residuals for a tobit model}
\usage{
dpit_tobit(y, mu, sd, plot=TRUE, scale="normal", line_args=list(), ...)
}
\arguments{
\item{y}{An observed outcome vector.}

\item{mu}{A vector of fitted mean values of latent variables.}

\item{sd}{A standard deviation of latent variables.}

\item{plot}{A logical value indicating whether or not to return QQ-plot}

\item{scale}{You can choose the scale of the residuals among \code{normal} and \code{uniform}.
The sample quantiles of the residuals are plotted against
the theoretical quantiles of a standard normal distribution under the normal scale,
and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
The default scale is \code{normal}.}

\item{line_args}{A named list of graphical parameters passed to
\code{graphics::abline()} to modify the reference (red) 45° line
in the QQ plot. If left empty, a default red dashed line is drawn.}

\item{...}{Additional graphical arguments passed to
\code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
\code{col}, \code{cex}, \code{xlab}, \code{ylab}).}
}
\value{
DPIT residuals.
}
\description{
Computes DPIT residuals for tobit regression models using the observed
responses (\code{y}) and their corresponding fitted distributional parameters (\code{mu}, \code{sd}).
}
\details{
For formulation details on semicontinuous outcomes, see \code{\link{dpit}}.
}
\examples{
## Tobit regression model
library(VGAM)
n <- 500
beta13 <- 1
beta14 <- -3
beta15 <- 3

set.seed(1234)
x11 <- runif(n)
x12 <- runif(n)
lambda1 <- beta13 + beta14 * x11 + beta15 * x12
sd0 <- 0.3
yun <- rnorm(n, mean = lambda1, sd = sd0)
y <- ifelse(yun >= 0, yun, 0)

# Using VGAM package
# True model
fit1 <- vglm(formula = y ~ x11 + x12,
             tobit(Upper = Inf, Lower = 0, lmu = "identitylink"))
# Missing covariate
fit1miss <- vglm(formula = y ~ x11,
                 tobit(Upper = Inf, Lower = 0, lmu = "identitylink"))

resid.tobit1 <- dpit_tobit(y = y, mu = VGAM::fitted(fit1), sd = sd0)
resid.tobit2 <- dpit_tobit(y = y, mu = VGAM::fitted(fit1miss), sd = sd0)

# Using AER package
library(AER)
# True model
fit2 <- tobit(y ~ x11 + x12, left = 0, right = Inf, dist = "gaussian")
# Missing covariate
fit2miss <- tobit(y ~ x11, left = 0, right = Inf, dist = "gaussian")

resid.aer1 <- dpit_tobit(y = y, mu = fitted(fit2), sd = sd0)
resid.aer2 <- dpit_tobit(y = y, mu = fitted(fit2miss), sd = sd0)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quasi.R
\name{quasi_plot}
\alias{quasi_plot}
\title{Quasi empirical residuals functions}
\usage{
quasi_plot(model, line_args=list(), ...)
}
\arguments{
\item{model}{Model object (e.g., \code{glm}, \code{glm.nb}, \code{polr}, \code{zeroinfl})}

\item{line_args}{A named list of graphical parameters passed to
\code{graphics::abline()} to modify the reference (red) 45° line
in the QQ plot. If left empty, a default red dashed line is drawn.}

\item{...}{Additional graphical arguments passed to
\code{stats::qqplot()} for customizing the QQ plot (e.g., \code{lty},
\code{col}, \code{lwd}, \code{xlab}, \code{ylab}).}
}
\value{
A plot of quasi-empirial residual distribution function \eqn{\hat{U}(s;\beta)} against \eqn{s}.
}
\description{
Draw the quasi-empirical residual distribution functions for regression models with discrete outcomes.
Specifically, the model assumption of GLMs with binary, ordinal, Poisson, negative binomial,
zero-inflated Poisson, and zero-inflated negative binomial outcomes can be assessed using \code{quasi_plot()}.
A plot far apart from the diagonal indicates lack of fit.
}
\details{
The quasi-empirical residual distribution function is defined as follows:
\deqn{\hat{U}(s; \beta) = \sum_{i=1}^{n} W_{n}(s;\mathbf{X}_{i},\beta) 1[F(Y_{i}| X_{i}) < H(s;X_{i})]}
where
\deqn{W_n(s; \mathbf{X}_i, \beta) = \frac{K[(H(s; \mathbf{X}_i)-s)/ \epsilon_n]}{\sum_{j=1}^{n} K[(H(s; \mathbf{X}_j)-s)/ \epsilon_n]},}
\eqn{\epsilon_n} is the bandwidth; \eqn{H(s, X_i) = \mathrm{argmin}_{F(k \mid X_i)} |F(k \mid X_i) - s|} and \eqn{K} is a bounded, symmetric, and Lipschitz continuous kernel.
}
\examples{
## Negative Binomial example
library(MASS)
# Covariates
n <- 500
x1 <- rnorm(n)
x2 <- rbinom(n, 1, 0.7)
### Parameters
beta0 <- -2
beta1 <- 2
beta2 <- 1
size1 <- 2
lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
# generate outcomes
y <- rnbinom(n, mu = lambda1, size = size1)

# True model
model1 <- glm.nb(y ~ x1 + x2)
resid.nb1 <- quasi_plot(model1)

# Overdispersion
model2 <- glm(y ~ x1 + x2, family = poisson(link = "log"))
resid.nb2 <- quasi_plot(model2)

## Zero inflated Poisson example
library(pscl)
n <- 500
set.seed(1234)
# Covariates
x1 <- rnorm(n)
x2 <- rbinom(n, 1, 0.7)
# Coefficients
beta0 <- -2
beta1 <- 2
beta2 <- 1
beta00 <- -2
beta10 <- 2

# Mean of Poisson part
lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
# Excess zero probability
p0 <- 1 / (1 + exp(-(beta00 + beta10 * x1)))
## simulate outcomes
y0 <- rbinom(n, size = 1, prob = 1 - p0)
y1 <- rpois(n, lambda1)
y <- ifelse(y0 == 0, 0, y1)
## True model
modelzero1 <- zeroinfl(y ~ x1 + x2 | x1, dist = "poisson", link = "logit")
resid.zero1 <- quasi_plot(modelzero1)
}
\references{
Lu Yang (2021). Assessment of Regression Models with Discrete Outcomes Using Quasi-Empirical Residual Distribution Functions, Journal of Computational and Graphical Statistics, 30(4), 1019-1035.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dpit_zpois.R
\name{dpit_zpois}
\alias{dpit_zpois}
\title{Residuals for regression models with zero-inflated Poisson outcomes}
\usage{
dpit_zpois(y, mu, pzero, plot=TRUE, scale="normal", line_args=list(), ...)
}
\arguments{
\item{y}{An observed outcome vector.}

\item{mu}{A vector of fitted mean values for the count (non-zero) component.}

\item{pzero}{A vector of fitted probabilities for the zero-inflation component.}

\item{plot}{A logical value indicating whether or not to return QQ-plot}

\item{scale}{You can choose the scale of the residuals among \code{normal} and \code{uniform}.
The sample quantiles of the residuals are plotted against
the theoretical quantiles of a standard normal distribution under the normal scale,
and against the theoretical quantiles of a uniform (0,1) distribution under the uniform scale.
The default scale is \code{normal}.}

\item{line_args}{A named list of graphical parameters passed to
\code{graphics::abline()} to modify the reference (red) 45° line
in the QQ plot. If left empty, a default red dashed line is drawn.}

\item{...}{Additional graphical arguments passed to
\code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
\code{col}, \code{cex}, \code{xlab}, \code{ylab}).}
}
\value{
DPIT residuals.
}
\description{
Computes DPIT residuals for regression models with zero-inflated Poisson
outcomes using the observed counts(\code{y}) and their fitted distributional
parameters(\code{mu}, \code{pzero}).
}
\details{
For formulation details on discrete outcomes, see \code{\link{dpit}}.
}
\examples{
## Zero-Inflated Poisson
library(pscl)
n <- 500
set.seed(1234)
# Covariates
x1 <- rnorm(n)
x2 <- rbinom(n, 1, 0.7)
# Coefficients
beta0 <- -2
beta1 <- 2
beta2 <- 1
beta00 <- -2
beta10 <- 2

# Mean of Poisson part
lambda1 <- exp(beta0 + beta1 * x1 + beta2 * x2)
# Excess zero probability
p0 <- 1 / (1 + exp(-(beta00 + beta10 * x1)))
## simulate outcomes
y0 <- rbinom(n, size = 1, prob = 1 - p0)
y1 <- rpois(n, lambda1)
y <- ifelse(y0 == 0, 0, y1)
## True model
modelzero1 <- zeroinfl(y ~ x1 + x2 | x1, dist = "poisson", link = "logit")
y1 <- modelzero1$y
mu1    <- stats::predict(modelzero1, type = "count")
pzero1 <- stats::predict(modelzero1, type = "zero")
resid.zero1 <- dpit_zpois(y= y1, pzero=pzero1, mu=mu1)

## Zero inflation
modelzero2 <- glm(y ~ x1 + x2, family = poisson(link = "log"))
y2 <- modelzero2$y
mu2    <- fitted(modelzero2)
resid.zero2 <- dpit_pois(y= y2, mu=mu2)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dpit_2pm.R
\name{dpit_2pm}
\alias{dpit_2pm}
\title{Residuals for regression models with two-part outcomes}
\usage{
dpit_2pm(model0, model1, y, part0, part1, plot=TRUE, scale = "normal",
 line_args= list(), ...)
}
\arguments{
\item{model0}{Model object for 0 outcomes (e.g., logistic regression)}

\item{model1}{Model object for the continuous part (gamma regression)}

\item{y}{Semicontinuous outcomes.}

\item{part0}{Alternative argument to \code{model0}. One can supply the sequence of probabilities \eqn{P(Y_i=0),~i=1,\ldots,n}.}

\item{part1}{Alternative argument to \code{model1}. One can fit a regression model on the positive data and supply their probability integral transform. Note that the length of \code{part1} is the number of positive values in \code{y} and can be shorter than \code{part0}.}

\item{plot}{A logical value indicating whether or not to return QQ-plot}

\item{scale}{You can choose the scale of the residuals among \code{normal} and \code{uniform}. The default scale is \code{normal}.}

\item{line_args}{A named list of graphical parameters passed to
\code{graphics::abline()} to modify the reference (red) 45° line
in the QQ plot. If left empty, a default red dashed line is drawn.}

\item{...}{Additional graphical arguments passed to
\code{stats::qqplot()} for customizing the QQ plot (e.g., \code{pch},
\code{col}, \code{cex}, \code{xlab}, \code{ylab}).}
}
\value{
Residuals. If plot=TRUE, also produces a QQ plot.
}
\description{
Calculates DPIT residuals with model for semi-continuous outcomes.
\code{dpit_2pm} can be used either with \code{model0} and \code{model1} or with \code{part0} and \code{part1} as arguments.
}
\details{
For formulation details on semicontinuous outcomes, see \code{\link{dpit}}.

In two-part models, the probability of zero can be modeled using a logistic regression, \code{model0},
while the positive observations can be modeled using a gamma regression, \code{model1.}
Users can choose to use different models and supply the resulting probabilities of zero and probability integral transforms.
\code{part0} should be the sequence of fitted probabilities of zeros \eqn{\hat{p}_0(\mathbf{X}_i) ,~i=1,\ldots,n}.
\code{part1} should be the probability integral transform of the positive part \eqn{\hat{G}(Y_i|\mathbf{X}_i)}.
Note that the length of \code{part1} is the number of positive values in \code{y} and can be shorter than \code{part0}.
}
\examples{
library(MASS)
n <- 500
beta10 <- 1
beta11 <- -2
beta12 <- -1
beta13 <- -1
beta14 <- -1
beta15 <- -2
x11 <- rnorm(n)
x12 <- rbinom(n, size = 1, prob = 0.4)

p1 <- 1 / (1 + exp(-(beta10 + x11 * beta11 + x12 * beta12)))
lambda1 <- exp(beta13 + beta14 * x11 + beta15 * x12)
y2 <- rgamma(n, scale = lambda1 / 2, shape = 2)
y <- rep(0, n)
u <- runif(n, 0, 1)
ind1 <- which(u >= p1)
y[ind1] <- y2[ind1]

# models as input
mgamma <- glm(y[ind1] ~ x11[ind1] + x12[ind1], family = Gamma(link = "log"))
m10 <- glm(y == 0 ~ x12 + x11, family = binomial(link = "logit"))
resid.model <- dpit_2pm(model0 = m10, model1 = mgamma, y = y)

# PIT as input
cdfgamma <- pgamma(y[ind1],
  scale = mgamma$fitted.values * gamma.dispersion(mgamma),
  shape = 1 / gamma.dispersion(mgamma)
)
p1f <- m10$fitted.values
resid.pit <- dpit_2pm(y = y, part0 = p1f, part1 = cdfgamma)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bballHR.R
\docType{data}
\name{bballHR}
\alias{bballHR}
\title{MLB Players' Home Run and Batted Ball Statistics with Red Zone Metrics (2017-2019)}
\format{
A data frame with the following columns:
\describe{
\item{name}{Player's full name (character).}
\item{playerID}{Player's unique identifier in the Lahman database (character).}
\item{teamID}{Team abbreviation (character).}

\item{year}{Season year (numeric).}
\item{HR}{Home runs hit during the season (integer).}
\item{AB}{At-bats during the season (integer).}
\item{mean_exit_velo}{Average exit velocity (mph) over the season (numeric).}
\item{mean_launch_angle}{Average launch angle (degrees) over the season (numeric).}
\item{launch_angle_75}{Launch angle at the 75th percentile of the player's distribution (numeric).}
\item{launch_angle_70}{Launch angle at the 70th percentile of the player's distribution (numeric).}
\item{launch_angle_65}{Launch angle at the 65th percentile of the player's distribution (numeric).}
\item{exit_velo_75}{Exit velocity at the 75th percentile of the player's distribution (numeric).}
\item{exit_velo_80}{Exit velocity at the 80th percentile of the player's distribution (numeric).}
\item{exit_velo_85}{Exit velocity at the 85th percentile of the player's distribution (numeric).}
\item{count_red_zone}{Seasonal count of batted balls in the red zone, defined as a launch angle between 20 and 35 degrees and an exit velocity greater than or equal to 95 mph (integer).}
\item{prop_red_zone}{Proportion of batted balls that fall into the red zone (numeric).}
\item{BPF}{Ballpark factor, indicating the effect of the player's home ballpark on offensive statistics (integer).}
}
}
\source{
\itemize{
\item Player statistics: \href{https://CRAN.R-project.org/package=Lahman}{Lahman R Package}
\item Batted ball data: \href{https://baseballsavant.mlb.com/}{Baseball Savant}
\item Additional analysis: \href{https://bayesball.github.io/BLOG/homeruns.html}{Patterns of Home Run Hitting in the Statcast Era} by Jim Albert
}
}
\usage{
bballHR
}
\description{
This dataset provides annual statistics for Major League Baseball (MLB) players, including home run counts, at-bats, mean exit velocities, launch angles, quantile statistics of exit velocities and launch angles, and red zone metrics. It is intended for analyzing batted ball performance, with additional variables on the red zone, which is defined as balls in play with a launch angle between 20 and 35 degrees and an exit velocity of at least 95 mph.
}
\details{
\describe{
\item{Mean Metrics}{\code{mean_exit_velo} and \code{mean_launch_angle} represent the player's average exit velocities and launch angles, respectively, over the course of a season.}
\item{Quantile Metrics}{The \code{launch_angle_x} and \code{exit_velo_x} columns denote the upper \eqn{x}-percentiles (e.g., 75th percentile) of the player's launch angle and exit velocity distributions for that year.}
\item{Red Zone Metrics}{\code{count_red_zone} gives the number of balls in play that fall into the red zone, while \code{prop_red_zone} represents the proportion of balls in play in this category.}
\item{BPF}{The Ballpark Factor (BPF) quantifies the influence of the player's home ballpark on offensive performance, with values above 100 indicating a hitter-friendly environment.}
}
}
\examples{
data(bballHR)
head(bballHR)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MEPS.R
\docType{data}
\name{MEPS}
\alias{MEPS}
\title{Healthcare expenditure data}
\format{
A data frame with 29784 rows and 29 variables:
\describe{
\item{\code{EXP}}{the aggregate annual office based expenditure per participants, semicontinuous outcomes}
\item{\code{AGE}}{Age}
\item{\code{GENDER}}{1 if female}
\item{\code{ASIAN}}{1 if Asian}
\item{\code{BLACK}}{1 if Black}
\item{\code{NORTHEAST}}{1 if Northeast}
\item{\code{MIDWEST}}{1 if Midwest}
\item{\code{SOUTH}}{1 if South}
\item{\code{USC}}{1 if have usual source of care}
\item{\code{COLLEGE}}{1 if colleage or higher degrees}
\item{\code{HIGHSCH}}{1 if high school degree}
\item{\code{MARRIED}}{1 if married}
\item{\code{WIDIVSEP}}{1 if widowed or divorced or separated}
\item{\code{FAMSIZE}}{Family Size}
\item{\code{HINCOME}}{1 if high income}
\item{\code{MINCOME}}{1 if middle income}
\item{\code{LINCOME}}{1 if low income}
\item{\code{NPOOR}}{1 if near poor}
\item{\code{POOR}}{1 if poor}
\item{\code{FAIR}}{1 if fair}
\item{\code{GOOD}}{1 if good}
\item{\code{VGOOD}}{1 if very good}
\item{\code{MNHPOOR}}{1 if poor or fair mental health}
\item{\code{ANYLIMIT}}{1 if any functional or activity limitation}
\item{\code{unemployed}}{1 if unemployed at the beginning of 2006}
\item{\code{EDUCHEALTH}}{1 if education, health and social services}
\item{\code{PUBADMIN}}{1 if public administration}
\item{\code{insured}}{1 if is insured at the beginning of the year 2006}
\item{\code{MANAGEDCARE}}{if enrolled in an HMO or a gatekeeper plan}
}
}
\source{
http://www.meps.ahrq.gov/mepsweb/
}
\usage{
MEPS
}
\description{
Healthcare expenditure data set.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LGPIF.R
\docType{data}
\name{LGPIF}
\alias{LGPIF}
\title{LGPIF Data}
\format{
A data frame with 5677 rows and 41 variables:
\describe{
\item{\code{PolicyNum}}{Policy number}
\item{\code{Year}}{Policy year}
\item{\code{ClaimBC}}{Total building and contents (BC) claims in the year}
\item{\code{ClaimIM}}{Total inland marine (IM) claims in the year (contractor’s equipment)}
\item{\code{ClaimPN}}{Total comprehensive claims from new motor vehicles in the year}
\item{\code{ClaimPO}}{Total comprehensive claims from old motor vehicles in the year}
\item{\code{ClaimCN}}{Total collision claims from new vehicles in the year}
\item{\code{ClaimCO}}{Total collision claims from old vehicles in the year}
\item{\code{TypeCity}}{Indicator for city entity}
\item{\code{TypeCounty}}{Indicator for county entity}
\item{\code{TypeMisc}}{Indicator for miscellaneous entity}
\item{\code{TypeSchool}}{Indicator for school entity}
\item{\code{TypeTown}}{Indicator for town entity}
\item{\code{TypeVillage}}{Indicator for village entity}
\item{\code{IsRC}}{Indicator for replacement cost (motor vehicles)}
\item{\code{CoverageBC}}{Log coverage amount for building and contents (in millions of dollars)}
\item{\code{lnDeductBC}}{Log deductible amount for building and contents}
\item{\code{NoClaimCreditBC}}{Indicator for no BC claims in prior year}
\item{\code{yAvgBC}}{Average BC claim amount}
\item{\code{FreqBC}}{Frequency of BC claims}
\item{\code{CoverageIM}}{Log coverage amount for inland marine (in millions of dollars)}
\item{\code{lnDeductIM}}{Log deductible amount for inland marine}
\item{\code{NoClaimCreditIM}}{Indicator for no IM claims in prior year}
\item{\code{yAvgIM}}{Average IM claim amount}
\item{\code{FreqIM}}{Frequency of IM claims}
\item{\code{CoveragePN}}{Log coverage amount for comprehensive new vehicles (in millions of dollars)}
\item{\code{NoClaimCreditPN}}{Indicator for no PN claims in prior year}
\item{\code{yAvgPN}}{Average PN claim amount}
\item{\code{FreqPN}}{Frequency of PN claims}
\item{\code{CoveragePO}}{Log coverage amount for comprehensive old vehicles (in millions of dollars)}
\item{\code{NoClaimCreditPO}}{Indicator for no PO claims in prior year}
\item{\code{yAvgPO}}{Average PO claim amount}
\item{\code{FreqPO}}{Frequency of PO claims}
\item{\code{CoverageCN}}{Log coverage amount for collision of new vehicles (in millions of dollars)}
\item{\code{NoClaimCreditCN}}{Indicator for no CN claims in prior year}
\item{\code{yAvgCN}}{Average CN claim amount}
\item{\code{FreqCN}}{Frequency of CN claims}
\item{\code{CoverageCO}}{Log coverage amount for collision of old vehicles (in millions of dollars)}
\item{\code{NoClaimCreditCO}}{Indicator for no CO claims in prior year}
\item{\code{yAvgCO}}{Average CO claim amount}
\item{\code{FreqCO}}{Frequency of CO claims}
}
}
\source{
https://sites.google.com/a/wisc.edu/jed-frees/
}
\usage{
LGPIF
}
\description{
Data from the Wisconsin Local Government Property Insurance Fund (LGPIF).
The LGPIF was established to provide property insurance for local
government entities that include counties, cities, towns,
villages, school districts, fire departments, and other
miscellaneous entities, and is administered by the Wisconsin Office of the Insurance Commissioner. Properties covered under this fund include government buildings, vehicles, and equipment.
}
\references{
Frees, E. W., Lee, G., & Yang, L. (2016). Multivariate frequency-severity regression models in insurance. Risks, 4(1), 4.
}
\keyword{datasets}
